#include "advancedconfig.h"

AdvancedConfigWindow::AdvancedConfigWindow(ManageSerialPort *_serialPort,vector<Module> *_moduleList, QWidget *parent) :
    QWidget(parent)
{
    serialCom = _serialPort;
    moduleList=_moduleList;
    settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    if(settingFile->contains("WINDOW_POS/ADV_CONFIG_WIN_X")&&
            settingFile->contains("WINDOW_POS/ADV_CONFIG_WIN_Y"))
    {
        int maxX=QApplication::desktop()->width();
        int maxY=QApplication::desktop()->height();
        int posX=settingFile->value("WINDOW_POS/ADV_CONFIG_WIN_X").toInt();
        int posY=settingFile->value("WINDOW_POS/ADV_CONFIG_WIN_Y").toInt();
        if(((maxX-posX-100)>0)&&((maxY-posY-70)>0))this->move(posX,posY);
    }
    setWindowIcon(QIcon(QString(DEFAULT_DIR)+"icon/simCard.png"));
    setWindowTitle("Multivoies - Advanced Configuration");
    setWindowModality(Qt::ApplicationModal);
    setMinimumWidth(780);
    resize(minimumSize());
    fillWindow();
    uint8_t dongle_version=serialCom->get_dongle_version();
    if(dongle_version<100) write_firm_status->setText("<font color=""red"">No compatible dongle found</font>");
    else write_firm_status->setText("Dongle version : "+QString::number(dongle_version));


}

//***************************
// GUI
//***************************

void AdvancedConfigWindow::fillWindow()
{
    //read data :
    QGroupBox *readGroup = new QGroupBox("Read Data");
        // building frame section
        serialNbLine= new QLineEdit(this);
        if(settingFile->contains("ADV_CONFIG/SN_TO_READ"))
            serialNbLine->setText(settingFile->value("ADV_CONFIG/SN_TO_READ").toString());
        else serialNbLine->setText("01000000");  // default = everyone
            QLabel *serialNbLineLabel = new QLabel(this);
            serialNbLineLabel->setText("Serial Number :");
            QVBoxLayout *serialNbLayout = new QVBoxLayout;
            serialNbLayout->addWidget(serialNbLineLabel);
            serialNbLayout->addWidget(serialNbLine);
        addressLine= new QLineEdit(this);
        if(settingFile->contains("ADV_CONFIG/ADDRESS_TO_READ"))
                addressLine->setText(settingFile->value("ADV_CONFIG/ADDRESS_TO_READ").toString());
        else addressLine->setText("03000000");  // default = external memory
            QLabel *addressLineLabel = new QLabel(this);
            addressLineLabel->setText("Address :");
            QVBoxLayout *addressLayout = new QVBoxLayout;
            addressLayout->addWidget(addressLineLabel);
            addressLayout->addWidget(addressLine);
            nbDataBox = new QSpinBox(this);
            if(settingFile->contains("ADV_CONFIG/NB_VALUE_TO_READ"))
                nbDataBox->setValue(settingFile->value("ADV_CONFIG/NB_VALUE_TO_READ").toInt());
            else nbDataBox->setValue(8);
            nbDataBox->setMinimum(8);
            nbDataBox->setMaximum(64);   //limit nb of bytes
            nbDataBox->setSingleStep(8);
            QLabel *nbDataBoxLabel = new QLabel(this);
            nbDataBoxLabel->setText("Nr of Data :");
            QVBoxLayout *nbDataLayout = new QVBoxLayout;
            nbDataLayout->addWidget(nbDataBoxLabel);
            nbDataLayout->addWidget(nbDataBox);

        QPushButton *sendReadOrderButton = new QPushButton("Read Data");
        connect(sendReadOrderButton, SIGNAL(clicked()), SLOT(ReadAndShowResponse()));

        QHBoxLayout *layoutFabricTrame = new QHBoxLayout;
            layoutFabricTrame->addLayout(serialNbLayout);
            layoutFabricTrame->addLayout(addressLayout);
            layoutFabricTrame->addLayout(nbDataLayout);
            layoutFabricTrame->addWidget(sendReadOrderButton);

        QHBoxLayout *layoutReceivedRead = new QHBoxLayout;
            dataReadDisplay=new QPlainTextEdit();
            dataReadDisplay->setReadOnly(true);
            dataReadDisplay->setMinimumHeight(120);
        QLabel *dataReadLineLabel = new QLabel(this);
            dataReadLineLabel->setText("Data :");
            layoutReceivedRead->addWidget(dataReadLineLabel);
            layoutReceivedRead->addWidget(dataReadDisplay);
        QVBoxLayout *layoutReadData = new QVBoxLayout;
            layoutReadData->addLayout(layoutFabricTrame);
            layoutReadData->addLayout(layoutReceivedRead);

            readGroup->setLayout(layoutReadData);
    //end readGroup

    //section write
    QGroupBox *writeGroup = new QGroupBox("Write Data");
        QVBoxLayout *writeDataLayout = new QVBoxLayout;
            serialNbLineWrite= new QLineEdit(this);
            if(settingFile->contains("ADV_CONFIG/SN_TO_WRITE"))
                serialNbLineWrite->setText(settingFile->value("ADV_CONFIG/SN_TO_WRITE").toString());
            else serialNbLineWrite->setText("01000000");  // default = everyone
                QLabel *serialNbLineWriteLabel = new QLabel(this);
                serialNbLineWriteLabel->setText("Serial Number :");
                QVBoxLayout *serialNbLineWriteLayout = new QVBoxLayout;
                serialNbLineWriteLayout->addWidget(serialNbLineWriteLabel);
                serialNbLineWriteLayout->addWidget(serialNbLineWrite);

            addressLineWrite= new QLineEdit(this);
            if(settingFile->contains("ADV_CONFIG/ADDRESS_TO_WRITE"))
                    addressLineWrite->setText(settingFile->value("ADV_CONFIG/ADDRESS_TO_WRITE").toString());
            else addressLineWrite->setText("03000000");  // default = external memory
                QLabel *addressLineWriteLabel = new QLabel(this);
                addressLineWriteLabel->setText("Address :");
                QVBoxLayout *addressWriteLayout = new QVBoxLayout;
                addressWriteLayout->addWidget(addressLineWriteLabel);
                addressWriteLayout->addWidget(addressLineWrite);

            valueToWrite = new QSpinBox(this);
                valueToWrite->setValue(0);
                valueToWrite->setMaximum(255);
                valueToWrite->setMinimum(0);
                QLabel *valueToWriteLabel = new QLabel(this);
                if(settingFile->contains("ADV_CONFIG/VALUE_TO_WRITE"))
                    valueToWriteLabel->setText(settingFile->value("ADV_CONFIG/VALUE_TO_WRITE").toString());
                valueToWriteLabel->setText("Value to write :");
                QVBoxLayout *valueToWriteLayout = new QVBoxLayout;
                valueToWriteLayout->addWidget(valueToWriteLabel);
                valueToWriteLayout->addWidget(valueToWrite);

            QPushButton *writeButton = new QPushButton("Write Data");

            connect(writeButton, SIGNAL(clicked()), SLOT(WriteAndShowResponse()));
            select_function=new QComboBox(this);
            select_function->addItem("1 byte by 1");
            select_function->addItem("4 bytes by 4");
            QPushButton *writeFileButton = new QPushButton("Write File");
            connect(writeFileButton,SIGNAL(clicked()),SLOT(send_file_clicked()));
            QHBoxLayout *writeFrameLayout = new QHBoxLayout;
                writeFrameLayout->addLayout(serialNbLineWriteLayout);
                writeFrameLayout->addLayout(addressWriteLayout);
                writeFrameLayout->addLayout(valueToWriteLayout);
                writeFrameLayout->addWidget(writeButton);

        writeDataLayout->addLayout(writeFrameLayout);

            receivedWritingLabel = new QLabel(this);
                receivedWritingLabel->setText(" ");
        QHBoxLayout *writeFrameLayout2 = new QHBoxLayout;
        writeFrameLayout2->addWidget(receivedWritingLabel);
        writeFrameLayout2->addWidget(select_function);
        writeFrameLayout2->addWidget(writeFileButton);
        writeFrameLayout2->setAlignment(Qt::AlignRight);
        writeDataLayout->addLayout(writeFrameLayout2);


        QHBoxLayout *writeFrameLayout3 = new QHBoxLayout;
        QPushButton *writeparamButton = new QPushButton("Write Param File");
        connect(writeparamButton,SIGNAL(clicked()),SLOT(write_param_file()));
        writeFrameLayout2->addWidget(writeparamButton);
        writeFrameLayout2->setAlignment(Qt::AlignRight);
        writeDataLayout->addLayout(writeFrameLayout3);

         writeGroup->setLayout(writeDataLayout);
    //end writeGroup

    //section move Pointer
        QGroupBox *pointerGroup = new QGroupBox("Move Pointer");
            QVBoxLayout *pointlayout = new QVBoxLayout;
            QHBoxLayout *pointerLayout = new QHBoxLayout;
            pointerSlider = new QSlider(Qt::Horizontal);
            pointerSlider->setMinimum(0);   //%
            pointerSlider->setMaximum(100);
            pointerSlider->setDisabled(true);
            pointerLayout->addWidget(pointerSlider);
            pointerMoveButton = new QPushButton("Move");
            pointerMoveButton->setDisabled(true);
            connect(pointerMoveButton,SIGNAL(clicked()),this,SLOT(pointerButtonPushed()));
            pointerLayout->addWidget(pointerMoveButton);
            pointerPosition = new QLabel;
            pointerLayout->addWidget(pointerPosition);
            QHBoxLayout *pointervalLayout = new QHBoxLayout;
            QLabel *manualpointer = new QLabel("Pointer Value ");
            pointeurvalue =new QLineEdit(this);
            pointerMovemanButton = new QPushButton("Set Pointer Value");
            connect(pointerMovemanButton,SIGNAL(clicked()),this,SLOT(pointerButtonmanPushed()));
            pointervalLayout->addWidget(manualpointer);
            pointervalLayout->addWidget(pointeurvalue);
            pointervalLayout->addWidget(pointerMovemanButton);
            pointlayout->addLayout(pointerLayout);
            pointlayout->addLayout(pointervalLayout);


        pointerGroup->setLayout(pointlayout);
        //Memory config

        QGroupBox *memoryGroup = new QGroupBox("Memory configuration");
        QHBoxLayout *memoryLayout = new QHBoxLayout;
        QLabel *memoryLabel= new QLabel("Select memory size :");
        memoryconfig = new QComboBox();
        memoryconfig->addItem("Single memory");
        memoryconfig->addItem("Double memory");
        memoryconfig->setCurrentIndex(-1);
        memoryLayout->addWidget(memoryLabel);
        memoryLayout->addWidget(memoryconfig);
        connect(memoryconfig,SIGNAL(currentIndexChanged(int)),this,SLOT(setMemoryConfig(int)));
        memoryGroup->setLayout(memoryLayout);

    QGroupBox *writfirmwareGroup = new QGroupBox("Update Dongle Firmware");
    QVBoxLayout *writfirmVLayout0 = new QVBoxLayout;
    QHBoxLayout *writfirmHLayout0 = new QHBoxLayout;
    QPushButton *writfirm = new QPushButton("Upload file to dongle");
    connect(writfirm,SIGNAL(clicked()),this,SLOT(sendFirm()));
    QHBoxLayout *writfirmHLayout1 = new QHBoxLayout;
    write_firm_status = new QLabel(this);
    write_firm_status->setText(" ");
    writfirmHLayout0->addWidget(writfirm);
    writfirmHLayout1->addWidget(write_firm_status);
    writfirmVLayout0->addLayout(writfirmHLayout0);
    writfirmVLayout0->addLayout(writfirmHLayout1);
    writfirmwareGroup->setLayout(writfirmVLayout0);

    QGroupBox *writfirmwaremoduleGroup = new QGroupBox("Upload Module Firmware");
    QVBoxLayout *writfirmmoduleVLayout0 = new QVBoxLayout;
    QHBoxLayout *writfirmmoduleHLayout0 = new QHBoxLayout;
  /*  mode_firmware_module = new QComboBox(this);
    mode_firmware_module->addItem("Update module");
    mode_firmware_module->addItem("Upload firmware to new module");*/
    QLabel *serial_nb_firm=new QLabel("Serial Nr : ");
    serialNbupdate= new QLineEdit();
    if(settingFile->contains("ADV_CONFIG/MODULE_SN_TO_UPDATE"))
        serialNbupdate->setText(settingFile->value("ADV_CONFIG/MODULE_SN_TO_UPDATE").toString());
    else serialNbupdate->setText("00005010");  // default
  //  writfirmmoduleHLayout0->addWidget(mode_firmware_module);
    writfirmmoduleHLayout0->addWidget(serial_nb_firm);
    writfirmmoduleHLayout0->addWidget(serialNbupdate);
   // QHBoxLayout *writfirmmmoduleHLayout1 = new QHBoxLayout;
    select_address_firmware = new QComboBox(this);
    select_address_firmware->addItem("0x03180000");
    select_address_firmware->addItem("0x031C0000");

    QLabel *add_firm=new QLabel("Addresse : ");
    writfirmmoduleHLayout0->addWidget(add_firm);
    writfirmmoduleHLayout0->addWidget(select_address_firmware);
    //writfirmmmoduleHLayout1
    QHBoxLayout *writfirmmmoduleHLayout2 = new QHBoxLayout;
   // changeadd_firmmodule = new QPushButton("Change firmware address");
    writfirmmodule = new QPushButton("Upload file to module");
  //  writfirmmmoduleHLayout2->addWidget(changeadd_firmmodule);
    writfirmmmoduleHLayout2->addWidget(writfirmmodule);
    //connect(changeadd_firmmodule,SIGNAL(clicked()),this,SLOT(update_adress_firm()));
    connect(writfirmmodule,SIGNAL(clicked()),this,SLOT(update_module_firm()));
    QHBoxLayout *writfirmmmoduleHLayout3 = new QHBoxLayout;
    write_firm_module_status = new QLabel(this);
    write_firm_module_status->setText(" ");
    writfirmmmoduleHLayout3->addWidget(write_firm_module_status);
    writfirmmoduleVLayout0->addLayout(writfirmmoduleHLayout0);
   // writfirmmoduleVLayout0->addLayout(writfirmmmoduleHLayout1);
    writfirmmoduleVLayout0->addLayout(writfirmmmoduleHLayout2);
    writfirmmoduleVLayout0->addLayout(writfirmmmoduleHLayout3);
    writfirmwaremoduleGroup->setLayout(writfirmmoduleVLayout0);

    QGroupBox *testMemgrp = new QGroupBox("Memory test");
    QLabel *moduleListLabel= new QLabel("Modules : ");
    QHBoxLayout *testMemHLayout1 = new QHBoxLayout;
        QVBoxLayout *moduleListLayout = new QVBoxLayout;
            moduleListView = new QPlainTextEdit;
            moduleListView->setMaximumHeight(140);
            moduleListView->setReadOnly(true);

            QLabel *clabel= new QLabel("Concentrator");
            c_memtest= new QTextEdit();
            c_memtest->setReadOnly(true);

            moduleListLayout->addWidget(moduleListLabel);
            moduleListLayout->addWidget(moduleListView);
            moduleListLayout->addWidget(clabel);
            moduleListLayout->addWidget(c_memtest);
            testMemHLayout1->addLayout(moduleListLayout);

         QVBoxLayout *testMemVLayout2 = new QVBoxLayout();
         QPushButton *memButton = new QPushButton("Test modules");
             connect(memButton, SIGNAL(clicked()), SLOT(testMemM()));
             testMemVLayout2->addWidget(memButton);
             testMemHLayout1->addLayout(testMemVLayout2);

         QPushButton *memButton2 = new QPushButton("Test concentrator");
                 connect(memButton2, SIGNAL(clicked()), SLOT(testMemC()));
                 testMemVLayout2->addWidget(memButton2);
                 testMemHLayout1->addLayout(testMemVLayout2);




    testMemgrp->setLayout(testMemHLayout1);

    QGroupBox *wifigrp = new QGroupBox("Wifi firmware Update");
    QVBoxLayout *wifiHLayout1 = new QVBoxLayout;
    testfirm = new QPushButton("Update firmware");
    connect(testfirm, SIGNAL(clicked()), SLOT(update_wifi()));

    write_wifi_status = new QLabel;
    write_wifi_prog= new QProgressBar;
    write_wifi_prog->hide();
    wifiHLayout1->addWidget(testfirm);
    wifiHLayout1->addWidget(write_wifi_status);
    wifiHLayout1->addWidget(write_wifi_prog);
    wifigrp->setLayout(wifiHLayout1);
  /*   QHBoxLayout *testMemVLayout2 = new QHBoxLayout;
    QPushButton *memButton = new QPushButton("test mem");
    connect(memButton, SIGNAL(clicked()), SLOT(testMem()));
    writeFrameLayout->addWidget(memButton);*/

    updating_wifi=false;

    QGroupBox *testLora = new QGroupBox("Test Lora COM");
    QVBoxLayout *loraVLayout1 = new QVBoxLayout;
    QPushButton *testLoraBut = new QPushButton("Test COM");
    connect(testLoraBut, SIGNAL(clicked()), SLOT(test_lora_com()));

    testLoraStatus = new QLabel;

    loraVLayout1->addWidget(testLoraBut);
    loraVLayout1->addWidget(testLoraStatus);
    testLora->setLayout(loraVLayout1);

    //layout complet
    QVBoxLayout* layoutVTotal1 = new QVBoxLayout;
    QVBoxLayout* layoutVTotal2 = new QVBoxLayout;
    QHBoxLayout* layoutHTotal = new QHBoxLayout;

    layoutVTotal1->addWidget(readGroup);
    layoutVTotal1->addWidget(writeGroup);
    layoutVTotal1->addWidget(pointerGroup);
    layoutVTotal1->addWidget(memoryGroup);
    layoutVTotal2->addWidget(writfirmwareGroup);
    layoutVTotal2->addWidget(writfirmwaremoduleGroup);
    layoutVTotal2->addWidget(testMemgrp);
    layoutVTotal2->addWidget(wifigrp);
    layoutVTotal2->addWidget(testLora);
    layoutHTotal->addLayout(layoutVTotal1);
    layoutHTotal->addLayout(layoutVTotal2);
    setLayout(layoutHTotal);
    setWindowModality(Qt::ApplicationModal);
    bool ok=serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x01000010,1,3);
    if(ok)
    {
        if( ((serialCom->getReceivedData()[6]==0x01)||(serialCom->getReceivedData()[6]==0x02))&&(serialCom->getReceivedData()[7]!=0xFF)) {
            memoryconfig->setEnabled(true);
        }
        else
        {
            memoryconfig->setEnabled(false);
        }
        if(serialCom->getReceivedData()[6]==0x02 && serialCom->getReceivedData()[7]==0x01)
        {
            memoryconfig->setCurrentIndex(1);
        }
        else memoryconfig->setCurrentIndex(0);
    }
    else memoryconfig->setCurrentIndex(-1);

    updateModuleListFromCentral();

}

void AdvancedConfigWindow::test_lora_com()
{
    serialCom->writeByteDontCheckResponse(0x00FD0000,0x157,1);
    uint8_t ret_val=1;
    int nn=60;
    do
    {
        if(serialCom->readBytesAtWithRetry(0x00FD0000,0x157,1,3))
        {
            ret_val=serialCom->getReceivedData()[0];

            switch((nn%3))
            {
            case 0:
                    testLoraStatus->setText("Searching lora MultiCOM ...");
                    break;
            case 1:
                testLoraStatus->setText("Searching lora MultiCOM .. ");
                break;
            case 2:
                testLoraStatus->setText("Searching lora MultiCOM .  ");
                break;
            }
            if(serialCom->getReceivedData()[2]==128)
                 ret_val=1;
            waitFor(500);
            refresh_events();
        }
    }while(ret_val&&(--nn));

    int8_t snr_tx=0;
    int8_t snr_rx=0;
    if(!ret_val)
    {
        snr_tx=(int8_t)serialCom->getReceivedData()[3];
        snr_rx=(int8_t)serialCom->getReceivedData()[2];
    }
    QString freq="unknown";
    if(serialCom->readBytesAtWithRetry(0x00FD0000,0x01000090,2,3))
    {
        char freq_raw[10];
        for(int i=0;i<9;i++)
        {
            freq_raw[i]=(char)serialCom->getReceivedData()[i];
        }
        freq_raw[9]='\0';
        freq=QString::fromStdString(freq_raw)+" hz";
    }
    if(!ret_val)
    {
        testLoraStatus->setText("<font color=""green"">COM Lora OK, rx snr: "+
                                QString::number(snr_rx)+",  tx snr: "+
                                QString::number(snr_tx)+
                                "</font>     frequency : "+freq);
    }
    else testLoraStatus->setText("<font color=""red"">COM Lora timeout</font>     frequency : "+freq);
}

void AdvancedConfigWindow::updateModuleListFromCentral()
{
    //update the moduleList widget with module List from ManageModule
    moduleListView->clear();
    QString moduleListString;
    for(uint32 moduleIndex=0; moduleIndex<moduleList->size(); moduleIndex++)
    {
        moduleListString.append(QString::number(
                    moduleList->at(moduleIndex).serialNb,16).rightJustified(8,'0')+
                                " v"+QString::number(moduleList->at(moduleIndex).version));
        if(moduleIndex!=moduleList->size()-1) moduleListString.append("\n");
    }
    moduleListView->setPlainText(moduleListString);
    c_memtest->clear();
    c_memtest->setText(QString::number(serialCom->getCentralSN(),16).rightJustified(8,'0'));
}

void  AdvancedConfigWindow::testMemM()
{
    bool ok=true;
    moduleListView->clear();

    for (uint32 imod=0;imod<moduleList->size();imod++)
    {
        ok=true;
        uint32 serialNb=moduleList->at(imod).serialNb;
        switch(moduleList->at(imod).typeOfModule)
        {


        case wirelessModule:
        case wirelessModule2:
            serialCom->writeByteAtWithRetry(serialNb,0x03000000,5,4);
            serialCom->readBytesAtWithRetry(serialNb,0x03000000,1,4);
            if(serialCom->getReceivedData()[0]!=5)
                ok=false;
            qDebug()<<"test 0"<<ok;

            serialCom->writeByteAtWithRetry(serialNb,0x0300000c,8,4);
            serialCom->readBytesAtWithRetry(serialNb,0x03040000,1,4);
            if(serialCom->getReceivedData()[0]==8)
                ok=ok&true;
            else ok=false;
                qDebug()<<"test 1"<<ok;
            serialCom->writeByteAtWithRetry(serialNb,0x03000018,24,4);
            serialCom->readBytesAtWithRetry(serialNb,0x03080000,1,4);
            if(serialCom->getReceivedData()[0]==24)
                ok=ok&true;
            else ok=false;
            qDebug()<<"test 2"<<ok;
            serialCom->writeByteAtWithRetry(serialNb,0x03000024,19,4);
            serialCom->readBytesAtWithRetry(serialNb,0x030c0000,1,4);
            if(serialCom->getReceivedData()[0]==19)
                ok=ok&true;
            else ok=false;
            qDebug()<<"test 3"<<ok;
            break;

        case powerModule:
        case powerModule2:
        case temperatureModule:
        case pressureModule:
            qDebug()<<"write 0";
            serialCom->writeByteAtWithRetry(serialNb,0x03000000,5,4);
            qDebug()<<"read 0";
            if(moduleList->at(imod).typeOfModule==powerModule2)
                serialCom->readBytesAtWithRetry(serialNb,0x05000000,1,4);
            else serialCom->readBytesAtWithRetry(serialNb,0x03000000,1,4);
            if(serialCom->getReceivedData()[0]!=5)
                ok=false;
             qDebug()<<"write 1";
            serialCom->writeByteAtWithRetry(serialNb,0x03020000,8,4);
             qDebug()<<"read 1";
            if(moduleList->at(imod).typeOfModule==powerModule2)
                serialCom->readBytesAtWithRetry(serialNb,0x05020000,1,4);
            else serialCom->readBytesAtWithRetry(serialNb,0x03020000,1,4);

            if(serialCom->getReceivedData()[0]==8)
                ok=ok&true;
            else ok=false;
            if(moduleList->at(imod).memoryAvailable==2)
            {
                qDebug()<<"2 mem";
                qDebug()<<"write 2";
                serialCom->writeByteAtWithRetry(serialNb,0x03040000,25,4);
                 qDebug()<<"read 2";
                if(moduleList->at(imod).typeOfModule==powerModule2)
                    serialCom->readBytesAtWithRetry(serialNb,0x05040000,1,4);
                else serialCom->readBytesAtWithRetry(serialNb,0x03040000,1,4);
                if(serialCom->getReceivedData()[0]==25)
                    ok=ok&true;
                else ok=false;
                qDebug()<<"write 3";
                serialCom->writeByteAtWithRetry(serialNb,0x03060000,19,4);
                 qDebug()<<"read 3";
                if(moduleList->at(imod).typeOfModule==powerModule2)
                    serialCom->readBytesAtWithRetry(serialNb,0x05060000,1,4);
                else serialCom->readBytesAtWithRetry(serialNb,0x03060000,1,4);
                if(serialCom->getReceivedData()[0]==19)
                    ok=ok&true;
                else ok=false;

            }

            break;

            }

        if(!ok)
            moduleListView->appendHtml("<span style=\"color: red\">" +QString::number(
                        serialNb,16).rightJustified(8,'0')+"</span>\n");
        else  moduleListView->appendHtml("<span style=\"color: green\">" +QString::number(
                serialNb,16).rightJustified(8,'0')+"</span>\n");

        }

   // QMessageBox::warning(this,"test mem","Mem OK",QMessageBox::Ok,QMessageBox::Ok);
}

void  AdvancedConfigWindow::testMemC()
{
    bool ok=true;
    c_memtest->clear();
    uint32 serialNb=serialCom->getCentralSN();
    serialCom->writeByteAtWithRetry(serialNb,0x03000000,5,4);
    serialCom->readBytesAtWithRetry(serialNb,0x03000000,1,4);
    if(serialCom->getReceivedData()[0]!=5)
        ok=false;

    serialCom->writeByteAtWithRetry(serialNb,0x03020000,8,4);
    serialCom->readBytesAtWithRetry(serialNb,0x03020000,1,4);
    if(serialCom->getReceivedData()[0]==8)
        ok=ok&true;
    else ok=false;
    if(serialNb&0xf000)
    {
        qDebug()<<"2 mem";
        serialCom->writeByteAtWithRetry(serialNb,0x03040000,25,4);
        serialCom->readBytesAtWithRetry(serialNb,0x03040000,1,4);
        if(serialCom->getReceivedData()[0]==25)
            ok=ok&true;
        else ok=false;

        serialCom->writeByteAtWithRetry(serialNb,0x03060000,19,4);
        serialCom->readBytesAtWithRetry(serialNb,0x03060000,1,4);
        if(serialCom->getReceivedData()[0]==19)
            ok=ok&true;
        else ok=false;

    }
    if(ok)
        c_memtest->setText("<span style=\"color: green\">"+QString::number(serialCom->getCentralSN(),16).rightJustified(8,'0')+"</span>");
    else
        c_memtest->setText("<span style=\"color: red\">"+QString::number(serialCom->getCentralSN(),16).rightJustified(8,'0')+" error</span>");
}


void AdvancedConfigWindow::ReadAndShowResponse()
{   //Method to send a reading frame and display response
    //serialCom->deleteErrorCom();
    dataReadDisplay->clear();
    uint32 serialNb=serialNbLine->text().toInt(0,16);
    uint32 address=addressLine->text().toInt(0,16);
  //  serialCom->deleteErrorCom();
    bool ok=serialCom->readBytesAtWithRetry(serialNb, address, (nbDataBox->value()/8),3);
    if(ok)
    {
        QString valueToDisplay;
        for(int charIndex=0; charIndex<serialCom->getReceivedDataLength();charIndex++)
        {   //display in a bytes-separated format
            valueToDisplay+=QString::number(serialCom->getReceivedData()[charIndex],16).rightJustified(2,'0').toUpper();
            if((charIndex+1)%8==0 && charIndex!=(serialCom->getReceivedDataLength()-1)) valueToDisplay+='\r';
            else valueToDisplay+="  ";
        }
        dataReadDisplay->setPlainText(valueToDisplay);
    }
    else    //reading not ok
        dataReadDisplay->setPlainText("Reading Failed");
}
void AdvancedConfigWindow::send_file_clicked(){
    bool mode_envoi=select_function->currentIndex();
    WriteHexFile(serialNbLineWrite->text().toInt(0,16),addressLineWrite->text().toInt(0,16),mode_envoi);
}

void AdvancedConfigWindow::update_module_firm(){
    update_module_firmware(true);
}

void AdvancedConfigWindow::update_adress_firm(){
    update_module_firmware(false);
}

void AdvancedConfigWindow::update_module_firmware(bool modewrite){
    //changeadd_firmmodule->setEnabled(false);
    writfirmmodule->setEnabled(false);
    uint32_t serialNb=serialNbupdate->text().toInt(0,16);
    uint32_t address=0;
    uint8_t dataToWrite=0;
    if(serialNb!=0x5000) //si module deja programm, passage en mode bootloader
    {
        address=0x0C00000C;
        //serialCom->deleteErrorCom();
        serialCom->writeByteAtWithRetry(serialNb, address, dataToWrite,1);
        waitFor(500);
        bool ok=serialCom->readBytesAtWithRetry(0x5000,0,1,3);
        if(!ok)
        {
            QMessageBox::critical(this,"Error sending command to enter bootloader","Error sending command to enter bootloader");
            writfirmmodule->setEnabled(true);
            return;
        }
    }
    if(!select_address_firmware->currentIndex())address=0x03180000;
    else address=0x031C0000;
    if(!WriteHexFile(0x00005000,address,1)) {
        write_firm_module_status->setText("<font color=""red"">Uploading failed</font>");
        writfirmmodule->setEnabled(true);
        return;
    }
    if(!select_address_firmware->currentIndex())address=0x0C000001;
    else address=0x0C000002;
    serialCom->writeByteAtWithRetry(0x00005000,address,0,1); //sortie du bootloader
    waitFor(500);
 /*   if(mode_firmware_module->currentIndex()) //si nouveau module, on change le numro serie
    {
        //set new SN
        address=0x01000002;
        serialCom->writeByteAtWithRetry(0x00005001,address,hi(serialNb),3);
        serialCom->writeByteAtWithRetry(0x00005001,address+1,lo(serialNb),3);
        //waitFor(200);
        serialCom->writeByteAtWithRetry(0x00005001,0x0C000001,0,1); //reset module
    }*/


    write_firm_module_status->setText("<font color=""green"">Module successfully updated</font>");
    //changeadd_firmmodule->setEnabled(true);
    writfirmmodule->setEnabled(true);
}

bool AdvancedConfigWindow::WriteHexFile(uint32_t serialNb, uint32_t address,bool mode_envoi){
    //serialCom->deleteErrorCom();
    receivedWritingLabel->setText(" ");
    uchar dataToWrite=0;
    uchar data4ToWrite[4];
    bool ok=false;
    QString filename_tmp="";
    if(settingFile->contains("ADV_CONFIG/DIR_FILE_TO_WRITE")){
       filename_tmp=(settingFile->value("ADV_CONFIG/DIR_FILE_TO_WRITE").toString());
    }
    QString hex_filename=QFileDialog::getOpenFileName(this,"Choose Filename",filename_tmp, "*.*");
    if(hex_filename=="") return false;
    QFile  hex_file(hex_filename);
    if(!hex_file.open((QIODevice::ReadOnly ))){
        QMessageBox::critical(this,"Error Opening File","The file can't be opened");
        return false;
    }
    uint32_t inc=0;
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local
    settingFile->setValue("ADV_CONFIG/DIR_FILE_TO_WRITE",QVariant(hex_filename));

    QProgressDialog *downloadProgress = new QProgressDialog("Data transfer in progress...", "Cancel", 0, 100);
    downloadProgress->setWindowTitle("Writing File...");
    downloadProgress->setMinimum(0);    //%
    downloadProgress->setMaximum(100);
    downloadProgress->setWindowModality(Qt::ApplicationModal);
    downloadProgress->show();
    downloadProgress->setValue(0);
    uint32_t file_size=hex_file.size();
    if(!mode_envoi)
    {

        while (!hex_file.atEnd()) {
            QByteArray line = hex_file.readLine();
            for(int i=0;i<line.size();i++){
                dataToWrite=line.at(i);
                do
                {
                    int n_try=3;
                    do
                    {
                        ok=serialCom->writeByteAtWithRetry(serialNb, address, dataToWrite,5);
                        if (!ok){
                            waitFor(50);
                            //serialCom->deleteErrorCom();
                        }
                    }while(!ok&&n_try--);
                    //waitFor(500);
                    qDebug()<<serialNb<<address<<QString::number(dataToWrite,16).rightJustified(2,'0').toUpper();
                    if(!ok)
                    {
                        int response = QMessageBox::Retry;
                        while(!ok && response==QMessageBox::Retry)
                        {
                            // In case of reading error, retry with a message window
                            response=QMessageBox::warning(this,"Error during writing data","Writing at address 0x"+QString::number(address,16)+", to Module SN "+QString::number(serialNb,16).rightJustified(8,'0')+", failed.",QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
                            if(response==QMessageBox::Retry)
                            {
                                n_try=3;
                                do
                                {
                                    ok=serialCom->writeByteAtWithRetry(serialNb, address, dataToWrite,5);
                                    if (!ok){
                                        waitFor(50);
                                        //serialCom->deleteErrorCom();
                                    }
                                }while(!ok&&n_try--);
                            }
                            else
                            {
                                downloadProgress->close();
                                delete downloadProgress;
                                hex_file.close();
                                return false;
                            }
                        }
                    }
                    if(downloadProgress->wasCanceled())
                    {
                        downloadProgress->close();
                        delete downloadProgress;
                        hex_file.close();
                        return false;
                    }
                    QCoreApplication::processEvents();
                }while(!ok);
                address++;
                inc++;
                int percent=(100*inc)/file_size;
                downloadProgress->setValue(percent);
                //test+=QString::number(dataToWrite,16).rightJustified(2,'0').toUpper()+" ";
                QCoreApplication::processEvents();
            }
        }
    }
    else {
        QByteArray line;
        line.clear();
       while (!hex_file.atEnd()) {
            line.append(hex_file.readLine());
       }
       qDebug()<<"line size"<<line.size();
       int j=0;
       do
       {

           if((j+3)<line.size())
           {
               for(int i=0;i<4;i++) data4ToWrite[i]=line.at(j+i);
              /* qDebug()<<j<<serialNb<<address<<QString::number(data4ToWrite[0],16).rightJustified(2,'0').toUpper()
                       <<QString::number(data4ToWrite[1],16).rightJustified(2,'0').toUpper()
                       <<QString::number(data4ToWrite[2],16).rightJustified(2,'0').toUpper()
                       <<QString::number(data4ToWrite[3],16).rightJustified(2,'0').toUpper();*/
                  do
                    {
                        int n_try=3;
                        do
                        {
                            ok=serialCom->write4BytesAtWithRetry(serialNb, address, data4ToWrite,5);
                            if (!ok){
                                waitFor(50);
                                //serialCom->deleteErrorCom();
                            }
                        }while(!ok&&n_try--);
                        //qDebug()<<serialNb<<address<<QString::number(dataToWrite,16).rightJustified(2,'0').toUpper();
                        if(!ok)
                        {
                            int response = QMessageBox::Retry;
                            while(!ok && response==QMessageBox::Retry)
                            {
                                // In case of reading error, retry with a message window
                                response=QMessageBox::warning(this,"Error during writing data","Writing at address 0x"+QString::number(address,16)+", to Module SN "+QString::number(serialNb,16).rightJustified(8,'0')+", failed.",QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
                                if(response==QMessageBox::Retry)
                                {
                                    n_try=3;
                                    do
                                    {
                                        ok=serialCom->write4BytesAtWithRetry(serialNb, address, data4ToWrite,5);
                                        if (!ok){
                                            waitFor(50);
                                            //serialCom->deleteErrorCom();
                                        }
                                    }while(!ok&&n_try--);
                                }
                                else
                                {
                                    downloadProgress->close();
                                    delete downloadProgress;
                                    hex_file.close();
                                    return false;
                                }
                            }
                        }
                        if(downloadProgress->wasCanceled())
                        {
                            downloadProgress->close();
                            delete downloadProgress;
                            hex_file.close();
                            return false;
                        }
                        QCoreApplication::processEvents();
                    }while(!ok);
               j+=4;
               address+=4;
               inc+=4;
           }
           else
           {
               dataToWrite=line.at(j++);
               qDebug()<<serialNb<<address<<QString::number(dataToWrite,16).rightJustified(2,'0').toUpper();
                do
                    {
                        int n_try=3;
                        do
                        {
                            ok=serialCom->writeByteAtWithRetry(serialNb, address, dataToWrite,1);
                            if (!ok){
                                waitFor(50);
                                //serialCom->deleteErrorCom();
                            }
                        }while(!ok&&n_try--);

                        qDebug()<<serialNb<<address<<QString::number(dataToWrite,16).rightJustified(2,'0').toUpper();
                        if(!ok)
                        {
                            int response = QMessageBox::Retry;
                            while(!ok && response==QMessageBox::Retry)
                            {
                                // In case of reading error, retry with a message window
                                response=QMessageBox::warning(this,"Error during writing data","Writing at address 0x"+QString::number(address,16)+", to Module SN "+QString::number(serialNb,16).rightJustified(8,'0')+", failed.",QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
                                if(response==QMessageBox::Retry)
                                {
                                    n_try=3;
                                    do
                                    {
                                        ok=serialCom->writeByteAtWithRetry(serialNb, address, dataToWrite,1);
                                        if (!ok){
                                            waitFor(50);
                                            //serialCom->deleteErrorCom();
                                        }
                                    }while(!ok&&n_try--);
                                }
                                else
                                {
                                    downloadProgress->close();
                                    delete downloadProgress;
                                    hex_file.close();
                                    return false;
                                }
                            }
                        }
                        if(downloadProgress->wasCanceled())
                        {
                            downloadProgress->close();
                            delete downloadProgress;
                            hex_file.close();
                            return false;
                        }
                        QCoreApplication::processEvents();
                    }while(!ok);
               address++;
               inc++;
           }
           int percent=(100*inc)/file_size;
           downloadProgress->setValue(percent);
           QCoreApplication::processEvents();
       }while(j<line.size());
    }
    delete downloadProgress;
    hex_file.close();
    return true;
}

void AdvancedConfigWindow::WriteAndShowResponse()
{   //Method to send a writing frame and check response

    receivedWritingLabel->setText("[...]");
    uint32 serialNb=serialNbLineWrite->text().toInt(0,16);
    uint32 address=addressLineWrite->text().toInt(0,16);
    uchar datatowrite=uchar(valueToWrite->value());
    bool ok=writedata(serialNb,address,datatowrite);
    if(ok)receivedWritingLabel->setText("Writing Success");
    else receivedWritingLabel->setText("Writing Failed");
}

bool AdvancedConfigWindow::writedata(uint32_t serialNb,uint32_t address,uint8_t datatowrite){
    //serialCom->deleteErrorCom();
    bool ok=false;
    //int n_try=3;
    ok=serialCom->writeByteAtWithRetry(serialNb, address, datatowrite,3);
   /* do{
        waitFor(10);
        if(serialCom->writeByteAtWithRetry(serialNb, address, datatowrite,1)) break;
        rx_ok=serialCom->readBytesAtWithRetry(serialNb, address,1,3);
        qDebug()<<"rx ok : "<<rx_ok;
        qDebug()<<serialCom->getReceivedData()[0];
        if(rx_ok&&(serialCom->getReceivedData()[0]!=datatowrite)) rx_ok=false;
        if(!rx_ok&&!n_try)
        {
            int response=QMessageBox::warning(0,"Communication failure","Cannot reach the system and write data.",QMessageBox::Retry|QMessageBox::Cancel,QMessageBox::Retry);
            if(response==QMessageBox::Cancel)
            {
                return false;
            }
            else n_try=3;
        }
    }while(!rx_ok);*/
    if(!ok) return false;
    return true;
}

void AdvancedConfigWindow::pointerButtonmanPushed(){

    uint32_t newPointer=pointeurvalue->text().toInt(0,16);
    qDebug()<<"new pointer position = 0x"<<QString::number(newPointer,16);
    bool ok=true;
    // Send new pointer :
    //serialCom->deleteErrorCom();
    ok=ok&&serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000010,(newPointer & 0x0000FF),3);
    ok=ok&&serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000011,(newPointer & 0x00FF00)>>8,3);
    ok=ok&&serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000012,(newPointer & 0xFF0000)>>16,3);
    // Writing at 0x12 makes the concentrator store a first record. (pointer change immediately
    pointeurvalue->setText(QString::number(newPointer,16));
    if(!ok)
    {
        QMessageBox::information(this, "Unable to move pointer", "Pointer can't be moved");
        pointerMoveButton->setDisabled(true);
        pointerMoveButton->setText("Error");
        pointerSlider->setDisabled(true);
        refreshCursorPosition();
    }
    else //move ok
        pointerMovemanButton->setText("Pointer Moved !");
    refreshCursorPosition();
}

void AdvancedConfigWindow::pointerButtonPushed()
{   // Send the new pointer position to the central
    // compute new position value :

    uint32_t newPointer;
    if(endOfMemory==0x7E004) //double memoire
    {
        if(pointerSlider->value()<=50)
        {
        newPointer=(pointerSlider->value()*2*0x3FFF0)/1200;
        newPointer=newPointer*12;                       // modulo 12
        }
        else
        {
            newPointer=(pointerSlider->value()*0x7E004)/1200;
            newPointer=newPointer*12+0x04;                       // modulo 12 +0x04 (saut double memoire)
        }
    }
    else
    {
    newPointer=pointerSlider->value()*endOfMemory/1200;
    newPointer=newPointer*12;                       // modulo 12
    }



    pointeurvalue->setText(QString::number(newPointer,16));
    qDebug()<<"new pointer position = 0x"<<QString::number(newPointer,16);
    bool ok=true;
    // Send new pointer :
    //serialCom->deleteErrorCom();
    ok=ok&&serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000010,(newPointer & 0x0000FF),3);
    ok=ok&&serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000011,(newPointer & 0x00FF00)>>8,3);
    ok=ok&&serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000012,(newPointer & 0xFF0000)>>16,3);
    // Writing at 0x12 makes the concentrator store a first record. (pointer change immediately

    if(!ok)
    {
        QMessageBox::information(this, "Unable to move pointer", "Pointer can't be moved");
        pointerMoveButton->setDisabled(true);
        pointerMoveButton->setText("Error");
        pointerSlider->setDisabled(true);
        refreshCursorPosition();
    }
    else //move ok
        pointerMoveButton->setText("Moved !");
    refreshCursorPosition();
}

void AdvancedConfigWindow::pointerSliderMoved(int _newPosition)
{   // To update percentage displayed

    if(pointerPosition->isEnabled()) pointerPosition->setText(QString::number(_newPosition)+"%");
    uint32 Pointerpos=0;
    if(endOfMemory==0x7E004)
    {
        if(_newPosition<=50)
        {
        Pointerpos=(_newPosition*2*0x3FFF0)/1200;
        Pointerpos=Pointerpos*12;                       // modulo 12
        }
        else
        {
            Pointerpos=(_newPosition*0x7E004)/1200;
            Pointerpos=Pointerpos*12+0x04;                       // modulo 12 +0x04 (saut double memoire)
        }
    }
    else
    {
    Pointerpos=_newPosition*endOfMemory/1200;
    Pointerpos=Pointerpos*12;                       // modulo 12
    }
    pointeurvalue->setText(QString::number(Pointerpos,16));
}

void AdvancedConfigWindow::refreshCursorPosition()
{   //refresh the cursor position
    //serialCom->deleteErrorCom();
    bool ok=serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x01000010,1,3);

    if(!ok) pointerPosition->setText("<font color=""red"">Not found</font>");
    else
    {
        uint32 memoryPointer=serialCom->getReceivedData()[0]
                +uint32(serialCom->getReceivedData()[1]<<8)
                +uint32(serialCom->getReceivedData()[2]<<16);
        endOfMemory = 0x3C000;   // End of External EEPROM
        if(serialCom->getReceivedData()[6]==0x02 && serialCom->getReceivedData()[7]==0x01)
            endOfMemory=0x7E004; // End of External EEPROM Central 2 (double memory)
        int percent=(memoryPointer*100 /endOfMemory);
        pointerSlider->setValue(percent);
        pointerPosition->setText(QString::number(percent)+"%");
        pointeurvalue->setText(QString::number(memoryPointer,16));
        connect(pointerSlider, SIGNAL(sliderMoved(int)),this,SLOT(pointerSliderMoved(int)));
        pointerSlider->setEnabled(true);
        pointerMoveButton->setEnabled(true);
    }
}

void AdvancedConfigWindow::sendFirm()
{

    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local
    QString filename_tmp="";
    if(settingFile->contains("ADV_CONFIG/DIR_SREC_FILE")){
       filename_tmp=(settingFile->value("ADV_CONFIG/DIR_SREC_FILE").toString());
    }
    QString srec_filename=QFileDialog::getOpenFileName(this,"Choose Filename",filename_tmp, "*.srec");
    if(srec_filename=="") return;
    settingFile->setValue("ADV_CONFIG/DIR_SREC_FILE",QVariant(srec_filename));
    connect(serialCom,SIGNAL(update_program_status(QString)),this,SLOT(update_status_line(QString)));
    serialCom->program_dongle(srec_filename);
    serialCom->close();
    if(settingFile->contains("COM/COM_PORT")&&settingFile->contains("COM/COM_TYPE"))
    {
        QString portName=settingFile->value("COM/COM_PORT").toString();
        serialCom->open(portName,settingFile->value("COM/COM_TYPE").toInt());
    }
}

bool AdvancedConfigWindow::ecrit5(uint32_t serial, uint8_t value[]) //ecrit 5 data 4 dans add , et 1 dans value
{
        uint32_t add;
        lo(add)=value[3];
        hi(add)=value[2];
        hh(add)=value[1];
        hhh(add)=value[0];
        return serialCom->writeByteAtWithRetry(serial,add,value[4],2);
}

bool AdvancedConfigWindow::ecrit12(uint32_t serial, uint8_t value[]) //ecrit 5 data 4 dans add , et 1 dans value
{
        uint32_t add;
        lo(add)=value[3];
        hi(add)=value[2];
        hh(add)=value[1];
        hhh(add)=value[0];
        return serialCom->writeByteAtWithRetryS(serial,add,value[4],2);
}

bool AdvancedConfigWindow::litr(uint32_t serial) //attent 1;
{

        if(serialCom->readBytesAt(serial,0,1))
        {
            if(serialCom->getReceivedData()[0]==0)
                return true;
        }
        return false;
}

uint8_t AdvancedConfigWindow::calc_checksum(QByteArray in) //xor de toutes les data + EF
{
    uint8_t csum=0xEF;
    for(int i=0;i<in.length();i++)
    {
        csum=csum^(uint8_t)in.at(i);
    }
    return csum;
}

QByteArray AdvancedConfigWindow::replacechar(QByteArray in) //remplace C0 par DB DC et DB par DB DD
{
    QByteArray out;
    out.append(0xC0);
    for(int i=0;i<in.length();i++)
    {
        if((uint8_t)in.at(i)==0xC0)
        {
            out.append(0xDB);
            out.append(0xDC);
        }
        else if((uint8_t) in.at(i)==0xDB)
        {
            out.append(0xDB);
            out.append(0xDD);
        }
        else out.append(in.at(i));
    }
    out.append(0xC0);
    return out;
}

QByteArray AdvancedConfigWindow::make_set_data_frame(uint32_t file_length,uint16_t n_sector, uint32_t start_address)
{
    QByteArray ret;
    static const char fr0[] = {0x00,0x02,0x10,0x00,0x78,0x01,0x3A,0x00};
    static const char fr1[] = {0x00,0x00,0x00,0x04,0x00,0x00};
    ret.append(QByteArray::fromRawData(fr0, sizeof(fr0)));
    ret.append(lo(file_length));
    ret.append(hi(file_length));
    ret.append(hh(file_length));
    ret.append(hhh(file_length));
    ret.append(lo(n_sector));
    ret.append(hi(n_sector));
   // qDebug()<<"n_sector:"<<n_sector;
    ret.append(QByteArray::fromRawData(fr1, sizeof(fr1)));
    ret.append(lo(start_address));
    ret.append(hi(start_address));
    ret.append(hh(start_address));
    ret.append(hhh(start_address));
    //debugDisplayQByteArray(ret);
    return replacechar(ret);
}

QByteArray AdvancedConfigWindow::make_data_frame(QByteArray data, uint16_t sector)
{
    static const char fr0[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
    static const char fr1[] = {0x00,0x00,0x00};
    uint32_t length=data.length();
    QByteArray temp0;
    temp0.append(lo(length));
    temp0.append(hi(length));
    temp0.append(hh(length));
    temp0.append(hhh(length));
    temp0.append(lo(sector));
    temp0.append(hi(sector));
  //  qDebug()<<"sector:"<<sector;
    temp0.append(QByteArray::fromRawData(fr0, sizeof(fr0)));
    temp0.append(data);
    uint8_t cs = calc_checksum(data);
    QByteArray ret;
    ret.append((char)0x00);
    ret.append(0x03);
    length+=16;
    ret.append(lo(length));
    ret.append(hi(length));
    ret.append(cs);
    ret.append(QByteArray::fromRawData(fr1, sizeof(fr1)));
    ret.append(temp0);
    ret=replacechar(ret);
   // debugDisplayQByteArray(ret);
    return ret;
}

bool AdvancedConfigWindow::sendWifiFrame(QByteArray data)
{
    uint16_t frame_length=data.length();
    uint16_t p_frame=0;
    uint8_t n_try=6;
    uint8_t i_trame=0;
    bool ok=false;
    while(--n_try&&!ok)
    {
        if(abort_wifi)
        {
            return false;
        }
        uint8_t content[5]={0,0,hi(frame_length),lo(frame_length),0}; //trame indique taille
        ok=ecrit5(0xFE0011,content);
       // ok=true;
        p_frame=0;
        i_trame=0;
        if(ok)
        {

            while(ok&&(p_frame<data.length()))
            {
                QCoreApplication::processEvents();
                if(abort_wifi)
                {
                    return false;
                }
               for(int i=0;i<4;i++)//decoupage en bloc de 4
                {
                   if((p_frame<data.length()))
                    content[i]=data.at(p_frame++);
                   else content[i]=0;

                }
                content[4]=i_trame++; //numero de trame
                ok=ecrit12(0xFE0012,content);
               // ok=true;
            }
        }
        if(ok) //trame complete ok
        {
            int nn=100;
            //  ok=true;
            ok=false;
            do{ //on attent reponse 0
                if(abort_wifi)
                {
                    return false;
                }
                waitFor(50);
                ok=litr(0xFE0010);
               QCoreApplication::processEvents();

            }while(!ok&&--nn);

        }
    }
    return ok;
}

int AdvancedConfigWindow::sendFileWifi(uint8_t i_file, QDir folder)
{
    bool ok=false;
    QString filename;
    uint32_t start_add=0;
    QFile txtfile(folder.absolutePath()+"/add.txt");
    if(!txtfile.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        write_wifi_status->setText("<font color=""red"">Can't open file "+folder.absolutePath()+"/add.txt</font>");
        return 2;
    }

    unsigned long s_add[3];

    QTextStream in(&txtfile);
    QString line_txt = in.readLine();
    int i_line_txt=0;
    while (!line_txt.isNull()) {
        if(i_line_txt<3)
        {
            QStringList str_add=line_txt.split(";",QString::KeepEmptyParts);
            if(str_add.length()<2)
            {
                qDebug()<<str_add;
                write_wifi_status->setText("<font color=""red"">Error in file "+folder.absolutePath()+"/add.txt</font>");
                return 2;
            }
            else
            {
                s_add[i_line_txt++]=str_add.at(1).toLong(&ok,16);
                if(!ok)
                {
                    write_wifi_status->setText("<font color=""red"">Error in file "+folder.absolutePath()+"/add.txt</font>");
                    return 2;
                }
            }

        }

        line_txt = in.readLine();


    }

    switch(i_file)
    {
        case 0:
            filename=folder.absolutePath()+"/0x00000.bin";
            start_add=s_add[i_file];
            break;
        case 1:
            filename=folder.absolutePath()+"/0x10000.bin";
            start_add=s_add[i_file];
            break;
        case 2:
            filename=folder.absolutePath()+"/spiff_rom.bin";
            start_add=s_add[i_file];
            break;
        default:
            return 1;
    }
    qDebug()<<filename<<start_add;
    QFile wififile(filename);
    if(!wififile.open(QIODevice::ReadOnly))
    {
        write_wifi_status->setText("<font color=""red"">Can't open file "+filename+"</font>");
        return 2;
    }


    write_wifi_prog->setValue(0);
    write_wifi_prog->show();
    int i_line=0;
    uint32_t file_size=wififile.size();
    if(file_size%16>0)
    {

        file_size=file_size+(16-(file_size%16));
    }
    uint16_t sector=file_size/1024;
    if(file_size%1024)
        sector++;
    QByteArray data;
    data=make_set_data_frame(file_size,sector,start_add); //trame start_data (indique addresse dpart)
    if(!sendWifiFrame(data))
    {
        write_wifi_status->setText("<font color=""red"">Can't upload file "+filename+"</font>");
        return 3;
    }
    write_wifi_status->setText("Upload file "+QString::number(i_file+1)+"/3...");
   // debugDisplayQByteArray(data);
    while (!wififile.atEnd()) //lecture du fichier
    {

        QByteArray line = wififile.read(1024); //par bloc de max 1024
        int rest_line=0;
        if(line.size()%16>0)
            rest_line=16-(line.size()%16);
        for(int i=0;i<rest_line;i++)
        {
            line.append((char)0x0);
        }
        if((!i_line)&&(!i_file)) //overide param in first file
        {
            const char tabchar[] = {0x02,0x40};
            line.replace(2,2,tabchar,2);
        }
        data=make_data_frame(line,i_line++);
       // write_wifi_status->setText("Upload file "+QString::number(i_file+1)+"/3,   "+QString::number(float(i_line*100./sector))+"%...");
        write_wifi_prog->setValue((i_line*100./sector));
        if(!sendWifiFrame(data)) //envoie de la trame sur bus CAN
        {
            if(abort_wifi)
                write_wifi_status->setText("<font color=""red"">Update aborted</font>");
            else
                write_wifi_status->setText("<font color=""red"">Can't upload file "+filename+"</font>");
            return 4;
        }
    }

    return 0;
}

bool AdvancedConfigWindow::update_wifi()
{
    if(updating_wifi)
    {
        abort_wifi=true;
        return false;
    }
    updating_wifi=true;
    abort_wifi=false;
    testfirm->setText("Cancel");
    write_wifi_prog->setValue(0);

    QDir folderwifi;
    QString folderwifi_temp;
    if(settingFile->contains("ADV_CONFIG/DIR_WIFI_FILES"))
    {
       folderwifi_temp=(settingFile->value("ADV_CONFIG/DIR_WIFI_FILES").toString());
    }
    folderwifi.setPath(QFileDialog::getExistingDirectory(this,"Choose wifi firmware directory, must contain: 0x00000.bin, 0x10000.bin and spiff_rom.bin",folderwifi_temp));
     if(!QFile::exists(folderwifi.absolutePath()+"/0x00000.bin"))
     {
         testfirm->setText("Update firmware");
          write_wifi_status->setText("<font color=""red"">No 0x00000.bin in folder</font>");
          updating_wifi=false;
         return false;
     }
     if(!QFile::exists(folderwifi.absolutePath()+"/0x10000.bin"))
     {
         testfirm->setText("Update firmware");
          write_wifi_status->setText("<font color=""red"">No 0x10000.bin in folder</font>");
          updating_wifi=false;
         return false;
     }
     if(!QFile::exists(folderwifi.absolutePath()+"/spiff_rom.bin"))
     {
         testfirm->setText("Update firmware");
          write_wifi_status->setText("<font color=""red"">No spiff_rom.bin in folder</font>");
          updating_wifi=false;
         return false;
     }
     if(!QFile::exists(folderwifi.absolutePath()+"/add.txt"))
     {
         testfirm->setText("Update firmware");
          write_wifi_status->setText("<font color=""red"">No add.txt in folder</font>");
          updating_wifi=false;
         return false;
     }

     settingFile->setValue("ADV_CONFIG/DIR_WIFI_FILES",QVariant(folderwifi.absolutePath()));

    uint8_t nn=50;
    uint8_t ti=60; //delais entre chaque sync
    uint8_t to = 10;//timeout apres requete
    uint8_t value[5]={0xAA,0x02,nn,ti,to};

    ecrit5(0xFE0010,value); //entre bootloader
    write_wifi_status->setText("Enter bootloader mode...");

    bool ok=false;
    do{
        QCoreApplication::processEvents();
        if(abort_wifi)
        {
            testfirm->setText("Update firmware");
              write_wifi_status->setText("<font color=""red"">Update aborted</font>");
             updating_wifi=false;
            return false;
        }
        waitFor(65);
        ok=litr(0xFE0010);
      //  ok=true;
    }while(!ok&&--nn);

    if(!ok)
    {
        qDebug()<<"can't read 0xFE0010";
         write_wifi_status->setText("<font color=""red"">Error to enter in booloader mode, please retry</font>");
         {
             testfirm->setText("Update firmware");
              updating_wifi=false;
              return false;
         }
    }
    for(int i=0;i<3;i++)
    {
        int ret=sendFileWifi(i,folderwifi);
        qDebug()<<"return code : "<<ret;
        if(ret!=0)
        {
            testfirm->setText("Update firmware");
             updating_wifi=false;
             return false;
        }

    }
    write_wifi_status->setText("<font color=""green"">Firmware update : success</font>");
    uint8_t content[5]={0xAA,0x03,0,0,0}; //sortie bootloader
    updating_wifi=false;
    testfirm->setText("Update firmware");
    if(!ecrit5(0xFE0010,content));
        return false;
    return true;

}

void AdvancedConfigWindow::update_status_line(QString status_line){

    write_firm_status->setText(status_line);
}


void AdvancedConfigWindow::setMemoryConfig(int config)
{   //Change memory config
    uchar valueToChange=config+1;
    if((valueToChange==1)||(valueToChange==2))
    {
    bool ok=serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x01000016,1,3);
    if(ok&&(serialCom->getReceivedData()[0]!=valueToChange))
    {
        bool incompatible_module=false;
        if(valueToChange==2)
        {
            for(uint i=0;i<moduleList->size();i++)
            {
                qDebug()<<"memory available"<<moduleList->at(i).memoryAvailable;
                if(moduleList->at(i).memoryAvailable!=2)incompatible_module=true;
            }
        }
        if(incompatible_module){
            QMessageBox::warning(this,"Incompatible module","Some module associated this central are not compatible with double memory",QMessageBox::Cancel,QMessageBox::Cancel);
        }
        else
        {
        qDebug()<<"valueToChange"<<valueToChange;
        serialCom->writeByteAtWithRetry(serialCom->getCentralSN(),0x01000016,valueToChange,1);
        //writedata(serialCom->getCentralSN(),0x01000013,valueToChange);
        }
    }
    }
    waitFor(50);
    bool ok=serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x01000010,1,3);
    if(ok)
    {
        if(serialCom->getReceivedData()[6]==0x02 && serialCom->getReceivedData()[7]==0x01)
        {
            memoryconfig->setCurrentIndex(1);
        }
        else memoryconfig->setCurrentIndex(0);
    }
    else memoryconfig->setCurrentIndex(-1);
}

void AdvancedConfigWindow::write_param_file()
{
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local
    QString filename_tmp="";
    if(settingFile->contains("ADV_CONFIG/PARAM_FILE")){
       filename_tmp=(settingFile->value("ADV_CONFIG/PARAM_FILE").toString());
    }
    QString param_filename=QFileDialog::getOpenFileName(this,"Choose Filename",filename_tmp, "*.txt");
    if(param_filename=="") return;
    settingFile->setValue("ADV_CONFIG/DIR_SREC_FILE",QVariant(param_filename));

    QFile  param_file(param_filename);
    if(!param_file.open((QIODevice::ReadOnly  | QIODevice::Text)))
    {
        QMessageBox::critical(this,"Error Opening File","The file can't be opened");
        return;
    }
     QTextStream in(&param_file);
     QStringList line_list;
     QString line = in.readLine();
     bool ok;
     while (!line.isNull())
     {
         line_list=line.split(';');
      serialCom->writeByteAtWithRetry(line_list.at(0).toULong(&ok,16),line_list.at(1).toULong(&ok,16),line_list.at(2).toUShort(&ok,16),3);
         /*qDebug()<<QString::number(line_list.at(0).toULong(&ok,16),16)
                    <<QString::number(line_list.at(1).toULong(&ok,16),16)
                       <<QString::number(line_list.at(2).toUShort(&ok,16),16);*/
         line = in.readLine();
     }
}

void AdvancedConfigWindow::closeEvent(QCloseEvent *event)
 {  //event when window is closed
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

    settingFile->setValue("ADV_CONFIG/SN_TO_READ",QVariant( serialNbLine->text()));
    settingFile->setValue("ADV_CONFIG/ADDRESS_TO_READ",QVariant( addressLine->text()));
    settingFile->setValue("ADV_CONFIG/SN_TO_WRITE",QVariant( serialNbLineWrite->text()));
    settingFile->setValue("ADV_CONFIG/ADDRESS_TO_WRITE",QVariant( addressLineWrite->text()));
    settingFile->setValue("ADV_CONFIG/VALUE_TO_WRITE",QVariant( valueToWrite->text()));
    settingFile->setValue("ADV_CONFIG/NB_VALUE_TO_READ",QVariant(nbDataBox->value()));
    settingFile->setValue("ADV_CONFIG/MODULE_SN_TO_UPDATE",QVariant(serialNbupdate->text()));
    settingFile->setValue("WINDOW_POS/ADV_CONFIG_WIN_X",QVariant(QString::number(this->pos().x())));
    settingFile->setValue("WINDOW_POS/ADV_CONFIG_WIN_Y",QVariant(QString::number(this->pos().y())));
     setVisible(false);
     emit closingWindow();
 }
