#include "connectiondialog.h"

ConnectionDialog::ConnectionDialog(ManageSerialPort *_serialPort, QWidget *parent) : QWidget(parent)
{   //Constructor
    serialCom=_serialPort;
    settingFile = NULL;
    settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    if(settingFile->contains("WINDOW_POS/COM_CONFIG_WIN_X")&&
            settingFile->contains("WINDOW_POS/COM_CONFIG_WIN_Y"))
    {
        int maxX=QApplication::desktop()->width();
        int maxY=QApplication::desktop()->height();
        int posX=settingFile->value("WINDOW_POS/COM_CONFIG_WIN_X").toInt();
        int posY=settingFile->value("WINDOW_POS/COM_CONFIG_WIN_Y").toInt();
        if(((maxX-posX-100)>0)&&((maxY-posY-70)>0))this->move(posX,posY);
    }
    setWindowIcon(QIcon(QString(DEFAULT_DIR)+"icon/port.png"));
    setWindowTitle("Serial Port Configuration");
    setWindowModality(Qt::ApplicationModal);
    fillWindow();
}

ConnectionDialog::~ConnectionDialog()
{   //Destructor
    delete settingFile;
}

void ConnectionDialog::fillWindow()
{   //add GUI
   setMinimumWidth(350);
    verticalLayout = new QVBoxLayout;
        message = new QLabel;
        verticalLayout->addWidget(message);

    //list of ports +refresh button
        portLayout = new QHBoxLayout;
        QLabel *portLabel = new QLabel("Com Port :");
        portLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        portLayout->addWidget(portLabel);
        comPortComboBox = new QComboBox;
        comPortComboBox->setMinimumWidth(140);
        portLayout->addWidget(comPortComboBox);

        refreshListButton = new QPushButton("Refresh List");
        connect(refreshListButton,SIGNAL(clicked()),this,SLOT(refreshComList()));
        portLayout->addWidget(refreshListButton);
       // verticalLayout->addLayout(portLayout);

    //type of connection
     //   typeLayout = new QHBoxLayout;
        QLabel *typeLabel = new QLabel("Connection Type :");
        typeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
       // typeLayout->addWidget(typeLabel);
        portLayout->addWidget(typeLabel);

        ComTypeComboBox = new QComboBox;
        ComTypeComboBox->addItem("InfraRed or Serial");
        ComTypeComboBox->addItem("802.15.4 2.4Ghz");
        ComTypeComboBox->addItem("Bluetooth");
        ComTypeComboBox->addItem("Wired");
        ComTypeComboBox->addItem("802.15.4 868Mhz");
        connect(ComTypeComboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(selectionOnComList(int)));
     //   typeLayout->addWidget(ComTypeComboBox);
            portLayout->addWidget(ComTypeComboBox);
     //   verticalLayout->addLayout(typeLayout);

        //wireless adress
     //  wirelessLayout= new QHBoxLayout;
        wireless_label = new QLabel("Wireless Destination : ");
      //  wirelessLayout->addWidget(wireless_label);
         portLayout->addWidget(wireless_label);

        uint64_t currentDest;
        bool ok=true;
        if(settingFile->contains("COM/ADDRESS_API"))
        {
            QString addAPI=settingFile->value("COM/ADDRESS_API").toString();
            currentDest=addAPI.toULongLong(&ok,16);
        } else  currentDest=serialCom->address_API;



        uint64_t currentDestHexa=QString::number(currentDest,16).toULongLong();
        wireless_address = new QLineEdit(QString::number(currentDestHexa));
       // wirelessLayout->addWidget(wireless_address);
        portLayout->addWidget(wireless_address);
        wireless_label->setEnabled(false);
        wireless_address->setEnabled(false);
      //  verticalLayout->addLayout(wirelessLayout);

        channelLabel = new QLabel("Channel :");
         channelLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
         portLayout->addWidget(channelLabel);
         channelLabel->setEnabled(false);
         ChannelComboBox = new QComboBox;
         for(int i=11;i<27;i++)
         {
         ChannelComboBox->addItem(QString::number(i)+" (0x"+QString::number(i,16)+")");
         }
         ChannelComboBox->addItem("~");
         ChannelComboBox->setCurrentIndex(16);
         if(settingFile->contains("COM/CHANNEL"))
         {
             uint16_t chanst=settingFile->value("COM/CHANNEL").toString().toInt(&ok,10);
             if (chanst > 11)
             ChannelComboBox->setCurrentIndex(chanst-11);
         }


         portLayout->addWidget(ChannelComboBox);
         ChannelComboBox->setEnabled(false);
    //     connect(ChannelComboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(setchan()));

        //connect button
      //  buttonLayout = new QHBoxLayout;
        openConnectionButton = new QPushButton("Open Connection");
        openConnectionButton->setDefault(true);
        //buttonLayout->addWidget(openConnectionButton);
        portLayout->addWidget(openConnectionButton);
        connect(openConnectionButton, SIGNAL(clicked()), this, SLOT(tryToOpenConnection()));
        //verticalLayout->addLayout(buttonLayout);
        verticalLayout->addLayout(portLayout);


    setLayout(verticalLayout);
    setWindowModality(Qt::ApplicationModal);

    //check file to print last config : connection type
    if(settingFile->contains("COM/COM_TYPE"))
        ComTypeComboBox->setCurrentIndex(settingFile->value("COM/COM_TYPE").toInt());

    // use enumerator to update the port list
    refreshComList();
}

//***************
//  Buttons
//***************

void ConnectionDialog::setchan()
{
    uint8_t channel=ChannelComboBox->currentIndex()+11;
    qDebug()<<"channel: "<<channel;
    QByteArray _data_to_send;
    _data_to_send[0]=0x36;
    _data_to_send[1]=channel;
    bool ok=false;
    ok=serialCom->write_serial_data(_data_to_send,0);
    if(channel!=read_chan())
    {
        message->setText("<font color=""red"">Unable to set new channel.");
    }
    else  message->setText("<font color=""green"">Channel successfully changed.");
}

uint8_t ConnectionDialog::read_chan()
{
    return serialCom->read_chan();
}


void ConnectionDialog::tryToOpenConnection()
{
    message->setText("Trying to open connection...");
    bool ok=false;
    uint64_t current_dest = wireless_address->text().toULongLong(&ok,16);
    if(current_dest!=0)
    {
       bool ok=true;
        //change currentDest from hexadecimal :
        uint64_t currentDestHexa=QString::number(current_dest,16).toULongLong();

        QString newDestination=wireless_address->text();
        uint64_t newDestinationhex=newDestination.toULongLong(&ok,16);
        if(newDestinationhex!=currentDestHexa)
        {
            serialCom->address_API=newDestinationhex;

            settingFile->setValue("COM/ADDRESS_API",QVariant(newDestination));
            wireless_address->setText(newDestination);
        }
    }
    QString comPortValue;
    if(comPortComboBox->currentIndex()>=0)  //if not -1
        comPortValue= portList.at(comPortComboBox->currentIndex()).portName;
    int comTypeValue = ComTypeComboBox->currentIndex();

    //  open / re-open :
    if(serialCom==NULL)
    {
        serialCom=new ManageSerialPort();
        serialCom->open(comPortValue,comTypeValue);
    }
    else
    {
        if(serialCom->isOpen()) {
            serialCom->close(); //re-open
            qDebug()<<"serial com already open";
        }
        serialCom->open(comPortValue,comTypeValue);
    }
    int timeout_open_com=100;
    qDebug()<<"0";
    while(!serialCom->isOpen()&&--timeout_open_com)  serialCom->open(comPortValue,comTypeValue);
    qDebug()<<"1";
    if(!serialCom->isOpen())
    {
        message->setText("<font color=""red"">Unable to open connection.</font><br />Please Try Again");
    }
    else
    {

        if(comTypeValue==radio802/*&&first_connect*/)
        {
            if(!serialCom->test_xstick())
            {
                qDebug()<<"try 111111";
                serialCom->close();
                serialCom->setBaud111();
                waitFor(100);
                serialCom->open(comPortValue,settingFile->value("COM/COM_TYPE").toInt());
                waitFor(100);
                serialCom->test_xstick();
                serialCom->config_xstick();
                //serialCom->close();
            }
            else serialCom->config_xstick();
            //first_connect=false;
            serialCom->close();
            int timeout_open_com=100;
            qDebug()<<"2";
            while(!serialCom->isOpen()&&--timeout_open_com)  serialCom->open(comPortValue,comTypeValue);
            qDebug()<<"3";
            if(!serialCom->isOpen())
            {
                message->setText("<font color=""red"">Unable to open connection.</font><br />Please Try Again");
                return;
            }
            if(!serialCom->test_xstick())
            {
                int d_ver=0;
                int n_try=3;
                do
                {
                    d_ver=serialCom->get_dongle_version();
                }while(!d_ver&&--n_try);
                qDebug()<<"dongle_ver"<<d_ver;
                if(!d_ver)
                {
                    message->setText("<font color=""red"">Unable to detect zigbit dongle, Please try to unplug/plug dongle.</font><br />");
                    return;
                }
                setchan();
                qDebug()<<"4";
               /* cur_chan=re))ad_chan();
                qDebug()<<cur_chan;
                if(cur_chan)
                    ChannelComboBox->setCurrentIndex(cur_chan-11);*/
            }
        }

        QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local
        message->setText("");
        settingFile->setValue("COM/COM_PORT",QVariant(comPortValue));
        settingFile->setValue("COM/COM_TYPE",QVariant(QString::number(comTypeValue)));
        settingFile->setValue("COM/ADDRESS_API",QVariant(wireless_address->text()));
        settingFile->setValue("COM/CHANNEL",QVariant(QString::number(ChannelComboBox->currentIndex()+11)));
         emit configurationClosed(); // send signal for reconnection

      //  close();
    }
}

void ConnectionDialog::refreshComList()
{   //update the com port list, using enumerator
    portList.clear();
    comPortComboBox->clear();
    portList= QextSerialEnumerator::getPorts();

    //fill comboBox with enumerator
    for (int i = 0; i < portList.size(); i++)
    {
        comPortComboBox->addItem(portList.at(i).portName);    //TODO : to test under Linux
        qDebug()<<portList.at(i).friendName<<portList.at(i).enumName<<portList.at(i).portName<<portList.at(i).physName;
    }

    //select one if only one
    if(portList.size()>1)
        comPortComboBox->setCurrentIndex(-1);   //blank
    else
        comPortComboBox->setCurrentIndex(0);    //the only one

    //select default port, if exists
    if(settingFile->contains("COM/COM_PORT"))
    {
        QString portNameFromConfigFile=settingFile->value("COM/COM_PORT").toString();
        for(int i=0; i<portList.size(); i++)
        {

            if(portList[i].portName==portNameFromConfigFile)
            {
                comPortComboBox->setCurrentIndex(i);
                break;
            }
        }
    }
}

void ConnectionDialog::selectionOnComList(int _selection)
{   // show destination button if 802.15.4 selected
    if(_selection==1||_selection==4) //802.15.4
    {
        wireless_label->setEnabled(true);
        wireless_address->setEnabled(true);
        channelLabel->setEnabled(true);
        ChannelComboBox->setEnabled(true);
    }
    else
    {
        wireless_label->setEnabled(false);
        wireless_address->setEnabled(false);
        channelLabel->setEnabled(false);
        ChannelComboBox->setEnabled(false);
    }
}
/*

void ConnectionDialog::closeEvent(QCloseEvent *event)
 {  //envent when window is closed
    uint64_t currentDest=serialCom->address_API;
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

    if(currentDest!=0)
    {
       bool ok=true;
        //change currentDest from hexadecimal :
        uint64_t currentDestHexa=QString::number(currentDest,16).toULongLong();

        QString newDestination=wireless_address->text();
        uint64_t newDestinationhex=newDestination.toULongLong(&ok,16);
        if(newDestinationhex!=currentDestHexa)
        {
            serialCom->address_API=newDestinationhex;

            settingFile->setValue("COM/ADDRESS_API",QVariant(newDestination));
            wireless_address->setText(newDestination);
        }
    }
    if(!serialCom->isOpen())tryToOpenConnection();
    setVisible(false);

    settingFile->setValue("WINDOW_POS/COM_CONFIG_WIN_X",QVariant(QString::number(this->pos().x())));
    settingFile->setValue("WINDOW_POS/COM_CONFIG_WIN_Y",QVariant(QString::number(this->pos().y())));
    emit configurationClosed(); // send signal for reconnection
 }*/
