#include "ethernetconfig.h"

EthernetConfig::EthernetConfig(ManageSerialPort *_serialPort,QWidget *parent) :
    QWidget(parent)
{
    serialCom = _serialPort;
    setWindowTitle("Multivoies - Ethernet Module Configuration");
    settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    if(settingFile->contains("WINDOW_POS/ETH_WIN_X")&&
            settingFile->contains("WINDOW_POS/ETH_WIN_Y"))
    {
        int maxX=QApplication::desktop()->width();
        int maxY=QApplication::desktop()->height();
        int posX=settingFile->value("WINDOW_POS/ETH_WIN_X").toInt();
        int posY=settingFile->value("WINDOW_POS/ETH_WIN_Y").toInt();
        if(((maxX-posX-100)>0)&&((maxY-posY-70)>0))this->move(posX,posY);
    }
    fillWindow();
  //  resize(300,340);
    setWindowModality(Qt::ApplicationModal);
}

void EthernetConfig::fillWindow()
{   //set GUI
    horizontalConfigLayout = new QHBoxLayout;   //split window
        normalConfigLayout = new QVBoxLayout;   //Boutons de menu
            networkConfigButton = new QPushButton("Network config");
            connect(networkConfigButton, SIGNAL(clicked()),this,SLOT(networkConfig()));
            ftpSendNowButton = new QPushButton("FTP Send Now");
            connect(ftpSendNowButton, SIGNAL(clicked()),this,SLOT(ftpSendNow()));
            ftpServerConfigButton = new QPushButton("FTP Server config");
            connect(ftpServerConfigButton, SIGNAL(clicked()),this,SLOT(ftpServerConfig()));
            timeConfigButton = new QPushButton("Time config");
            connect(timeConfigButton, SIGNAL(clicked()),this,SLOT(timeConfig()));
            mvConfigButton = new QPushButton("MV Systems Config");
            connect(mvConfigButton, SIGNAL(clicked()),this,SLOT(mvConfig()));
            normalConfigLayout->addWidget(networkConfigButton);
            normalConfigLayout->addWidget(ftpSendNowButton);
            normalConfigLayout->addWidget(ftpServerConfigButton);
            normalConfigLayout->addWidget(timeConfigButton);
            normalConfigLayout->addWidget(mvConfigButton);
    horizontalConfigLayout->addLayout(normalConfigLayout);
    setLayout(horizontalConfigLayout);
    rightWidget = NULL;
}

QString EthernetConfig::ipToString(QByteArray IPBytes,uint32 offset)
{
    if (IPBytes.length()>=int(offset+4))
            return (QString::number(uchar(IPBytes.at(0+offset)))+'.'+QString::number(uchar(IPBytes.at(1+offset)))+'.'+
                    QString::number(uchar(IPBytes.at(2+offset)))+'.'+QString::number(uchar(IPBytes.at(3+offset))));
    else
            return ("Error");
}

QString EthernetConfig::macToString(QByteArray MACBytes)
{
    if (MACBytes.length()>=6)
            return (QString::number(uchar(MACBytes.at(0)),16)+'.'+QString::number(uchar(MACBytes.at(1)),16)+'.'+QString::number(uchar(MACBytes.at(2)),16)+'.'+
                    QString::number(uchar(MACBytes.at(3)),16)+'.'+QString::number(uchar(MACBytes.at(4)),16)+'.'+QString::number(uchar(MACBytes.at(5)),16));
    else
            return ("Error");
}

QByteArray EthernetConfig::stringToIp(QString IPString)
{
    QStringList ipStringList = IPString.split(".");
    QByteArray ipQByteArray;
    for (uint8_t i=0;i<4;i++)
        ipQByteArray.push_back(ipStringList.at(i).toInt());
    return(ipQByteArray);
}

QByteArray EthernetConfig::stringToQBA(QString textString,uint16 nchar)
{
    QByteArray textQByteArray;
    for (uint8_t i=0;i<nchar;i++)
        if (i<textString.length())
            textQByteArray.append(textString.at(i));
        else
            textQByteArray.push_back(char(0));
    return(textQByteArray);
}

uint32 QBA2int(QByteArray * _inputQBA)
{
    uint32 outputUint32=0;
    for(int i=0;i<4;i++)
    {
        outputUint32+=(uchar(_inputQBA->at(i)))<<(i*8);
    }
    return outputUint32;
}

void EthernetConfig::closeQuit()
{
    if (rightWidget!=NULL)
    {
        horizontalConfigLayout->removeWidget(rightWidget);
        delete rightWidget;
        rightWidget = NULL;
    }
    this->close();
}
	
void EthernetConfig::networkConfig()
{
    if (rightWidget!=NULL)
    {
        horizontalConfigLayout->removeWidget(rightWidget);
        delete rightWidget;
    }
    rightWidget = new QWidget;
    rightlayout = new QGridLayout;
        QLabel *lab1 = new QLabel("Current IP:");
        lab1->setAlignment(Qt::AlignRight);
        rightlayout->addWidget(lab1,0,0);
        QLabel *lab2 = new QLabel("Mask:");
        lab2->setAlignment(Qt::AlignRight);
        rightlayout->addWidget(lab2,1,0);
        QLabel *lab3 = new QLabel("Gateway:");
        lab3->setAlignment(Qt::AlignRight);
        rightlayout->addWidget(lab3,2,0);
        QLabel *lab4 = new QLabel("DNS:");
        lab4->setAlignment(Qt::AlignRight);
        rightlayout->addWidget(lab4,3,0);
        QLabel *lab5 = new QLabel("Default IP:");
        lab5->setAlignment(Qt::AlignRight);
        rightlayout->addWidget(lab5,4,0);
        QLabel *lab6 = new QLabel("Default Mask:");
        lab6->setAlignment(Qt::AlignRight);
        rightlayout->addWidget(lab6,5,0);
        QLabel *lab7 = new QLabel("MAC addr:");
        lab7->setAlignment(Qt::AlignRight);
        rightlayout->addWidget(lab7,6,0);
        checkDHCP = new QCheckBox("DHCP Enabled");
        rightlayout->addWidget(checkDHCP,7,1);
		
        currentIPValue = new QLineEdit();
        rightlayout->addWidget(currentIPValue,0,1);
        maskValue = new QLineEdit();
        rightlayout->addWidget(maskValue,1,1);
        gatewayValue = new QLineEdit();
        rightlayout->addWidget(gatewayValue,2,1);
        dnsValue = new QLineEdit();
        rightlayout->addWidget(dnsValue,3,1);
        defaultIPValue = new QLineEdit();
        rightlayout->addWidget(defaultIPValue,4,1);
        defaultMaskValue = new QLineEdit();
        rightlayout->addWidget(defaultMaskValue,5,1);
        macAddrValue = new QLineEdit();
        rightlayout->addWidget(macAddrValue,6,1);
		
        networkConfigSaveButton = new QPushButton("Save");
        rightlayout->addWidget(networkConfigSaveButton,8,0);
        connect(networkConfigSaveButton, SIGNAL(clicked()),this,SLOT(networkConfigSave()));
        connect(networkConfigSaveButton, SIGNAL(clicked()),this,SLOT(networkUpdate()));
        networkConfigQuitButton = new QPushButton("Quit");
        rightlayout->addWidget(networkConfigQuitButton,8,1);
        connect(networkConfigQuitButton, SIGNAL(clicked()),this,SLOT(networkConfigQuit()));
    rightWidget->setLayout(rightlayout);
    horizontalConfigLayout->addWidget(rightWidget);
    networkUpdate();
}


void EthernetConfig::networkUpdate()
{
        // Remplit les valeurs
    QByteArray receivedCharArray=serialCom->ReadEthernetModuleRam(0x4C0,28);
    if (receivedCharArray.length()>0)
    {
        currentIPValue->setText (ipToString(receivedCharArray,0));
        maskValue->setText (ipToString(receivedCharArray,4));
        gatewayValue->setText (ipToString(receivedCharArray,8));
        dnsValue->setText (ipToString(receivedCharArray,12));
        defaultIPValue->setText (ipToString(receivedCharArray,20));
        defaultMaskValue->setText (ipToString(receivedCharArray,24));
    }

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x4DC,6);
    if (receivedCharArray.length()>0)
    {
        macAddrValue->setText (macToString(receivedCharArray));
    }

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x4F2,1);
    if (receivedCharArray.length()>0)
    {
        if(!(receivedCharArray[0] & 0x40))   // Bit6
            checkDHCP->setChecked(false);
        else
            checkDHCP->setChecked(true);
    }
    DHCPmem = receivedCharArray[0];
}

void  EthernetConfig::ftpServerConfig()
{
    if (rightWidget!=NULL)
    {
        horizontalConfigLayout->removeWidget(rightWidget);
        delete rightWidget;
    }
    rightWidget = new QWidget;
    rightlayout = new QGridLayout;
        QLabel *lab1 = new QLabel("FTP Server Name:");
        rightlayout->addWidget(lab1,0,0);
        QLabel *lab2 = new QLabel("User Name:");
        rightlayout->addWidget(lab2,1,0);
        QLabel *lab3 = new QLabel("Password:");
        rightlayout->addWidget(lab3,2,0);
        QLabel *lab4 = new QLabel("FTP Directory");
        rightlayout->addWidget(lab4,3,0);
        QLabel *lab5 = new QLabel("Last FTP Result:");
        rightlayout->addWidget(lab5,4,0);
        QLabel *lab6 = new QLabel("FTP Last Error:");
        rightlayout->addWidget(lab6,5,0);
		
        ftpServerNameValue = new QLineEdit();
        rightlayout->addWidget(ftpServerNameValue,0,1);
        ftpUserNameValue = new QLineEdit();
        ftpUserNameValue->setMinimumWidth(180);
        rightlayout->addWidget(ftpUserNameValue,1,1);
        ftpPasswordValue = new QLineEdit();
        rightlayout->addWidget(ftpPasswordValue,2,1);
        ftpDirectoryValue = new QLineEdit();
        rightlayout->addWidget(ftpDirectoryValue,3,1);
        ftpLastResultValue = new QLineEdit();
        rightlayout->addWidget(ftpLastResultValue,4,1);
        ftpLastErrorValue = new QLineEdit();
        rightlayout->addWidget(ftpLastErrorValue,5,1);
		
        ftpConfigSaveButton = new QPushButton("Save");
        rightlayout->addWidget(ftpConfigSaveButton,8,0);
        connect(ftpConfigSaveButton, SIGNAL(clicked()),this,SLOT(ftpConfigSave()));
        connect(ftpConfigSaveButton, SIGNAL(clicked()),this,SLOT(ftpServerUpdate()));

        ftpConfigQuitButton = new QPushButton("Quit");
        rightlayout->addWidget(ftpConfigQuitButton,8,1);
        connect(ftpConfigQuitButton, SIGNAL(clicked()),this,SLOT(ftpConfigQuit()));
    rightWidget->setLayout(rightlayout);
    horizontalConfigLayout->addWidget(rightWidget);
    ftpServerUpdate();
}

void  EthernetConfig::ftpServerUpdate()
{
    // Remplit les valeurs
    QByteArray receivedCharArray=serialCom->ReadEthernetModuleRam(0x347,31);
    if (receivedCharArray.length()>0)   ftpServerNameValue->setText (QString(receivedCharArray));

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x367,31);
    if (receivedCharArray.length()>0)   ftpUserNameValue->setText (QString(receivedCharArray));

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x387,16);
    if (receivedCharArray.length()>0)
    {
        QString pass = QString(receivedCharArray);
        for (int i=3;i<pass.length();i++) pass[i]='*';
        ftpPasswordValue->setText (pass);
    }

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x3A7,8);
    if (receivedCharArray.length()>0)   ftpDirectoryValue->setText(QString(receivedCharArray));

    receivedCharArray=serialCom->ReadEthernetModuleRam(0xE1B,32);
    if (receivedCharArray.length()>0)   ftpLastResultValue->setText(QString(receivedCharArray));

    receivedCharArray=serialCom->ReadEthernetModuleRam(0xAA2,1);
    if (receivedCharArray.length()>0)   ftpLastErrorValue->setText(QString(receivedCharArray));
}

void EthernetConfig::networkConfigSave()
{
    bool needToSave = false;
    if(defaultIPValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x4D4,stringToIp(defaultIPValue->text()));
        needToSave = true;
    }
    if(defaultMaskValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x4D8,stringToIp(defaultMaskValue->text()));
        needToSave = true;
    }
    if(gatewayValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x4C8,stringToIp(gatewayValue->text()));
        needToSave = true;
    }
    if(checkDHCP->isChecked() && ((DHCPmem & 0x40)==0))
    {
        QByteArray configQByteArray;
        configQByteArray.push_back(DHCPmem | 0x40);
        serialCom->WriteEthernetModuleRam(0x4F2,configQByteArray);
        needToSave = true;
    }
    if(!checkDHCP->isChecked() && ((DHCPmem & 0x40)!=0))
    {
        QByteArray configQByteArray;
        configQByteArray.push_back(DHCPmem & 0xBF);
        serialCom->WriteEthernetModuleRam(0x4F2,configQByteArray);
        needToSave = true;
    }
    if (needToSave)
    {
        QByteArray configQByteArray;
        serialCom->WriteEthernetModuleSpecial(0x21,configQByteArray);
    }
}

void EthernetConfig::ftpConfigSave()
{
    bool needToSave = false;
    if(ftpServerNameValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x347,stringToQBA(ftpServerNameValue->text(),32));
        needToSave = true;
    }
    if(ftpUserNameValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x367,stringToQBA(ftpUserNameValue->text(),32));
        needToSave = true;
    }
    if(ftpPasswordValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x387,stringToQBA(ftpPasswordValue->text(),16));
        needToSave = true;
    }
    if(ftpDirectoryValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x3A7,stringToQBA(ftpDirectoryValue->text(),8));
        needToSave = true;
    }
    if (needToSave)
    {
        QByteArray configQByteArray;
        serialCom->WriteEthernetModuleSpecial(0x22,configQByteArray);
    }
}

void EthernetConfig::ftpConfigQuit()
{
    closeQuit();
}

void EthernetConfig::networkConfigQuit()
{
    closeQuit();
}

void EthernetConfig::timeConfigQuit()
{
    closeQuit();
}

void  EthernetConfig::ftpSendNow()
{
    QByteArray configQByteArray;
    serialCom->WriteEthernetModuleSpecial(0x20,configQByteArray);

    if (rightWidget!=NULL)
    {
        horizontalConfigLayout->removeWidget(rightWidget);
        delete rightWidget;
    }
    rightWidget = new QWidget;
    rightVLayout = new QVBoxLayout;
    QLabel *lab1 = new QLabel("Ongoing Transfer..");
    rightVLayout->addWidget(lab1);
    QLabel *lab2 = new QLabel("Please wait until");
    rightVLayout->addWidget(lab2);
    QLabel *lab3 = new QLabel("yellow LED blinks slow.");
    rightVLayout->addWidget(lab3);
    rightWidget->setLayout(rightVLayout);
    horizontalConfigLayout->addWidget(rightWidget);
}

void  EthernetConfig::timeConfig()
{
    if (rightWidget!=NULL)
    {
        horizontalConfigLayout->removeWidget(rightWidget);
        delete rightWidget;
    }
    rightWidget = new QWidget;
    rightlayout = new QGridLayout;
        QLabel *lab1 = new QLabel("Time Server IP:");
        rightlayout->addWidget(lab1,0,0);
        QLabel *lab2 = new QLabel("Call Time:");
        rightlayout->addWidget(lab2,1,0);
        QLabel *lab3 = new QLabel("Time Zone:");
        rightlayout->addWidget(lab3,2,0);
        QLabel *lab4 = new QLabel("Time Now:");
        rightlayout->addWidget(lab4,3,0);

        timeServerIPValue = new QLineEdit();
        rightlayout->addWidget(timeServerIPValue,0,1);
        callTimeValue = new QLineEdit();
        rightlayout->addWidget(callTimeValue,1,1);
        timeZoneValue = new QLineEdit();
        rightlayout->addWidget(timeZoneValue,2,1);
        timeNowValue = new QLineEdit();
        rightlayout->addWidget(timeNowValue,3,1);

        timeConfigSaveButton = new QPushButton("Save");
        rightlayout->addWidget(timeConfigSaveButton,8,0);
        connect(timeConfigSaveButton, SIGNAL(clicked()),this,SLOT(timeConfigSave()));
        connect(timeConfigSaveButton, SIGNAL(clicked()),this,SLOT(timeConfigUpdate()));
        timeConfigQuitButton = new QPushButton("Quit");
        rightlayout->addWidget(timeConfigQuitButton,8,1);
        connect(timeConfigQuitButton, SIGNAL(clicked()),this,SLOT(timeConfigQuit()));
    rightWidget->setLayout(rightlayout);
    horizontalConfigLayout->addWidget(rightWidget);
    timeConfigUpdate();
}

void EthernetConfig::timeConfigUpdate()
{
    // Remplit les valeurs
    QByteArray receivedCharArray=serialCom->ReadEthernetModuleRam(0x397,16);
    if (receivedCharArray.length()>0)
    {
        timeServerIPValue->setText(receivedCharArray);
    }

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x3B0,3);
    if (receivedCharArray.length()>1)
    {
        callTimeValue->setText (QString::number(uchar(receivedCharArray.at(0))).rightJustified(2,'0') + ':' +
                                QString::number(uchar(receivedCharArray.at(1))).rightJustified(2,'0') );
        timeZoneValue->setText(QString::number(uchar(receivedCharArray.at(2))));
    }

    receivedCharArray=serialCom->ReadEthernetModuleRam(0xa9f,3);
    if (receivedCharArray.length()>1)
    {
        timeNowValue->setText (QString::number(uchar(receivedCharArray.at(2))).rightJustified(2,'0') + ':' +
                              QString::number(uchar(receivedCharArray.at(1))).rightJustified(2,'0') + ':' +
                              QString::number(uchar(receivedCharArray.at(0))).rightJustified(2,'0') );
    }
}

void EthernetConfig::timeConfigSave()
{
    bool needToSave = false;
    if(timeServerIPValue->isModified())
    {
        serialCom->WriteEthernetModuleRam(0x397,stringToQBA(timeServerIPValue->text(),16));
        needToSave = true;
    }
    if(callTimeValue->isModified())
    {
        QStringList callStringList = callTimeValue->text().split(":");
        QByteArray textQByteArray;
                textQByteArray.append(callStringList.at(0).toInt());
                textQByteArray.append(callStringList.at(1).toInt());
        serialCom->WriteEthernetModuleRam(0x3B0,textQByteArray);
        needToSave = true;
    }
    if(timeZoneValue->isModified())
    {
        QByteArray timeZoneQByteArray;
        timeZoneQByteArray.append(timeZoneValue->text().toInt());
        serialCom->WriteEthernetModuleRam(0x3B2,timeZoneQByteArray);
        needToSave = true;
    }
    if (needToSave)
    {
        QByteArray configQByteArray;
        serialCom->WriteEthernetModuleSpecial(0x22,configQByteArray);
    }
}

void EthernetConfig:: mvConfig()
{
    if (rightWidget!=NULL)
    {
        horizontalConfigLayout->removeWidget(rightWidget);
        delete rightWidget;
    }
    rightWidget = new QWidget;
    rightVLayout = new QVBoxLayout;
    QHBoxLayout *hb = new QHBoxLayout;
    QLabel *lab1 = new QLabel("Add System:");
    hb->addWidget(lab1);
    systemAddEdit = new QLineEdit();
    hb->addWidget(systemAddEdit);
    mvAddButton = new QPushButton( "Add");
    hb->addWidget(mvAddButton);
    connect(mvAddButton, SIGNAL(clicked()),this,SLOT(mvConfigAdd()));
    connect(mvAddButton, SIGNAL(clicked()),this,SLOT(mvConfigUpdate()));

    rightVLayout->addLayout(hb);

    QLabel *lab2 = new QLabel("Recorded Systems");
    rightVLayout->addWidget(lab2);
    systemsListBox = new QListView( this );
    rightVLayout->addWidget(systemsListBox);

    QHBoxLayout *hb2 = new QHBoxLayout;
    mvConfigButton = new QPushButton( "Config");
    hb2->addWidget(mvConfigButton);
    connect(mvConfigButton, SIGNAL(clicked()),this,SLOT(mvConfigConfig()));

    mvDelButton = new QPushButton( "Del");
    hb2->addWidget(mvDelButton);
    connect(mvDelButton, SIGNAL(clicked()),this,SLOT(mvConfigDel()));
    connect(mvDelButton, SIGNAL(clicked()),this,SLOT(mvConfigUpdate()));

    mvQuitButton = new QPushButton( "Quit");
    hb2->addWidget(mvQuitButton);
    connect(mvQuitButton, SIGNAL(clicked()),this,SLOT(mvConfigQuit()));

    rightVLayout->addLayout(hb2);

    rightWidget->setLayout(rightVLayout);
    horizontalConfigLayout->addWidget(rightWidget);
    mvConfigUpdate();
}

void EthernetConfig::mvConfigUpdate()
{
    // Remplit les valeurs
    QByteArray receivedCharArray=serialCom->ReadEthernetModuleRam(0x3AF,1);
    if (receivedCharArray.length()>0)
    {
        nbSys = receivedCharArray.at(0);
    }

    listSysView.clear();
    for (int i=1;i<(nbSys+1);i++)
    {
        listSys[i]=0;
        listSysNom[i]="-";
        receivedCharArray=serialCom->ReadEthernetModuleRam(0x301+2*(i-1),2);
        if (receivedCharArray.length()>0)
        {
            listSys[i]=receivedCharArray.at(0)+256*receivedCharArray.at(1);
            receivedCharArray=serialCom->ReadEthernetModuleRam(0x500+16*(i-1),16);
            if (receivedCharArray.length()>0)
            {
                listSysNom[i] = receivedCharArray.data();
            }
        }
        listSysView.append(QString::number(listSys[i],16) +' ' + listSysNom[i]  );
    }
    systemsListBox->setModel(new QStringListModel(listSysView));
    systemsListBox->setSelectionMode(QAbstractItemView::SingleSelection);

    QModelIndex index = systemsListBox->model()->index(0,0);
    systemsListBox->setCurrentIndex(index);
}

void EthernetConfig::mvConfigAdd()
{
    QByteArray configQByteArray;
    configQByteArray.push_back(systemAddEdit->text().toInt(0,16) & 0xff);
    configQByteArray.push_back(systemAddEdit->text().toInt(0,16) >>8);
    serialCom->WriteEthernetModuleSpecial(0x03,configQByteArray);
}

void EthernetConfig::mvConfigConfig()
{
    QModelIndex index = systemsListBox->currentIndex();
    radioSelIndex = index.row();

    if (rightWidget!=NULL)
    {
        horizontalConfigLayout->removeWidget(rightWidget);
        delete rightWidget;
    }
    rightWidget = new QWidget;
    QHBoxLayout *hb1 = new QHBoxLayout;

    rightVLayout = new QVBoxLayout;

    QHBoxLayout *hb2 = new QHBoxLayout;
    QLabel *lab1 = new QLabel("Wireless Module:");
    hb2->addWidget(lab1);
    wModuleEdit = new QLineEdit();
    hb2->addWidget(wModuleEdit);
    rightVLayout->addLayout(hb2);

    QHBoxLayout *hb3 = new QHBoxLayout;
    QLabel *lab2 = new QLabel("Concentrator:");
    hb3->addWidget(lab2);
    centralEdit = new QLineEdit();
    hb3->addWidget(centralEdit);
    rightVLayout->addLayout(hb3);

    QHBoxLayout *hb4 = new QHBoxLayout;
    QLabel *lab3 = new QLabel("Name:");
    hb4->addWidget(lab3);
    centralNameEdit = new QLineEdit();
    hb4->addWidget(centralNameEdit);
    rightVLayout->addLayout(hb4);

    enableUpload = new QCheckBox("Enable Upload");
    rightVLayout->addWidget(enableUpload);
    connect(enableUpload, SIGNAL(stateChanged(int)),this,SLOT(mvUploadChanged(int)));

    QHBoxLayout *hb5 = new QHBoxLayout;
    QLabel *lab4 = new QLabel("Records to upload:");
    hb5->addWidget(lab4);
    dataToLoadEdit = new QLineEdit();
    hb5->addWidget(dataToLoadEdit);
    rightVLayout->addLayout(hb5);

    resetDataButton = new QPushButton("Reset Data Pointer");
    rightVLayout->addWidget(resetDataButton);
    connect(resetDataButton, SIGNAL(clicked()),this,SLOT(mvResetData()));
    connect(resetDataButton, SIGNAL(clicked()),this,SLOT(mvModuleUpdate()));

    QHBoxLayout *hb6 = new QHBoxLayout;
    searchModuleButton = new QPushButton("Search Modules");
    hb6->addWidget(searchModuleButton);
    connect(searchModuleButton, SIGNAL(clicked()),this,SLOT(mvSearch()));
    //connect(searchModuleButton, SIGNAL(clicked()),this,SLOT(mvModuleUpdate()));

    searchMoreButton = new QPushButton("Search More");
    hb6->addWidget(searchMoreButton);
    connect(searchMoreButton, SIGNAL(clicked()),this,SLOT(mvSearchMore()));
    //connect(searchMoreButton, SIGNAL(clicked()),this,SLOT(mvModuleUpdate()));
    rightVLayout->addLayout(hb6);

    moduleQuitButton = new QPushButton("Quit");
    connect(moduleQuitButton, SIGNAL(clicked()),this,SLOT(mvModuleQuit()));
    rightVLayout->addWidget(moduleQuitButton);

    hb1->addLayout(rightVLayout);

    QVBoxLayout *hb7 = new QVBoxLayout;
    QLabel *lab5 = new QLabel("Modules:");
    hb7->addWidget(lab5);

    modulesListView = new QListView;
    hb7->addWidget(modulesListView);

    hb1->addLayout(hb7);
    rightWidget->setLayout(hb1);
    horizontalConfigLayout->addWidget(rightWidget);
    //mvSearch();
    mvModuleUpdate();
}

void EthernetConfig::mvConfigDel()
{
    QModelIndex index = systemsListBox->currentIndex();
    radioSelIndex=index.row();
    for (int i = 0 ; i < nbSys ; i++)
    {
        if (radioSelIndex==i)
        {
            QByteArray configQByteArray;
            configQByteArray.push_back(i);
            serialCom->WriteEthernetModuleSpecial(0x04,configQByteArray);
        }
    }
}

void EthernetConfig::mvModuleUpdate()
{
    // Remplit les valeurs
    QByteArray receivedCharArray=serialCom->ReadEthernetModuleRam(0x333+radioSelIndex,1);
    if (receivedCharArray.length()>0)
    {
        enableUpload->blockSignals(true);
        if(!receivedCharArray[0])   // Bit6
            enableUpload->setChecked(false);
        else
            enableUpload->setChecked(true);
        enableUpload->blockSignals(false);
    }

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x33D+radioSelIndex,1);
    if (receivedCharArray.length()>0)
    {
        int nbMod = receivedCharArray.at(0);
        listModView.clear();
        for (int i=0;i<nbMod;i++)
        {
            receivedCharArray=serialCom->ReadEthernetModuleFlash(64*(0x3DC+radioSelIndex)+4*i,4);
            uint32 modSN = QBA2int(&receivedCharArray);
            listModView.append(QString::number(modSN,16).rightJustified(8,'0'));
        }
        modulesListView->setModel(new QStringListModel(listModView));
    }

    wModuleEdit->setText(QString::number(listSys[radioSelIndex+1],16));

    QByteArray configQByteArray;
    configQByteArray.push_back(radioSelIndex);
    serialCom->WriteEthernetModuleSpecial(0x12,configQByteArray);   // Recherche Concentrateur
    waitFor(500);

    receivedCharArray=serialCom->ReadEthernetModuleRam(0x480+4*radioSelIndex,4);
    if (receivedCharArray.length()>0)
    {
       uint32 conSN = QBA2int(&receivedCharArray);
       centralEdit->setText(QString::number(conSN,16).rightJustified(8,'0'));
    }

     receivedCharArray=serialCom->ReadEthernetModuleRam(0x500+16*radioSelIndex,16);
     if (receivedCharArray.length()>0)
     {
        centralNameEdit->setText(QString(receivedCharArray));
     }

     receivedCharArray=serialCom->ReadEthernetModuleRam(0x315+3*radioSelIndex,4);
     if (receivedCharArray.length()>0)
     {
        uint32 ad1 = QBA2int(&receivedCharArray) & 0xffffff;
        receivedCharArray=serialCom->ReadEthernetModuleRam(0x154,10);
        int ndata=0;
        if (receivedCharArray.length()>0)
        {
           uint32 ad2 = QBA2int(&receivedCharArray) & 0xffffff;
           if (ad1>ad2)
           {
                if (receivedCharArray.at(5))
                {
                    if (receivedCharArray.at(9)==2) ndata = 0x7E000 - ad2 + ad1;
                    else  ndata = 0x3C000 - ad2 + ad1;
                }
                else ndata = ad1;
           }
           else ndata = ad2 - ad1;

        dataToLoadEdit->setText(QString::number(ndata / 12,10));
        }
    }
}

void EthernetConfig::mvConfigQuit()
{
    closeQuit();
}

void EthernetConfig::mvResetData()
{
    QByteArray configQByteArray;
    configQByteArray.push_back(radioSelIndex);
    serialCom->WriteEthernetModuleSpecial(0x14,configQByteArray);
    waitFor(100);
}

void EthernetConfig::mvSearch()
{
    QByteArray configQByteArray;
    configQByteArray.push_back(radioSelIndex);
    serialCom->WriteEthernetModuleSpecial(0x10,configQByteArray);
    waitFor(300);
    mvModuleUpdate();
}

void EthernetConfig::mvSearchMore()
{
    QByteArray configQByteArray;
    configQByteArray.push_back(radioSelIndex);
    serialCom->WriteEthernetModuleSpecial(0x11,configQByteArray);
    waitFor(300);
    mvModuleUpdate();
}

void EthernetConfig::mvUploadChanged(int state)
{
    QByteArray configQByteArray;
    if (!state)
        configQByteArray.push_back(char(0));
    else
        configQByteArray.push_back(char(255));
    serialCom->WriteEthernetModuleRam(0x333+radioSelIndex,configQByteArray);
    configQByteArray.clear();
    serialCom->WriteEthernetModuleSpecial(0x22,configQByteArray);
    waitFor(300);
}

void EthernetConfig::mvModuleQuit()
{
    mvConfig();
}

//***************
// Events
//***************

void EthernetConfig::closeEvent(QCloseEvent *event)
 {  //event when window is closed
 //   setVisible(false);
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

    settingFile->setValue("WINDOW_POS/ETH_WIN_X",QVariant(QString::number(this->pos().x())));
    settingFile->setValue("WINDOW_POS/ETH_WIN_Y",QVariant(QString::number(this->pos().y())));
    emit closingWindow();
 }
