#include "gprsconfig.h"

GprsConfig::GprsConfig(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, vector<Module> *_moduleList, QWidget *parent) :
    QWidget(parent)
{   //Constructor
    serialCom = _serialPort;
    commonRefreshTimer=_commonRefreshTimer;
    firstTimeOpened=true;
    saveIMEI=false;
    centralModuleList=_moduleList;

    setWindowIcon(QIcon(QString(DEFAULT_DIR)+"icon/simCard.png"));
    setWindowTitle("Multivoies - GPRS Configuration");
    settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    if(settingFile->contains("WINDOW_POS/GPRS_WIN_X")&&
            settingFile->contains("WINDOW_POS/GPRS_WIN_Y"))
    {
        int maxX=QApplication::desktop()->width();
        int maxY=QApplication::desktop()->height();
        int posX=settingFile->value("WINDOW_POS/GPRS_WIN_X").toInt();
        int posY=settingFile->value("WINDOW_POS/GPRS_WIN_Y").toInt();
        if(((maxX-posX-100)>0)&&((maxY-posY-70)>0))this->move(posX,posY);
    }
    fillWindow();
    updateModuleListFromCentral();
    show();
    fetchGprsInfo();

    resize(300,340);
    setWindowModality(Qt::ApplicationModal);
    qDebug()<<"test";
    timerSlot();
}

//***************************
// GUI
//***************************

void GprsConfig::fillWindow()
{   //set GUI
    no_com=true;
    windowLayout = new QHBoxLayout;
    normalConfigLayout = new QVBoxLayout;   //configuration menu
    windowLayout->addLayout(normalConfigLayout);

        versionLabel = new QLabel("xx");
            normalConfigLayout->addWidget(versionLabel);
        //Lists of modules
        QHBoxLayout *listsLayout = new QHBoxLayout;
            QVBoxLayout *moduleListLayout = new QVBoxLayout;
                moduleListView = new QPlainTextEdit;
                moduleListView->setMaximumHeight(140);
                moduleListView->setReadOnly(true);
                moduleListLayout->addWidget(moduleListView);
                QLabel *moduleListLabel= new QLabel("Multivoies");
                moduleListLayout->addWidget(moduleListLabel);
                listsLayout->addLayout(moduleListLayout);
            shiftModules = new QPushButton(">");
            shiftModules->setMaximumWidth(30);
            connect(shiftModules,SIGNAL(clicked()),this,SLOT(setGprsModuleListFromCentral()));
            listsLayout->addWidget(shiftModules);
            QVBoxLayout *moduleGprsListLayout = new QVBoxLayout;
                gprsModuleListView = new QPlainTextEdit;
                gprsModuleListView->setMaximumHeight(140);
                gprsModuleListView->setReadOnly(true);
                moduleGprsListLayout->addWidget(gprsModuleListView);
                QLabel *moduleGprsListLabel= new QLabel("GPRS");
                moduleGprsListLayout->addWidget(moduleGprsListLabel);
                listsLayout->addLayout(moduleGprsListLayout);
            normalConfigLayout->addLayout(listsLayout);

        QSpacerItem *spacerModules = new QSpacerItem(20,20,QSizePolicy::Minimum,QSizePolicy::Expanding);
            normalConfigLayout->addItem(spacerModules);

        //Call time
        QHBoxLayout *callTimeLayout = new QHBoxLayout;
            QLabel *callTimeLabel = new QLabel("Call Time :");
            callTimeLayout->addWidget(callTimeLabel);
            callTimeHours = new QSpinBox();
            callTimeLayout->addWidget(callTimeHours);
            callTimeHours->setMinimum(0);
            callTimeHours->setMaximum(25);
            callTimeHours->setMaximumWidth(60);
            QLabel *hourSeparator = new QLabel(":");
            hourSeparator->setMaximumWidth(15);
            callTimeLayout->addWidget(hourSeparator);
            callTimeMinutes = new QSpinBox();
            callTimeLayout->addWidget(callTimeMinutes);
            callTimeMinutes->setMinimum(0);
            callTimeMinutes->setMaximum(59);
            callTimeMinutes->setMaximumWidth(60);
            checkHeader = new QCheckBox("Header");
            callTimeLayout->addWidget(checkHeader);
            normalConfigLayout->addLayout(callTimeLayout);
        //Directory
            QHBoxLayout *ftpsLayout = new QHBoxLayout;
                    QLabel *ftpsLabel = new QLabel("FTP server :");
                    ftpsLayout->addWidget(ftpsLabel);
                    ftps = new QLineEdit();
                    ftps->setMaxLength(31);
                    ftps->setMaximumWidth(200);
                    ftpsLayout->addWidget(ftps);


            normalConfigLayout->addLayout(ftpsLayout);

            QHBoxLayout *ftpuLayout = new QHBoxLayout;
            QLabel *ftpuLabel = new QLabel("FTP user :");
            ftpuLayout->addWidget(ftpuLabel);
            ftpu = new QLineEdit();
            ftpu->setMaxLength(15);
            ftpu->setMaximumWidth(80);
            ftpuLayout->addWidget(ftpu);
            QLabel *ftppLabel = new QLabel("FTP pass :");
            ftpuLayout->addWidget(ftppLabel);
            ftpp = new QLineEdit();
            ftpp->setMaxLength(15);
            ftpp->setMaximumWidth(80);
            ftpuLayout->addWidget(ftpp);

            normalConfigLayout->addLayout(ftpuLayout);


            QHBoxLayout *apnLayout = new QHBoxLayout;
            QLabel *apnLabel = new QLabel("APN :");
            apnLayout->addWidget(apnLabel);
            apn= new QLineEdit();
            apn->setMaxLength(31);
            apn->setMaximumWidth(200);
            apnLayout->addWidget(apn);
            normalConfigLayout->addLayout(apnLayout);

            QHBoxLayout *apnupLayout = new QHBoxLayout;
            checkAPN = new QCheckBox();
            apnupLayout->addWidget(checkAPN);

            QLabel *apnuLabel = new QLabel("APN user :");
            apnupLayout->addWidget(apnuLabel);
            apnu = new QLineEdit();
            apnu->setMaxLength(15);
            apnu->setMaximumWidth(60);
            apnupLayout->addWidget(apnu);

            QLabel *apnpLabel = new QLabel("APN pass :");

            apnupLayout->addWidget(apnpLabel);
            apnp = new QLineEdit();
            apnp->setMaxLength(15);
            apnp->setMaximumWidth(60);
            apnupLayout->addWidget(apnp);

            normalConfigLayout->addLayout(apnupLayout);

            QHBoxLayout *dirLayout = new QHBoxLayout;
            QLabel *directoryLabel = new QLabel("FTP Directory :");
            dirLayout->addWidget(directoryLabel);
            ftpDirectory = new QLineEdit();
            ftpDirectory->setMaxLength(5);
            ftpDirectory->setMaximumWidth(60);
            dirLayout->addWidget(ftpDirectory);
            normalConfigLayout->addLayout(dirLayout);





        saveConfigButton = new QPushButton("Save config");
        connect(saveConfigButton, SIGNAL(clicked()),this,SLOT(saveConfiguration()));
        normalConfigLayout->addWidget(saveConfigButton);

        QSpacerItem *spacerTime = new QSpacerItem(20,20,QSizePolicy::Minimum,QSizePolicy::Expanding);
            normalConfigLayout->addItem(spacerTime);
        //Gprs time
        QHBoxLayout *gprsTimeLayout = new QHBoxLayout;
            QLabel *gprsTimeLabel = new QLabel("GPRS Time :");
            gprsTimeLayout->addWidget(gprsTimeLabel);
            gprsTime = new QLabel("xx/xx/20xx xx:xx:xx");
            gprsTimeLayout->addWidget(gprsTime);
            centralTimeButton = new QPushButton("Apply to central");
            connect(centralTimeButton,SIGNAL(clicked()),this,SLOT(sendTimeToCentral()));
            gprsTimeLayout->addWidget(centralTimeButton);
            normalConfigLayout->addLayout(gprsTimeLayout);
        //Number of data to send
        QHBoxLayout *dataPointerLayout = new QHBoxLayout;
            QLabel *dataPointerLabel = new QLabel("Last data :");
            dataPointerLayout->addWidget(dataPointerLabel);
            dataPointer = new QLabel("xx");
            dataPointerLayout->addWidget(dataPointer);
            sendRemainingData = new QPushButton("Send all data");
            dataPointerLayout->addWidget(sendRemainingData);
            connect(sendRemainingData,SIGNAL(clicked()),this,SLOT(resendAllData()));
            normalConfigLayout->addLayout(dataPointerLayout);

        QHBoxLayout *FSLayout = new QHBoxLayout;
            QPushButton *f_send_butt= new QPushButton("Force Send");
            FSLayout->addWidget(f_send_butt);
            connect(f_send_butt,SIGNAL(clicked()),this,SLOT(force_send()));
            normalConfigLayout->addLayout(FSLayout);

        QHBoxLayout *refreshLayout = new QHBoxLayout;
            QPushButton *refresh_butt= new QPushButton("Refresh");
            FSLayout->addWidget(refresh_butt);
            connect(refresh_butt,SIGNAL(clicked()),this,SLOT(fetchGprsInfo()));
            normalConfigLayout->addLayout(refreshLayout);



    QFrame *line = new QFrame();
           line->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);
           line->setFrameShape(QFrame::VLine);
    windowLayout->addWidget(line);
           line->setFrameShadow(QFrame::Sunken);

    //Commands configuration
    advancedConfigLayout= new QVBoxLayout;
    windowLayout->addLayout(advancedConfigLayout);
        logView = new QPlainTextEdit;
        logView->setReadOnly(true);
        advancedConfigLayout->addWidget(logView);
        preselectedCommands = new QComboBox;
        advancedConfigLayout->addWidget(preselectedCommands);
        preselectedCommands->setMaxVisibleItems(50);    //high to see all

        //Take preselected commands from the .ini file
        QSettings *settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
        settingFile->beginGroup("MODEM");
        QStringList keyList = settingFile->allKeys();
        QStringList commandList;
        for(int keyIndex=0; keyIndex<keyList.size(); keyIndex++)
        {
           QString commandToAdd=settingFile->value(keyList.at(keyIndex)).toString();
           commandList.append(commandToAdd);
        }
       preselectedCommands->addItems(commandList);
       delete settingFile;
       preselectedCommands->setCurrentIndex(-1);
       connect(preselectedCommands,SIGNAL(currentIndexChanged(int)),this,SLOT(newCommandSelected(int)));

       commandLineEdit = new QLineEdit;
       advancedConfigLayout->addWidget(commandLineEdit);

       QHBoxLayout *commandButtonLayout=new QHBoxLayout;
       advancedConfigLayout->addLayout(commandButtonLayout);
       sendCommandLine = new QPushButton("Send");   //to send to gprs
       commandButtonLayout->addWidget(sendCommandLine);
       connect(sendCommandLine,SIGNAL(clicked()),this,SLOT(sendCommand()));
       saveCommandLine = new QPushButton("Save");   //to save into ini file
       commandButtonLayout->addWidget(saveCommandLine);
       connect(saveCommandLine,SIGNAL(clicked()),this,SLOT(saveCommand()));

       QHBoxLayout *commandButtonLayout2=new QHBoxLayout;
       QPushButton *save_IMEI_to_file = new QPushButton("Get and save IMEI (imei.txt)");
       commandButtonLayout2->addWidget(save_IMEI_to_file);
       QPushButton *clr_IMEI_file = new QPushButton("Clear imei.txt file");
       commandButtonLayout2->addWidget(clr_IMEI_file);
       connect(save_IMEI_to_file,SIGNAL(clicked()),this,SLOT(save_IMEI()));
       connect(clr_IMEI_file,SIGNAL(clicked()),this,SLOT(clr_IMEI()));
       advancedConfigLayout->addLayout(commandButtonLayout2);

       setLayout(windowLayout);

}

//*********************************
//  Init methods
//*********************************

void GprsConfig::updateModuleListFromCentral()
{
    //update the moduleList widget with module List from ManageModule
    moduleListView->clear();
    QString moduleListString;
    for(uint32 moduleIndex=0; moduleIndex<centralModuleList->size(); moduleIndex++)
    {
        moduleListString.append(QString::number(
                    centralModuleList->at(moduleIndex).serialNb,16).rightJustified(8,'0'));
        if(moduleIndex!=centralModuleList->size()-1) moduleListString.append("\n");
    }
    moduleListView->setPlainText(moduleListString);
}

void GprsConfig::updateGprsModuleListFromGprs()
{   //Read module list from gprs

    //read nb of modules from GPRS
    uchar nbModulesFromGprs=0;
    bool ok=serialCom->readBytesAtWithRetry(GPRS_SN,0x01000008,1,3);
    if(ok) nbModulesFromGprs=serialCom->getReceivedData()[0];
    if(nbModulesFromGprs>19) nbModulesFromGprs=0;

    //read modules SN from GPRS
    QString gprsModuleList;
    if(nbModulesFromGprs==0) gprsModuleList="No Module";
    else
    {
        for(int moduleIndex=0; moduleIndex<nbModulesFromGprs; moduleIndex++)
        {
            if(serialCom->readBytesAtWithRetry(GPRS_SN,0x01000030+4*moduleIndex,1,3))
            {
                gprsModuleList.append(QString::number(peekUint32FromUcharTabLittleEndian(
                                        serialCom->getReceivedData(),0),16).rightJustified(8,'0'));
            }
            else gprsModuleList.append("Err COM");
            if(moduleIndex!=(nbModulesFromGprs-1))
            {
                gprsModuleList.append("\n");
            }
        }
    }
    gprsModuleListView->setPlainText(gprsModuleList);
}

void GprsConfig::initWindow()
{   //start window : update values and check if there is a gprs module
    commonRefreshTimer->stop();
     commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startInterval();
}

bool GprsConfig::fetchGprsInfo()
 {  //Get call Time, data to send, directory... from gprs module
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x01000000,2,3))
        return false;

    //Call time, data pointer, version & header
    callTimeHours->setValue(serialCom->getReceivedData()[6]);
    callTimeMinutes->setValue(serialCom->getReceivedData()[7]&0x3f);    //MSBit = header
    dataPointer->setText(QString::number(peekUint32FromUcharTabLittleEndian(serialCom->getReceivedData(),10)));
    versionLabel->setText("V"+QString::number(serialCom->getReceivedData()[5]));
    if(serialCom->getReceivedData()[7]>>7==1)   //MSBit
        checkHeader->setChecked(true);
    else
        checkHeader->setChecked(false);

    //fetch Ftp Directory
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x01000010,1,3))
        return false;
    char directory[6];  //Not more than 6 char
    for(uint32 i =0; i<6;i++)
    {
        if(serialCom->getReceivedData()[i]!=(0x00)&&serialCom->getReceivedData()[i]!=(0xFF))
            directory[i]=serialCom->getReceivedData()[i];
        else
        {
            directory[i]='\0'; //clear next char
            break;
        }
    }
    ftpDirectory->setText(QString(directory));

    //fetch Ftp server
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x010000B0,8,3))
        return false;
    char ftp_serv[64];  //Not more than 6 char
    for(uint32 i =0; i<64;i++)
    {
        if(serialCom->getReceivedData()[i]!=(0x00)&&serialCom->getReceivedData()[i]!=(0xFF))
            ftp_serv[i]=serialCom->getReceivedData()[i];
        else
        {
            ftp_serv[i]='\0'; //clear next char
            break;
        }
    }
    ftps->setText(QString(ftp_serv));

    //fetch Ftp user
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x010000D0,2,3))
        return false;
    char ftp_u[16];  //Not more than 16 char
    for(uint32 i =0; i<16;i++)
    {
        if(serialCom->getReceivedData()[i]!=(0x00)&&serialCom->getReceivedData()[i]!=(0xFF))
            ftp_u[i]=serialCom->getReceivedData()[i];
        else
        {
            ftp_u[i]='\0'; //clear next char
            break;
        }
    }
    ftpu->setText(QString(ftp_u));

    //fetch Ftp server
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x010000E0,2,3))
        return false;
    char ftp_p[16];  //Not more than 6 char
    for(uint32 i =0; i<16;i++)
    {
        if(serialCom->getReceivedData()[i]!=(0x00)&&serialCom->getReceivedData()[i]!=(0xFF))
            ftp_p[i]=serialCom->getReceivedData()[i];
        else
        {
            ftp_p[i]='\0'; //clear next char
            break;
        }
    }
    ftpp->setText(QString(ftp_p));



    //fetch Ftp server
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x01000090,4,3))
        return false;
    char apn_c[32];  //Not more than 6 char
    for(uint32 i =0; i<32;i++)
    {
        if(serialCom->getReceivedData()[i]!=(0x00)&&serialCom->getReceivedData()[i]!=(0xFF))
            apn_c[i]=serialCom->getReceivedData()[i];
        else
        {
            apn_c[i]='\0'; //clear next char
            break;
        }
    }
    apn->setText(QString(apn_c));

    //fetch Ftp server
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x010000F0,2,3))
        return false;
    char apn_u[10];  //Not more than 6 char
    for(uint32 i =0; i<10;i++)
    {
        if(serialCom->getReceivedData()[i]!=(0x00)&&serialCom->getReceivedData()[i]!=(0xFF))
            apn_u[i]=serialCom->getReceivedData()[i];
        else
        {
            apn_u[i]='\0'; //clear next char
            break;
        }
    }
    if(apn_u[0]==0)
    {
    checkAPN->setChecked(false);
    apnu->setText("");
    apnp->setText("");

    }
    else
    {
        checkAPN->setChecked(true);
        apnu->setText(QString(apn_u));
            //fetch Ftp server
            if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x010000FA,2,3))
                return false;
            char apn_p[10];  //Not more than 6 char
            for(uint32 i =0; i<10;i++)
            {
                if(serialCom->getReceivedData()[i]!=(0x00)&&serialCom->getReceivedData()[i]!=(0xFF))
                    apn_p[i]=serialCom->getReceivedData()[i];
                else
                {
                    apn_p[i]='\0'; //clear next char
                    break;
                }
            }
            apnp->setText(QString(apn_p));

    }

    updateGprsModuleListFromGprs();
   updateGprsTime();

    return true;
}

//*********************************
//  Update values methods
//*********************************

void GprsConfig::timerSlot()
{   // connected to refresh timer, update time and log view (commands)
    commonRefreshTimer->stop();
    //serialCom->deleteErrorCom();    //retry

  /* if(no_com==true)
   {
        qDebug()<<"test2";
       if(fetchGprsInfo())
       {
            updateGprsModuleListFromGprs();
            if(updateGprsTime())
                no_com=false;
        }
   }*/
    //update GPRS Time
   // else
        if(!updateGprsTime())
    {
        gprsModuleListView->setPlainText("Err COM\n");
       /* versionLabel->setText("xx");
        ftpDirectory->setText(" ");*/
        gprsTime->setText("xx/xx/20xx xx:xx:xx");
      //  callTimeHours->setValue(0);
      //  callTimeMinutes ->setValue(0);
        logView->appendPlainText("NO COM");
        no_com=true;
    }
    //update the log view
    updateLogView();

    commonRefreshTimer->startInterval();    //next refresh
}

bool GprsConfig::updateGprsTime()
{   //update date and time from Gprs
    if(!serialCom->readBytesAtWithRetry(GPRS_SN,0x00000020,1,3))
        return false;
    QString dateToShow = QString::number(serialCom->getReceivedData()[2]).rightJustified(2,'0')+"/"   //day
        +QString::number(serialCom->getReceivedData()[1]).rightJustified(2,'0')+"/20"      //month
        +QString::number(serialCom->getReceivedData()[0]).rightJustified(2,'0')+" "        //year
        +QString::number(serialCom->getReceivedData()[3]).rightJustified(2,'0')+":"        //hour
        +QString::number(serialCom->getReceivedData()[4]).rightJustified(2,'0')+":"        //min
        +QString::number(serialCom->getReceivedData()[5]).rightJustified(2,'0');        //sec
    gprsTime->setText(dateToShow);
    return true;
}

//*********************************
//  Send values to module
//*********************************

void GprsConfig::setGprsModuleListFromCentral()
{   //Send module list from detection to Central
    bool ok=true;
    uchar nbOfModulesToGprs=0;
    commonRefreshTimer->stop();//---------------------------

    for(uint32 moduleIndex=0; moduleIndex<centralModuleList->size(); moduleIndex++)
    {
        nbOfModulesToGprs++;
        //send serialNb
        for(uint32 pos=0; pos<4; pos++)
        {
            ok=ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x01000033+(moduleIndex<<2)-pos,
                    (centralModuleList->at(moduleIndex).serialNb>>(8*pos)) & 0xFF,3);
        }
    }
    //send nbOfModulesToGprs
    ok= ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x01000008,nbOfModulesToGprs,3);

    //reinit data pointer
    resendAllData();
    //restart the window
  //  initWindow();
    no_com=true;
    commonRefreshTimer->startSmallInterval();
}

void GprsConfig::saveConfiguration()
{   //With button, save call time, header and directory
    //hour
    bool ok=serialCom->writeByteAtWithRetry(GPRS_SN,0x01000006,callTimeHours->value(),3);
    //minutes + header
    uchar minutesValue = callTimeMinutes->value() & 0x3f;
    if(checkHeader->isChecked()) minutesValue = minutesValue | 0x80;
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x01000007,minutesValue,3);

    //Directory
    string str_to_send=ftpDirectory->text().toStdString();
    int index=0;
    uchar charToSend;
    while((index<5) && (str_to_send[index]!=0))
    {
        charToSend=uchar(str_to_send.at(index));
    //    qDebug()<<"send char "<<index<<" = "<<charToSend;
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x01000010+index,uchar(charToSend),3);
        index++;
    }
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x01000010+index,0,3);

    //ftp_serv
    str_to_send=ftps->text().toStdString();
    index=0;
    while((index<31) && (str_to_send[index]!=0))
    {
        charToSend=uchar(str_to_send.at(index));
        qDebug()<<"send char "<<index<<" = "<<str_to_send.at(index);
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000B0+index,uchar(charToSend),3);
        index++;
    }
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000B0+index,0,3);

    //ftp_user
    str_to_send=ftpu->text().toStdString();
    index=0;
    while((index<15) && (str_to_send[index]!=0))
    {
        charToSend=uchar(str_to_send.at(index));
        qDebug()<<"send char "<<index<<" = "<<str_to_send.at(index);
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000D0+index,uchar(charToSend),3);
        index++;
    }
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000D0+index,0,3);

    //ftp_pass
    str_to_send=ftpp->text().toStdString();
    index=0;
    while((index<15) && (str_to_send[index]!=0))
    {
        charToSend=uchar(str_to_send.at(index));
         qDebug()<<"send char "<<index<<" = "<<str_to_send.at(index);
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000E0+index,uchar(charToSend),3);
        index++;
    }
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000E0+index,0,3);

    //apn
    str_to_send=apn->text().toStdString();
    index=0;
    while((index<31) && (str_to_send[index]!=0))
    {
        charToSend=uchar(str_to_send.at(index));
         qDebug()<<"send char "<<index<<" = "<<str_to_send.at(index);
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x01000090+index,uchar(charToSend),3);
        index++;
    }
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x01000090+index,0,3);

    if(checkAPN->isChecked())
    {
        //apn u
        str_to_send=apnu->text().toStdString();
        index=0;
        while((index<15) && (str_to_send[index]!=0))
        {
            charToSend=uchar(str_to_send.at(index));
            qDebug()<<"send char "<<index<<" = "<<str_to_send.at(index);
            ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000F0+index,uchar(charToSend),3);
            index++;
        }
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000F0+index,0,3);

        //apn p
        str_to_send=apnp->text().toStdString();
        index=0;
        while((index<15) && (str_to_send[index]!=0))
        {
            charToSend=uchar(str_to_send.at(index));
           // qDebug()<<"send char "<<index<<" = "<<charToSend;
            ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000FA+index,uchar(charToSend),3);
            index++;
        }
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000FA+index,0,3);
    }
    else ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x010000F0,0,3);

}

void GprsConfig::sendTimeToCentral()
{   //send order to update central time with gprs time
    if(!serialCom->writeByteAtWithRetry(GPRS_SN,0x26,1,2))
    {
        qDebug()<<"Error sending time to Central";
    }
}

void GprsConfig::resendAllData()
{   // Reset data to send
    bool ok=true;
    for(uint32 pos=0; pos<4; pos++)
    {
        ok= ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x0100000A+pos,0,3);
    }
    fetchGprsInfo();
}

//*********************************
//  Commands section
//*********************************

void GprsConfig::sendCommand()
{   //Send command from command line
    string commandToSend="";
    if(!saveIMEI)  commandToSend=commandLineEdit->text().toStdString();
    else commandToSend="+CGSN";
    bool ok=true;
    int index=0;
    while((index<100) && (commandToSend[index]!=0))
    {
        ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x492+index,uchar(commandToSend.at(index)),3);
        index++;
    }
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x492+index,0,3);
    ok = ok & serialCom->writeByteAtWithRetry(GPRS_SN,0x490,1,3);      //Execute command

    logView->appendPlainText(QString::fromStdString(commandToSend));
}

void GprsConfig::saveCommand()
{   //Add command from line in config file + in list
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

    QSettings *settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
    settingFile->setValue("MODEM/C"+QString::number(preselectedCommands->count()+1).rightJustified(2,'0'),
                    QVariant(commandLineEdit->text()));
    preselectedCommands->addItem(commandLineEdit->text());
}

void GprsConfig::newCommandSelected(int _newCommand)
{   //Put selected command in the command line
    commandLineEdit->clear();
    commandLineEdit->setText(preselectedCommands->currentText());
}

void GprsConfig::updateLogView()
{   // Update the log view (previous commands and responses)
    bool ok = serialCom->readBytesAtWithRetry(GPRS_SN,0x480,1,3);
    if(ok)
    {
        QString stringToAppend;
        uchar downloadedPointer=serialCom->getReceivedData()[0];    //get pointer
        if(firstTimeOpened)
        {
            logPointer=(downloadedPointer+1) & 0x7f;
            firstTimeOpened=false;
        }
        if(downloadedPointer!=logPointer)
        {
            //reading of 128bytes by 2x64B reading
            uchar receivedChars[128];
            for(int i=0; i<128; i++) receivedChars[i]=0xFF; //erase

            ok = ok & serialCom->readBytesAtWithRetry(GPRS_SN,0x400+64,8,3);
            if(ok) for(int i=0; i<64; i++) receivedChars[i+64]=serialCom->getReceivedData()[i];
            ok=ok & serialCom->readBytesAtWithRetry(GPRS_SN,0x400,8,3);
            if(ok) for(int i=0; i<64; i++) receivedChars[i]=serialCom->getReceivedData()[i];

            if(ok)
            {
                QString _IMEI;
                int _IMEI_index=0;
                int index = logPointer; //previous position
                while(index!=downloadedPointer)
                {
                    if(receivedChars[index]==0x0a || receivedChars[index]==0x0d)    //new line
                    {
                        //Show + New Line
                        logView->appendPlainText(stringToAppend);
                        stringToAppend.clear();
                        index = (index + 1) & 0x7f;
                    }
                    else
                    {
                        if(receivedChars[index]>0) stringToAppend.append(receivedChars[index]);
                        if(saveIMEI)
                        {
                            if((receivedChars[index]>0)&&(_IMEI_index<15))
                            {
                                _IMEI.append(receivedChars[index]);
                                _IMEI_index++;
                            }
                        }
                        index = (index + 1) & 0x7f;
                        if((index==downloadedPointer)|| stringToAppend.length()>33)
                        {
                            //Show + New Line
                            logView->appendPlainText(stringToAppend);
                            stringToAppend.clear();
                        }
                    }

                }
                if(saveIMEI)
                {
                    qDebug()<<"IMEI : "<<_IMEI;
                    QString filename_tmp="imei.txt";
                    if(settingFile->contains("IMEI_DIR")){
                       filename_tmp=(settingFile->value("IMEI_DIR").toString());
                    }
                  //  QString filename=QFileDialog::getSaveFileName(this,"Choose Filename",filename_tmp, "*.hex");
                  //  settingFile->setValue("IMEI_DIR",QVariant(filename));
                    QFile *sav_file= new QFile(filename_tmp);
                    if(!sav_file->open(QFile::WriteOnly | QFile::Append)){
                        QMessageBox::critical(this,"Error Opening File","The file can't be opened");
                        return;
                    }
                    QTextStream  sav_fileStream;
                    sav_fileStream.setDevice(sav_file);   //set target for stream
                    sav_fileStream<<_IMEI<<"\r\n";
                    sav_file->close();
                    delete sav_file;

                    _IMEI.clear();
                    saveIMEI=false;
                }
                logPointer=index;
            }
        }
    }
}

 void GprsConfig::force_send()
 {
     serialCom->writeByteAt(GPRS_SN,0x27,1);
 }

void GprsConfig::save_IMEI()
{
    if(!saveIMEI)
    {
        saveIMEI=true;
        sendCommand();
    }
}

void GprsConfig::clr_IMEI()
{
    QString filename_tmp="imei.txt";
    if(settingFile->contains("IMEI_DIR")){
       filename_tmp=(settingFile->value("IMEI_DIR").toString());
    }
    QFile *sav_file= new QFile(filename_tmp);
    if(!sav_file->open(QFile::WriteOnly | QFile::Truncate)){
        QMessageBox::critical(this,"Error Opening File","The file can't be opened");
        return;
    }
    sav_file->close();
    delete sav_file;
}

//***************
// Events
//***************

void GprsConfig::closeEvent(QCloseEvent *event)
 {  //event when window is closed
    commonRefreshTimer->disconnectTimeout();
    setVisible(false);
    QFile(QString(DEFAULT_DIR)+"config.ini").copy(QString(DEFAULT_DIR)+"config.ini.old"); //copy du fichier en local

    settingFile->setValue("WINDOW_POS/GPRS_WIN_X",QVariant(QString::number(this->pos().x())));
    settingFile->setValue("WINDOW_POS/GPRS_WIN_Y",QVariant(QString::number(this->pos().y())));
    emit closingWindow();
 }
