#include "managemodule.h"

//***********************************************************************
// Constructor/Destructor
//***********************************************************************

ManageModule::ManageModule(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, vector<Module> *_moduleList, QWidget *parent) :
    QWidget(parent)
{
    //init values and pointers
    serialCom = _serialPort;
    commonRefreshTimer=_commonRefreshTimer;
    moduleList = _moduleList;
    nbRemainingRetryToFindModule=NB_RETRY_TO_FIND_MODULES;
    advancedUserMode=false;
    autodetectRetry=5;  //init value, will change next
    isStarted=false;
    centralDetected=false;
    left_pannel_type=0;
    pannel_row=0;
    //init graphics
    fillWindow();
    setTableWidget();
    rightPanelWidget=NULL;
    QCoreApplication::processEvents();  //force to refresh window
}

ManageModule::~ManageModule()
{   //destructor
    tableWidget->clear(); //delete pointers
    itemTableForWidget.clear();
}

//***********************************************************************
// GUI
//***********************************************************************

void ManageModule::fillWindow()
{   //add graphices elements in window

    horizontalLayout = new QHBoxLayout; // to split in 2 panels
        groupBoxModules = new QGroupBox("Central SN:");
            verticalModuleLayout=new QVBoxLayout;

                tableWidget= new QTableWidget(0,7); //to show the module grid
                setTableWidget();
                verticalModuleLayout->addWidget(tableWidget);

                //Voltage display
                voltageLayout = new QGridLayout;
                verticalModuleLayout->addLayout(voltageLayout);
                    QLabel *voltageTitle = new QLabel("Voltage :");
                    voltageLayout->addWidget(voltageTitle,0,0);
                    QLabel *ph1Title = new QLabel("Ph1 : ");
                    ph1Title->setAlignment(Qt::AlignRight);
                    voltageLayout->addWidget(ph1Title,0,1);
                    voltagePh1Values = new QLabel("x.x");
                    voltageLayout->addWidget(voltagePh1Values,0,2);
                    QLabel *ph2Title = new QLabel("Ph2 : ");
                    ph2Title->setAlignment(Qt::AlignRight);
                    voltageLayout->addWidget(ph2Title,0,3);
                    voltagePh2Values = new QLabel("x.x");
                    voltageLayout->addWidget(voltagePh2Values,0,4);
                    QLabel *ph3Title = new QLabel("Ph3 : ");
                    ph3Title->setAlignment(Qt::AlignRight);
                    voltageLayout->addWidget(ph3Title,0,5);
                    voltagePh3Values = new QLabel("x.x");
                    voltageLayout->addWidget(voltagePh3Values,0,6);

                buttonLayout=new QHBoxLayout;
                    autoDetectButton=new QPushButton("Search Modules");
                    autoDetectButton->setMaximumWidth(120);
                    connect(autoDetectButton,SIGNAL(clicked()),this,SLOT(initCentralDetection()));
                    buttonLayout->addWidget(autoDetectButton);
                    QSpacerItem *spacerButton1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
                    buttonLayout->addItem(spacerButton1);
                    resetSystemButton=new QPushButton("Reset system");
                    resetSystemButton->setMaximumWidth(120);
                    resetSystemButton->setDisabled(true);
                    connect(resetSystemButton,SIGNAL(clicked()),this,SLOT(resetSystem()));
                    buttonLayout->addWidget(resetSystemButton);
                    QSpacerItem *spacerButton2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
                    buttonLayout->addItem(spacerButton2);
                    //startConfigButton=new QPushButton("Start Config");        //TODO : un-comment to do startConfig
                    //startConfigButton->setDisabled(true);
                    //startConfigButton->setMaximumWidth(80);
                    //buttonLayout->addWidget(startConfigButton);
                    //QSpacerItem *spacerButton = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
                    //buttonLayout->addItem(spacerButton);
                    dataButton=new QPushButton("Data");
                    dataButton->setMaximumWidth(120);
                    dataButton->setDisabled(true);
                    buttonLayout->addWidget(dataButton);
                    connect(dataButton,SIGNAL(clicked()),this,SLOT(dataAcquisitionAsked()));
                verticalModuleLayout->addLayout(buttonLayout);
        groupBoxModules->setLayout(verticalModuleLayout);
        groupBoxModules->setFixedWidth(490);
        groupBoxModules->setMinimumHeight(400);
    horizontalLayout->addWidget(groupBoxModules);

    //Right panel : QStackedLayout to change side panel
    rightPanelLayout= new QStackedLayout;
    rightPanelLayout->setStackingMode(QStackedLayout::StackOne);
        emptyRightPanelWidget = new QWidget;
        emptyRightPanelWidget->setMinimumWidth(550);
        //rightPanelLayout->set
        rightPanelLayout->addWidget(emptyRightPanelWidget);
        rightPanelWidget=NULL;
    rightPanelLayout->setCurrentIndex(0);
    horizontalLayout->addLayout(rightPanelLayout);
    setLayout(horizontalLayout);
}

void ManageModule::setTableWidget()
{   //Add columns to tableWidget
    tableWidget->setVisible(true);
    tableWidget->verticalHeader()->setVisible(false);
    moduleSNTableItem = new QTableWidgetItem("Module");
    tableWidget->setHorizontalHeaderItem(0,moduleSNTableItem);
    channel1TableItem = new QTableWidgetItem("ch1");
    tableWidget->setHorizontalHeaderItem(1,channel1TableItem);
    channel2TableItem = new QTableWidgetItem("ch2");
    tableWidget->setHorizontalHeaderItem(2,channel2TableItem);
    channel3TableItem = new QTableWidgetItem("ch3");
    tableWidget->setHorizontalHeaderItem(3,channel3TableItem);
    channel4TableItem = new QTableWidgetItem("ch4");
    tableWidget->setHorizontalHeaderItem(4,channel4TableItem);
    channel5TableItem = new QTableWidgetItem("ch5");
    tableWidget->setHorizontalHeaderItem(5,channel5TableItem);
    channel6TableItem = new QTableWidgetItem("ch6");
    tableWidget->setHorizontalHeaderItem(6,channel6TableItem);
    tableWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
    tableWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
    tableWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    tableWidget->setAutoScroll(false);
    tableWidget->horizontalHeader()->setResizeMode( QHeaderView::Stretch) ;
    connect(tableWidget,SIGNAL(cellClicked(int,int)),this,SLOT(clickOnACell(int,int)));
}

//***********************************************************************
// Manage rightPanel
//***********************************************************************

void ManageModule::clickOnACell(int row, int column)
{ // According to type of module clicked, show the configuration panel

    //stop other refresh
    commonRefreshTimer->disconnectTimeout();

    //To be sure : set to empty panel and remove panel
    rightPanelLayout->setCurrentIndex(0);
    rightPanelLayout->removeWidget(rightPanelWidget);
    if(rightPanelWidget!=NULL);
    delete rightPanelWidget;
    //instantiate panel widget according to type
    pannel_row=row;
    switch(moduleList->at(row).typeOfModule)
    {
    //debug
    case wirelessModule :
        left_pannel_type=wireless;
        rightPanelWidget = new WirelessModule(serialCom, commonRefreshTimer, &moduleList->at(row));
        connect(rightPanelWidget, SIGNAL(backToMainWindow()),this,SLOT(closeRightPanel()));
        connect(this, SIGNAL(restart_timer_right_panel()),rightPanelWidget,SLOT(startTimerAgain()));
        break;

    case powerModule :

    case powerModule2 :
        left_pannel_type=powerm;
        rightPanelWidget = new PowerModule(serialCom, commonRefreshTimer,&moduleList->at(row),column,this,advancedUserMode);
        connect(rightPanelWidget, SIGNAL(backToMainWindow()),this,SLOT(closeRightPanel()));
        connect(this, SIGNAL(restart_timer_right_panel()),rightPanelWidget,SLOT(startTimerAgain()));
        connect(this, SIGNAL(setadvusermode()),rightPanelWidget,SLOT(setadvance()));
        break;

    default : // temperature or pressure module
        left_pannel_type=presst;
        rightPanelWidget = new PressTempModule(serialCom, commonRefreshTimer,&moduleList->at(row),this);
        connect(rightPanelWidget, SIGNAL(backToMainWindow()),this,SLOT(closeRightPanel()));
        connect(this, SIGNAL(restart_timer_right_panel()),rightPanelWidget,SLOT(startTimerAgain()));
    }

    //show side panel
    rightPanelLayout->addWidget(rightPanelWidget);
    rightPanelLayout->setCurrentIndex(1); //set rightPanel

    //disable the main frame
    groupBoxModules->setDisabled(true);
    blurAllModules(true);
}

void ManageModule::dataAcquisitionAsked()
{   // button Data slot, to open DataAcquisition panel

    commonRefreshTimer->disconnectTimeout();

    // In order to update central infos
    if(serialCom->getCentralSN()==ALL_CENTRAL) serialCom->centralDetection();

    //To be sure : set to empty panel and remove panel
    rightPanelLayout->setCurrentIndex(0);
    rightPanelLayout->removeWidget(rightPanelWidget);
    left_pannel_type=dataacq;
    pannel_row=0;
    // Create a new DataAcquisition and show it
    if(rightPanelWidget!=NULL);
    delete rightPanelWidget;
    rightPanelWidget = new DataAcquisition(serialCom, commonRefreshTimer,*moduleList);
    rightPanelLayout->addWidget(rightPanelWidget);
    rightPanelLayout->setCurrentIndex(1); //set rightPanelWidget
    rightPanelWidget->setVisible(true);

    //disable the main frame
    groupBoxModules->setDisabled(true);
    blurAllModules(true);
    connect(rightPanelWidget, SIGNAL(backToMainWindow()),this,SLOT(closeRightPanel()));
    connect(this,SIGNAL(restart_timer_right_panel()),rightPanelWidget,SLOT(startTimerAgain()));
}

void ManageModule::closeRightPanel()
{   // close the side panel and return in modules grid

    if(rightPanelLayout->count()>1) //Prevent closing if there is no side panel
    {
        pannel_row=0;
        //get the last size
        emptyRightPanelWidget->resize(emptyRightPanelWidget->width(),emptyRightPanelWidget->height());
        rightPanelLayout->setCurrentIndex(0); //set to emptyRightPanel

        //delete rightPanel to replace
        left_pannel_type=0;
        rightPanelLayout->removeWidget(rightPanelWidget);
      /*  delete rightPanelWidget;
        rightPanelWidget=NULL;*/
        search_more();
       // if(nbRemainingRetryToFindModule>0) //module detection not complete
       //     initCentralDetection();
      /*  else
        {
            commonRefreshTimer->connectTimeout(this);
            commonRefreshTimer->startSmallInterval();
        }*/
        commonRefreshTimer->connectTimeout(this);
        commonRefreshTimer->startSmallInterval();

        groupBoxModules->setEnabled(true);
        blurAllModules(false);
    }
}

//***********************************************************************
// Interaction with other windows
//***********************************************************************

void ManageModule::switchToAdvancedUserMode() { //after entering password
    advancedUserMode=true;
}

void ManageModule::enableWindowAndStartTimerAgain()
{   //enable the window and restart refresh after closing a configuration window
    setEnabled(true);
//    serialCom->deleteErrorCom();
    if(!left_pannel_type) //si pas de left panel
    {
        commonRefreshTimer->connectTimeout(this); //on lance le timer du left panel
        commonRefreshTimer->startSmallInterval();
    }

    if(advancedUserMode) {
        switch(left_pannel_type)
        {
        case powerm:
            emit setadvusermode();
            emit restart_timer_right_panel();
            break;
        default:
            emit restart_timer_right_panel();
        }
    }
    else emit restart_timer_right_panel();

}

void ManageModule::connect_timer(){
    //serialCom->deleteErrorCom();
    commonRefreshTimer->connectTimeout(this);
}

void ManageModule::disableWindow()
{   //disable the window after opening a configuration window
    //closeRightPanel();
    commonRefreshTimer->disconnectTimeout();
    setDisabled(true);
}

//***********************************************************************
// Module/central Detection
//***********************************************************************

void ManageModule::initCentralDetection()
{
    centralDetected=false;

    //clear existing list
    moduleList->clear();
    tableWidget->clearContents();
    tableWidget->setRowCount(0);
    itemTableForWidget.clear();
    emit eraseCentralInTitle();

    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval();
}

bool ManageModule::initModuleDetection()
{
    nbRemainingRetryToFindModule=5; //this is not the real value (3), will be applied when first module found
    // clear lists of last modules founded
    moduleList->clear();
    tableWidget->clearContents();
    tableWidget->setRowCount(0);
    itemTableForWidget.clear();

    // ask to all modules to listen the autodetection request
    bool ok=serialCom->writeByteAt(ALL_MODULE,0x155,0);
    waitFor(10);
    ok=serialCom->writeByteAt(ALL_MODULE,0x155,0);    //done again to be sure
    waitFor(10);

    // ask to all centrals to initialize the module detection
    ok = serialCom->writeByteAt(ALL_CENTRAL,0x160,0);

    autodetectRetry=3; //max of retry
    endOfFindingModules=false;

    return ok;
}

bool ManageModule::search_more()
{
    bool ok=true;
  /* ok=serialCom->writeByteAt(ALL_MODULE,0x155,0);
    waitFor(10);
    ok=serialCom->writeByteAt(ALL_MODULE,0x155,0);    //done again to be sure
    waitFor(10);
    // ask to all centrals to initialize the module detection
    ok = serialCom->writeByteAt(ALL_CENTRAL,0x160,0);*/
    return ok;
}


int ManageModule::fetchModuleDetectionResult()
{   // read number of module detected and serial number of these modules
    bool ok =false;
    uchar nbOfModulesBytesFound=0;
    // reading nb of modules detected
    ok = serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x160,1,3);
    if(ok)
    {
        nbOfModulesBytesFound = serialCom->getReceivedData()[1];
        if(nbOfModulesBytesFound==0)    //No new modules
            return 0;
        if(serialCom->getReceivedData()[0]>0)
        {
            nbOfModulesBytesFound=0; // Not yet updated
            return 0;
        }
        //else qDebug()<<"Value is updated, "<<nbOfModulesBytesFound<<" bytes found (nb modules x4).";

        // New module(s) detected, looking for serial number
        if((nbOfModulesBytesFound>0) && ((nbOfModulesBytesFound & 0x3)==0))
        {
            if(nbOfModulesBytesFound>64) nbOfModulesBytesFound=64;
            // Not more than 16 modules at once

            //Read the list of SN found :
            ok=serialCom->readBytesAtWithRetry(serialCom->getCentralSN(), 0x400,8,3);
            if(ok)
            {
                // Copy to not overwrite by a read/write operation
                uchar receivedModulesSN [nbOfModulesBytesFound];
                for(uchar i=0; i<nbOfModulesBytesFound; i++) receivedModulesSN[i]= serialCom->getReceivedData()[i];

                // Take modules SN from the data received
                for(int i=0; i<(nbOfModulesBytesFound/4);i++)
                {
                    uint32 moduleSN=peekUint32FromUcharTabLittleEndian(receivedModulesSN,i*4);
                   // qDebug()<<"module found : "<<QString::number(moduleSN,16);
                    addFoundModuleOnList(moduleSN);

                    //prevent future answer of this module :
                    if(!serialCom->writeByteDontCheckResponse(moduleSN,0x155,1))
                    {
                        qDebug()<<"error on preventing answer";
                    }
                    waitFor(20);
                }
            }else qDebug()<<"error 0x400";
        }
    }
    else    //not ok
    {
        qDebug()<<"No response from the central";
    }

    if(!ok) return -1;
    else return nbOfModulesBytesFound/4;    //=nb of modules found
}

//***********************************************************************
// Timer slot & Update Event
//***********************************************************************

void ManageModule::timerSlot()
{
    // in order to avoid multiple request during a timeout :
    commonRefreshTimer->stop();
    if(!centralDetected)
    {

        if(!serialCom->centralDetection())  //Central not found
        {
            if(--autodetectRetry>0)
            {
                //serialCom->deleteErrorCom();
                commonRefreshTimer->startSmallInterval();
           }
            else
            {
            centralDetected=true;
            qDebug()<<"error ad";
            emit ad_ok();
            }
            //retry next time
        }
        else  //Central found
        {
            centralDetected=true;
            groupBoxModules->setTitle("Central SN: "+QString::number(serialCom->getCentralSN(),16).rightJustified(8,'0'));
            //Central found, search for modules
            initModuleDetection();
            commonRefreshTimer->startSmallInterval();
        }

    }
    else
    {

        refreshTableValues();

        if(nbRemainingRetryToFindModule>0)
        {
            commonRefreshTimer->startSmallInterval();  //less time to search new modules
        }
        else
        {
         //   fetchModuleDetectionResult();
            commonRefreshTimer->startInterval(); // longer delay once module detection ending
          // search_more();
        }


    }


    //qDebug()<<"END of refresh : time = "<<QTime::currentTime().toString("hh:mm:ss:zzz");
    //qDebug(" ");
}

void ManageModule::refreshTableValues()
{
    //qDebug()<<"ManageModule::refreshTableEvent() !";

    if(serialCom->isOpen())
    {
        //serialCom->deleteErrorCom();    //retry in case of error

        // autodetection needed
        if(autodetectRetry>0 && nbRemainingRetryToFindModule>0)
        {
            //Check if new modules
            int returnFromModuleDetection=fetchModuleDetectionResult();
            if(returnFromModuleDetection==0) //No module
            {
                qDebug("No module found");
                nbRemainingRetryToFindModule--; //no module found = decrement
                qDebug()<<"nbRemainingRetryToFindModule="<<nbRemainingRetryToFindModule;
            }
            else if(returnFromModuleDetection<0)
            {
                qDebug("Error com during module detection fetching");
            }
            else
            {
                qDebug("New module found");
                //new modules found = restart counter
                nbRemainingRetryToFindModule=NB_RETRY_TO_FIND_MODULES;
            }
        }

        // Add new modules and update values
       updateTableLines();
        //Display voltage from phases
        updateVoltageValues();

        if(autodetectRetry>0 && nbRemainingRetryToFindModule>0) //Still in module detection
        {
            if(moduleList->size()==0)
            {
                qDebug()<<"No module in the widget";
                serialCom->writeByteDontCheckResponse(ALL_MODULE,0x155,0); //to unlock them all
            }
            //Request new module detection :
            if(!serialCom->writeByteAt(ALL_CENTRAL,0x160,1))
            {
                qDebug()<<"error in retrying module request !";
            }
            autodetectRetry--;
        }
        else
        {
            if(!endOfFindingModules)    // for first one only
            {
                qDebug()<<"Finding new modules terminates";
                dataButton->setEnabled(true);
                endOfFindingModules=true;
                nbRemainingRetryToFindModule=0;
                emit ad_ok();
            }
        }
    }   //else = wait for com port
}

//***********************************************************************
// Print/Update the table widget
//***********************************************************************

void ManageModule::changeTextOnCell(int _lineNb, int _channel, QString _newCellText)
{   //update the text field of a tableWidget cell

    if(_lineNb<(int)itemTableForWidget.size())        //Security
    {
        itemTableForWidget[_lineNb][_channel]->setTextColor(Qt::black); // to reprint after not responding
        itemTableForWidget[_lineNb][_channel]->setText(_newCellText);
    }
    else qDebug()<<"/!\\ Error addTextOnCell";  //May never happen
}

void ManageModule::fetchModuleValues(int lineNb)
{
    // This method update the line values
    bool ok=true;
    uchar mulCoeff;
    uchar divCoeff;
    switch(moduleList->at(lineNb).typeOfModule)
    {
        case wirelessModule2 :
        case wirelessModule :
            changeTextOnCell(lineNb,1,"Wireless");
            break;

        case pressureModule :
            ok=serialCom->readBytesAtWithRetry(moduleList->at(lineNb).serialNb,0x1E0,2,2);
            if((lineNb!=-1) && ok)
            {
                int16 value;
                moduleAnswer(lineNb,true);  //module OK, put in black
                //display values
                for(int currentChannel=0; currentChannel<4; currentChannel++)
                {
                    value=peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),2*currentChannel);
                    if(currentChannel<2) changeTextOnCell(lineNb,currentChannel+1,QString::number((value+0.5)/255.0,'f',1));    //temperature
                    else changeTextOnCell(lineNb,currentChannel+1,QString::number((value+0.5)/60.0,'f',1)); //pressure
                }
                //display Pulses
                value = peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),8);
                changeTextOnCell(lineNb,5,QString::number(value));
                value = peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),10);
                changeTextOnCell(lineNb,6,QString::number(value));

                //TODO : VoieConfiguree ?
            }
            else if(!ok && lineNb<tableWidget->rowCount()) moduleAnswer(lineNb,false);  // No response, put the cell in red

            break;

        case temperatureModule :
            ok=serialCom->readBytesAtWithRetry(moduleList->at(lineNb).serialNb,0x1E0,2,2);
            if((lineNb!=-1) && ok)
            {
                moduleAnswer(lineNb,true);  //module OK, put in black
                //display temperature (coeff 255)
                int16 value;
                for(int currentChannel=0; currentChannel<3; currentChannel++)
                {
                    value=peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),2*currentChannel);
                    changeTextOnCell(lineNb,currentChannel+1,QString::number((value+0.5)/255.0,'f',1));
                }
                //display temperature (coeff 100)
                value = peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),6);
                changeTextOnCell(lineNb,4,QString::number((value+0.5)/100.0,'f',1));
                //display hygro
                value = peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),8);
                changeTextOnCell(lineNb,5,QString::number((value+0.5)/100.0,'f',1));
                //display pulses
                value = peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),10);
                changeTextOnCell(lineNb,6,QString::number(value));

                //TODO : VoieConfiguree ?
            }
            else if(!ok && lineNb<tableWidget->rowCount()) moduleAnswer(lineNb,false);  // No response, put the cell in red

            break;

        default:   //powerModule
            ok=serialCom->readBytesAtWithRetry(moduleList->at(lineNb).serialNb,0x1E0,2,2);
            if(!ok) qDebug()<<"error 0x1e0";
            if((lineNb!=-1) && ok)
            {
                moduleAnswer(lineNb,true);  //module OK, put in black
                for(int currentChannel=0; currentChannel<6; currentChannel++)
                {
                    int16 powerValue=peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),2*currentChannel);
                    //not uint16 because of signed values

                    mulCoeff=moduleList->at(lineNb).mulCoeff[currentChannel];
                    divCoeff=moduleList->at(lineNb).divCoeff[currentChannel];
                    if(divCoeff==0) divCoeff=1;   //security
                    powerValue= int16(powerValue*mulCoeff/divCoeff);

                    changeTextOnCell(lineNb,currentChannel+1,QString::number(powerValue));

                    //TODO : VoieConfiguree ?
                }
            }
            else if(!ok && lineNb<tableWidget->rowCount()) moduleAnswer(lineNb,false);  // No response, put the cell in red
        }
}

bool ManageModule::addFoundModuleOnList(uint32 _moduleSN)
{
    // Query informations about module and add it in the module list.
    for(uint i=0; i<moduleList->size(); i++) //prevent double entry
        if(moduleList->at(i).serialNb==_moduleSN) return false;
    if(_moduleSN==0x00FD0000)
        return false;

    //reading @1000080  //nbLines, type and coeffs
    if(!serialCom->readBytesAtWithRetry(_moduleSN,0x01000080,4,3))
    {
        qDebug()<<"Error on updating a module !";           //TODO : ?
        return false;
    }
    else    //read OK
    {
        int nbOfLines=1;    //default nbOfLines
        if(serialCom->getReceivedData()[12]==250) // module type wireless
        {
            nbOfLines=serialCom->getReceivedData()[13];
            if(nbOfLines>8) nbOfLines=8;    //to limit nOfLines to 8
        }

        Module moduleFound;
        moduleFound.serialNb=_moduleSN;
        moduleFound.version=serialCom->getReceivedData()[0x1f];
        qDebug()<<"type:"<<serialCom->getReceivedData()[12];
        switch(serialCom->getReceivedData()[12])    //type of module
        {
        case 0:
        case 255:
            moduleFound.typeOfModule=powerModule;
            break;
        case 249:
            moduleFound.typeOfModule=wirelessModule2;
            break;
        case 250 :
            moduleFound.typeOfModule=wirelessModule;
            break;
        case 251:
            moduleFound.typeOfModule=pulsesModule;
            break;
        case 252 :
            moduleFound.typeOfModule=pressureModule;
            break;
        case 253 :
            moduleFound.typeOfModule=temperatureModule;
            break;
        case 140:
            moduleFound.typeOfModule=powerModule2;
            break;
        default :
            moduleFound.typeOfModule=powerModule;
        }
        moduleFound.memoryAvailable=serialCom->getReceivedData()[14];
        moduleFound.nbOfLines=nbOfLines;

        //fetching coefficient values :
        for(int channelIndex=0;channelIndex<6;channelIndex++)
        {
            moduleFound.mulCoeff[channelIndex]=serialCom->getReceivedData()[2*channelIndex];
            moduleFound.divCoeff[channelIndex]=serialCom->getReceivedData()[(2*channelIndex)+1];
        }

        // Place on the list and sort
        moduleList->push_back(moduleFound);
        sort(moduleList->begin(),moduleList->end(),compareModules());
        return true;
    }
}

void ManageModule::moduleAnswer(int _moduleNb, bool _answer)
{   // put module SN in red if not _answer, or in black
    if(_answer) //answer, may be black.
    {
        itemTableForWidget[_moduleNb][0]->setTextColor(Qt::black);
    }
    else    //no answer, may be red.
    {
        itemTableForWidget[_moduleNb][0]->setTextColor(Qt::red);
    }
}

void ManageModule::blurAllModules(bool blur)
{   // put module text in gray when disabling tableWidget
    for(uint r=0; r<itemTableForWidget.size(); r++)
    {
        for(uint c=0; c<itemTableForWidget[r].size(); c++)
        {
            if(blur) itemTableForWidget[r][c]->setTextColor(Qt::darkGray);
            else itemTableForWidget[r][c]->setTextColor(Qt::black);
        }
    }
}

void ManageModule::updateTableLines()
{
    //Update the tableWidget and itemTableForWidget from the modules list
    if(tableWidget->rowCount()<(int)moduleList->size())  // New modules
    {   // new modules
        tableWidget->setRowCount(moduleList->size()); //Add lines on the tableWidget

        //Add lines to the itemTable
        if(itemTableForWidget.size()<moduleList->size())
        {
            for(uint16 lineIndex=itemTableForWidget.size(); lineIndex<moduleList->size(); lineIndex++)    //Add only new lines
            {
                vector<QTableWidgetItem *> newLine;
                for(uint16 columnIndex=0; columnIndex<7; columnIndex++)
                {
                    QTableWidgetItem *newItem = new QTableWidgetItem;
                    newItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
                    newItem->setFlags(newItem->flags() & ~Qt::ItemIsEditable & ~Qt::ItemIsSelectable);
                    newLine.push_back(newItem);
                }
                //add the line to the item table
                itemTableForWidget.push_back(newLine);

                //Affect new lines to the tableWidget
                for(uint16 columnIndex=0; columnIndex<7; columnIndex++)
                {
                    tableWidget->setItem(lineIndex,columnIndex,itemTableForWidget[lineIndex][columnIndex]);
                }
            }
        }

        //Fill with SN and affect each one in the tableWidget
        for(uint16 lineIndex=0; lineIndex<itemTableForWidget.size(); lineIndex++)
        {
            itemTableForWidget[lineIndex][0]->setText(QString::number(moduleList->at(lineIndex).serialNb,16).toUpper().rightJustified(8,'0'));
            tableWidget->resizeRowsToContents();
            tableWidget->resizeColumnToContents(0);

            fetchModuleValues(lineIndex);   //update values
        }
    }
    else    //No new modules
    {
        for(uint16 lineIndex=0; lineIndex<itemTableForWidget.size(); lineIndex++)
        {
            fetchModuleValues(lineIndex);   //update values
        }
    }
}

void ManageModule::updateVoltageValues()
{
    // Update display of phases voltage
    if(serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x100,1,3))
    {
        float voltage1 =sqrt(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),0) / 5859.0 / 2.0) *256.0;
        float voltage2 =sqrt(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),2) / 5859.0 / 2.0) *256.0;
        float voltage3 =sqrt(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),4) / 5859.0 / 2.0) *256.0;
        if(voltage1<10) voltage1 = 0;
        if(voltage2<10) voltage2 = 0;
        if(voltage3<10) voltage3 = 0;
        voltagePh1Values->setText(QString::number(voltage1,'f',1)+" V");
        voltagePh2Values->setText(QString::number(voltage2,'f',1)+" V");
        voltagePh3Values->setText(QString::number(voltage3,'f',1)+" V");
    }
    else    //error reading voltage values
    {
        voltagePh1Values->setText("x.x");
        voltagePh2Values->setText("x.x");
        voltagePh3Values->setText("x.x");
    }
}

//***********************************************************************
// Reset system
//***********************************************************************

void ManageModule::resetSystem()
{
    //Advanced mode method, to reset the system into default configuration, after a system rent
    int response=QMessageBox::warning(this,"Reset configuration of whole system", "Are you sure to reset the whole configuration of the system ?<br>All modules parameters will be erased.",QMessageBox::Ok||QMessageBox::Cancel,QMessageBox::Cancel);
    if(response == QMessageBox::Ok)
    {

        //TODO : stop the refresh

        //TODO : central name

        //TODO : boucle modules
            //if(module==pressure)
                //TODO : set names (voie1 to voie6)
            //if(module==power)
                //TODO : set names (voie1 to voie6)
                //TODO : set Coeffs
                //TODO : set phases 1,2,3...
            //if radio module
                //TODO : erase SN ?
                //TODO : put 0 sensors ?

        //TODO : erase campaign
    }
}
