#include "powermodule.h"
extern    bool refreshIsPaused;

//************************
// Constructor and destructor

PowerModule::PowerModule(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, Module *_powModule, int _channel, QWidget *parent, bool _advancedUserMode) :
    QWidget(parent)
{
    //Constructor
    waveformwindow=NULL;
    serialCom=_serialPort;
    commonRefreshTimer=_commonRefreshTimer;
    powModule=_powModule;
    advancedUserMode=_advancedUserMode;
    if(_channel<1||_channel>6) currentChannel=1; //in case of error
    else currentChannel=_channel;
    phase = -1;
    updating_phase=false;
    //Place GUI
    fillWindow();

    //get coeff, name and phase
    channelInfoUpdated=updateChannelInfo();

    //init timer
    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval();   //small delay before first refresh
}

PowerModule::~PowerModule()
{   //Destructor
    //TODO : delete ?
    if(waveformwindow!=NULL)
    delete waveformwindow;
    waveformwindow=NULL;
}

void PowerModule::fillWindow()
{
    //Place GUI elements
    globalLayout= new QVBoxLayout;
    setLayout(globalLayout);
        groupBoxPowerModule = new QGroupBox("Power Module");
        globalLayout->addWidget(groupBoxPowerModule);
            verticalLayout= new QVBoxLayout;
            groupBoxPowerModule->setLayout(verticalLayout);

                // Module SN, channel and name
                QGridLayout *moduleChannelLayout = new QGridLayout;
                    QLabel *moduleSNLabel = new QLabel("Module SN :");
                    moduleChannelLayout->addWidget(moduleSNLabel,0,0);
                    moduleSN = new QLabel(QString::number(powModule->serialNb,16).rightJustified(8,'0'));
                    moduleChannelLayout->addWidget(moduleSN,0,1);

                    QSpacerItem *spacer1=new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
                    moduleChannelLayout->addItem(spacer1,1,0);

                    QLabel *channelLabel = new QLabel("Channel :");
                    moduleChannelLayout->addWidget(channelLabel,2,0);
               /*  channelComboBox = new QComboBox;
                    for(int i=1; i<7;i++)
                        channelComboBox->addItem(QString::number(i));
                    channelComboBox->setCurrentIndex(currentChannel-1);
                    channelComboBox->setMaximumWidth(60);*/

                    chan_b_group=new QButtonGroup;
                    chan1 = new QRadioButton("1");
                    chan2 = new QRadioButton("2");
                    chan3 = new QRadioButton("3");
                    chan4 = new QRadioButton("4");
                    chan5 = new QRadioButton("5");
                    chan6 = new QRadioButton("6");
                    chan_b_group->addButton(chan1);
                    chan_b_group->addButton(chan2);
                    chan_b_group->addButton(chan3);
                    chan_b_group->addButton(chan4);
                    chan_b_group->addButton(chan5);
                    chan_b_group->addButton(chan6);
                    chan_b_group->setId(chan1,1);
                    chan_b_group->setId(chan2,2);
                    chan_b_group->setId(chan3,3);
                    chan_b_group->setId(chan4,4);
                    chan_b_group->setId(chan5,5);
                    chan_b_group->setId(chan6,6);
                    switch(currentChannel){
                    case 1:
                        chan1->setChecked(true);
                        break;
                    case 2:
                        chan2->setChecked(true);
                        break;
                    case 3:
                        chan3->setChecked(true);
                        break;
                    case 4:
                        chan4->setChecked(true);
                        break;
                    case 5:
                        chan5->setChecked(true);
                        break;
                    case 6:
                        chan6->setChecked(true);
                        break;
                    }
                    connect(chan_b_group,SIGNAL(buttonClicked(int)),this,SLOT(channelChanged(int)));
                    QHBoxLayout *layouth_chan = new QHBoxLayout;
                    layouth_chan->addWidget(chan1);
                    layouth_chan->addWidget(chan2);
                    layouth_chan->addWidget(chan3);
                    layouth_chan->addWidget(chan4);
                    layouth_chan->addWidget(chan5);
                    layouth_chan->addWidget(chan6);

                    moduleChannelLayout->addLayout(layouth_chan,2,1);
                    //moduleChannelLayout->addWidget(channelComboBox,2,1);
                    QLabel *nameLabel = new QLabel("Name :");
                    moduleChannelLayout->addWidget(nameLabel,3,0);
                    channelNameEdit = new QLineEdit("xxx");
                    channelNameEdit->setMaxLength(16);
                    moduleChannelLayout->addWidget(channelNameEdit,3,1);
                verticalLayout->addLayout(moduleChannelLayout);

                QSpacerItem *spacer2=new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
                verticalLayout->addItem(spacer2);

                // Frame with values (power, current...)
                valueFrame = new QFrame;
                    QGridLayout *valueGridLayout = new QGridLayout;
                    valueFrame->setLayout(valueGridLayout);
                    valueFrame->setFrameShape(QFrame::StyledPanel);
                    powerValue = new QLabel("xxx <b>Watts</b>");
                    powerValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(powerValue,0,0);
                    apparentPowerValue = new QLabel("xxx <b>VA</b>");
                    apparentPowerValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(apparentPowerValue,1,0);
                    reactivePowerValue = new QLabel("xxx <b>VAR</b>");
                    reactivePowerValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(reactivePowerValue,2,0);
                    voltageValue = new QLabel("xxx <b>Volts</b>");
                    voltageValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(voltageValue,0,1);
                    currentValue = new QLabel("xxx <b>A</b>");
                    currentValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(currentValue,1,1);
                    energyValue = new QLabel("xxx <b>Wh</b>");
                    energyValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(energyValue,2,1);
                    reactiveEnergyValue = new QLabel("xxx <b>Wh</b>");
                    reactiveEnergyValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(reactiveEnergyValue,3,1);
                    powerFactorValue = new QLabel("<b>PF:</b> xxx");
                    powerFactorValue->setAlignment(Qt::AlignRight);
                    valueGridLayout->addWidget(powerFactorValue,3,0);
                verticalLayout->addWidget(valueFrame);

                QSpacerItem *spacer3=new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
                verticalLayout->addItem(spacer3);

                //Coefficients
                QHBoxLayout *coeffLayout = new QHBoxLayout;
                    QLabel *coeffLabel = new QLabel("Coeffs :");
                    coeffLayout->addWidget(coeffLabel);
                    mulCoeff = new QSpinBox;
                    mulCoeff->setMinimum(1);
                    mulCoeff->setEnabled(advancedUserMode);
                    coeffLayout->addWidget(mulCoeff);
                    QLabel *slashLabel = new QLabel("/");
                    slashLabel->setMaximumWidth(15);
                    coeffLayout->addWidget(slashLabel);
                    divCoeff = new QSpinBox;
                    divCoeff->setMinimum(1);
                    divCoeff->setEnabled(advancedUserMode);
                    coeffLayout->addWidget(divCoeff);
                    coeffLayout->setAlignment(Qt::AlignLeft);
                verticalLayout->addLayout(coeffLayout);

                // Phase / invert
                QHBoxLayout *phaseInvertLayout = new QHBoxLayout;

                    phaseGroupBox = new QGroupBox("Phase");
                    QVBoxLayout *phaseGroupBoxL = new QVBoxLayout;
                    QHBoxLayout *radioButtonPhaseLayout = new QHBoxLayout;
                    QHBoxLayout *invertLayout = new QHBoxLayout;
                        phase1 = new QRadioButton("1");
                        radioButtonPhaseLayout->addWidget(phase1);
                        connect(phase1,SIGNAL(clicked()),this,SLOT(phaseChanged()));
                        phase2 = new QRadioButton("2");
                        radioButtonPhaseLayout->addWidget(phase2);
                        connect(phase2,SIGNAL(clicked()),this,SLOT(phaseChanged()));
                        phase3 = new QRadioButton("3");
                        radioButtonPhaseLayout->addWidget(phase3);
                        connect(phase3,SIGNAL(clicked()),this,SLOT(phaseChanged()));
                        not_configured = new QRadioButton("Not Configured");
                        radioButtonPhaseLayout->addWidget(not_configured);
                        not_configured->setEnabled(false);
                       // connect(not_configured,SIGNAL(clicked()),this,SLOT(phaseChanged()));
                        phase_auto = new QCheckBox("Auto");
                        radioButtonPhaseLayout->addWidget(phase_auto);
                        connect(phase_auto,SIGNAL(clicked()),this,SLOT(phaseChanged()));

                        if(powModule->typeOfModule==powerModule2)
                        {
                            if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x0100009E,1,3)) //pas de phase auto
                            {
                                if(!serialCom->getReceivedData()[0]) phase_auto->setEnabled(true);
                                else
                                {
                                    phase_auto->setChecked(false);
                                    phase_auto->setEnabled(false);
                                }
                            }
                        }
                        else
                        {
                            phase_auto->setChecked(false);
                            phase_auto->setEnabled(false);
                        }
                        invertBox = new QCheckBox("Invert");
                        invertLayout->addWidget(invertBox);
                        radioButtonPhaseLayout->setAlignment(Qt::AlignLeft);
                        phaseGroupBoxL->addLayout(radioButtonPhaseLayout);
                        phaseGroupBoxL->addLayout(invertLayout);
                    phaseGroupBox->setLayout(phaseGroupBoxL);
                    //phaseGroupBox->setLayout(phaseInvertLayout);
                    phaseInvertLayout->addWidget(phaseGroupBox);


                verticalLayout->addLayout(phaseInvertLayout);

                QSpacerItem *spacer4=new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
                verticalLayout->addItem(spacer4);

                // Buttons (back, clear, waveform)
                buttonLayout = new QHBoxLayout;
                    returnButton = new QPushButton("<= Back");
                    buttonLayout->addWidget(returnButton);
                    connect(returnButton,SIGNAL(clicked()),this,SLOT(backToManageModule()));
                    clearKwhButton = new QPushButton("Clear kWh");
                    buttonLayout->addWidget(clearKwhButton);
                    connect(clearKwhButton,SIGNAL(clicked()),this,SLOT(clearKwhClicked()));
                    waveformButton = new QPushButton("WaveForm");
                    //waveformButton->setEnabled(false);
                    buttonLayout->addWidget(waveformButton);
                    connect(waveformButton,SIGNAL(clicked()),SLOT(waveformClicked()));
                verticalLayout->addLayout(buttonLayout);
}

//*****************
// Private Slots

void PowerModule::backToManageModule()
{
    //button back clicked, stop refreshing window and return to module grid
    commonRefreshTimer->disconnect();
    emit backToMainWindow();    //Signal to restart the main window (refresh Values)
}

void PowerModule::startTimerAgain()
{
    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval(); // insert a small delay before fetching values
}

void PowerModule::setadvance()
{
    if(!advancedUserMode)
    {
    advancedUserMode=true;
    mulCoeff->setEnabled(advancedUserMode);
    divCoeff->setEnabled(advancedUserMode);
    connect(mulCoeff,SIGNAL(valueChanged(int)),this,SLOT(mulCoeffChanged(int)));
    connect(divCoeff,SIGNAL(valueChanged(int)),this,SLOT(divCoeffChanged(int)));
    }
}
//****************
// Update window values

bool PowerModule::updateChannelInfo()
{   //update Coefficients, name and phase

    //Disconnect (avoid signal process when update values)
    //channelComboBox->disconnect();
    channelNameEdit->disconnect();
    mulCoeff->disconnect();
    divCoeff->disconnect();
    phase1->disconnect();
    phase2->disconnect();
    phase3->disconnect();
    invertBox->disconnect();
    phase_auto->disconnect();
    chan_b_group->disconnect();
   // not_configured->disconnect();

    //Update values
    mulCoeff->setValue(powModule->mulCoeff[currentChannel-1]);
    divCoeff->setValue(powModule->divCoeff[currentChannel-1]);
    bool ok = updateModuleName();
    ok = ok && updatePhaseAndInversion();

    //Re-Connect
 //   connect(channelComboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(channelChanged(int)));
    connect(chan_b_group,SIGNAL(buttonClicked(int)),this,SLOT(channelChanged(int)));
    connect(channelNameEdit,SIGNAL(returnPressed()),this,SLOT(channelNameChanged()));
    if(advancedUserMode) connect(mulCoeff,SIGNAL(valueChanged(int)),this,SLOT(mulCoeffChanged(int)));
    if(advancedUserMode) connect(divCoeff,SIGNAL(valueChanged(int)),this,SLOT(divCoeffChanged(int)));
    connect(phase1,SIGNAL(clicked()),this,SLOT(phaseChanged()));
    connect(phase2,SIGNAL(clicked()),this,SLOT(phaseChanged()));
    connect(phase3,SIGNAL(clicked()),this,SLOT(phaseChanged()));
  //  connect(not_configured,SIGNAL(clicked()),this,SLOT(phaseChanged()));
    connect(phase_auto,SIGNAL(clicked()),this,SLOT(phaseChanged()));
    connect(invertBox,SIGNAL(clicked(bool)),this,SLOT(invertSign(bool)));

    return ok;
}

void PowerModule::timerSlot()
{
    //Connected to timer, Refresh values of window : power/current/voltage values
    //(source pascal : MV1Event.pas::ConfigVoieEvent)

    commonRefreshTimer->stop();

    //serialCom->deleteErrorCom();    //retry

    if(!channelInfoUpdated) channelInfoUpdated=updateChannelInfo();

    float power=0;
    float voltage=0;
    double current=0;
    float powerFactor=0;
    float apparent=0;
    uchar mulCoeff=powModule->mulCoeff[currentChannel-1];
    uchar divCoeff=powModule->divCoeff[currentChannel-1];

    // Power
    if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x1E0+2*(currentChannel-1),1,3))
    {
        power=int16(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),0));
        power= power*mulCoeff/divCoeff;
        powerValue->setText("<b>Power</b> :"+QString::number(power,'f',0)+" <b>Watts</b>");
    }

    // Accumulated energy
    if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x130+5*(currentChannel-1),1,3))
    {
        QString signChar;
        uchar valueTab[5];
        float accumEnergy;

        if((serialCom->getReceivedData()[4] & 0x80) !=0)
        {
            for(int i=0; i<5; i++) valueTab[i]=255-serialCom->getReceivedData()[i];
            signChar="-";
        }
        else
        {
            for(int i=0; i<5; i++) valueTab[i]=serialCom->getReceivedData()[i];
            signChar=" ";
        }
        if((valueTab[4]>0) || (valueTab[3]>128))    //if highest bit=1 (not signed)
        {
            accumEnergy=peekUint32FromUcharTabBigEndian(valueTab,1) / divCoeff * mulCoeff /3600;
            energyValue->setText("<b>Accumulated Energy</b> :"+signChar+QString::number(accumEnergy/1000.0*256,'f',0)+" <b>kWh</b>");
        }
        else    //highest bit=0
        {
            accumEnergy=peekUint32FromUcharTabBigEndian(valueTab,0) / divCoeff * mulCoeff /3600.0;
            if(accumEnergy<100)
                energyValue->setText("<b>Accumulated Energy</b> :"+signChar+QString::number(accumEnergy,'f',1)+" <b>Wh</b>");
            else if(accumEnergy<100000)
                energyValue->setText("<b>Accumulated Energy</b> :"+signChar+QString::number(accumEnergy,'f',0)+" <b>Wh</b>");
            else    //kWh display
                energyValue->setText("<b>Accumulated Energy</b> :"+signChar+QString::number(accumEnergy/1000.0,'f',0)+" <b>kWh</b>");
        }
    }

    // Accumulated reactive energy
    if(powModule->typeOfModule==powerModule2)
    {
        if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x550+6*(currentChannel-1),1,3))
        {
            QString signChar;
            uchar valueTab[5];
            float accumEnergy;

            if((serialCom->getReceivedData()[4] & 0x80) !=0) //nombre negatif
            {
                for(int i=0; i<5; i++) valueTab[i]=255-serialCom->getReceivedData()[i];
                signChar="-";
            }
            else
            {
                for(int i=0; i<5; i++) valueTab[i]=serialCom->getReceivedData()[i];
                signChar=" ";
            }
            if((valueTab[4]>0) || (valueTab[3]>128))    //if highest bit=1 (not signed)
            {
                accumEnergy=peekUint32FromUcharTabBigEndian(valueTab,1) / divCoeff * mulCoeff /3600;
                reactiveEnergyValue->setText("<b>Accumulated Reactive Energy</b> :"+signChar+QString::number(accumEnergy/1000.0*256,'f',0)+" <b>kVARh</b>");
            }
            else    //highest bit=0
            {
                accumEnergy=peekUint32FromUcharTabBigEndian(valueTab,0) / divCoeff * mulCoeff /3600.0;
                if(accumEnergy<100)
                    reactiveEnergyValue->setText("<b>Accumulated Reactive Energy</b> :"+signChar+QString::number(accumEnergy,'f',1)+" <b>VARh</b>");
                else if(accumEnergy<100000)
                    reactiveEnergyValue->setText("<b>Accumulated Reactive Energy</b> :"+signChar+QString::number(accumEnergy,'f',0)+" <b>VARh</b>");
                else    //kWh display
                    reactiveEnergyValue->setText("<b>Accumulated Reactive Energy</b> :"+signChar+QString::number(accumEnergy/1000.0,'f',0)+" <b>kVARh</b>");
            }
        }
    }
    else reactiveEnergyValue->setText("<b>Accumulated Reactive Energy</b> : ~");


    //Voltage
    if(serialCom->readBytesAtWithRetry(serialCom->getCentralSN(),0x100+2*(phase & 0x03),1,3))
    {
        voltage =sqrt(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),0) / 5859.0 / 2.0) *256.0;
        voltageValue->setText("<b>Voltage RMS</b> :"+QString::number(voltage,'f',1)+" <b>Volts</b>");
    }
    //Current
    if(powModule->typeOfModule==powerModule){//ancien module

        if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x520+4*(currentChannel-1),1,3))
        {
            current=sqrt(peekUint32FromUcharTabBigEndian(serialCom->getReceivedData(),0) / 117.0 * 256.0 /8.0) /200.0;
            current=current * mulCoeff / divCoeff;
            currentValue->setText("<b>Current RMS</b> :"+QString::number(current,'f',2)+" <b>A</b>");
        }
    }
    else
    {//nouveaux module (typeOfModule=powerModule2)
        if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x520+6*(currentChannel-1),1,3))
        {
            uint64_t value=peekUint48FromUcharTabBigEndian(serialCom->getReceivedData(),0);
            current=sqrt(value)/19442.2;
            current=current * mulCoeff / divCoeff;
            if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x110+(currentChannel-1),1,3))
            {
                //qDebug()<<"phase"<<QString::number(serialCom->getReceivedData()[0],16);
                if(serialCom->getReceivedData()[0]&0x20){
                   currentValue->setText("<b>Current RMS</b> : <font color= red> > "+QString::number(current,'f',2)+" </font><b>A</b>");
                }
                else currentValue->setText("<b>Current RMS</b> :"+QString::number(current,'f',2)+" <b>A</b>");
            }

        }
    }


    if(powModule->typeOfModule==powerModule)
    {
        //Apparent Power
        if(power<0) power=-power;
        apparent=current*voltage;
        if(apparent < 5) apparent = 0;
        if(apparent < power) apparent = power;
        apparentPowerValue->setText("<b>Apparent Power</b> :"+QString::number(apparent,'f',0)+" <b>VA</b>");

        //Reactive Power
        float reactif = sqrt(apparent*apparent-power*power);
        reactivePowerValue->setText("<b>Reactive Power</b> :"+QString::number(reactif,'f',0)+" <b>VAR</b>");
    }
    else
    {
        if(serialCom->readBytesAtWithRetry(powModule->serialNb,0x170+2*(currentChannel-1),1,3)){
            //Apparent Power
            float reactif=peekInt16FromUcharTabBigEndian(serialCom->getReceivedData(),0)* mulCoeff / divCoeff;
            reactivePowerValue->setText("<b>Reactive Power</b> :"+QString::number(reactif,'f',0)+" <b>VAR</b>");
            //Reactive Power
            apparent=sqrt(reactif*reactif+power*power);
            apparentPowerValue->setText("<b>Apparent Power</b> :"+QString::number(apparent,'f',0)+" <b>VA</b>");
        }
    }


    //Power Factor
    if(apparent>0) powerFactor=power/apparent;
    else powerFactor=0;
    powerFactorValue->setText("<b>Power Factor:</b> "+QString::number(powerFactor,'f',2));
    if(phase_auto->isChecked()){
        updatePhaseAndInversion();
    }
    commonRefreshTimer->startInterval();
}

bool PowerModule::updateModuleName()
{
    //Get the name of module, and display it

    if(!serialCom->readBytesAtWithRetry(powModule->serialNb,0x010000A0+16*(currentChannel-1),2,3))
        return false;
    char channelName[17];
    for(int charIndex=0; charIndex<16; charIndex++)
    {
        if((serialCom->getReceivedData()[charIndex]>0x1F)&&(serialCom->getReceivedData()[charIndex]!=0xFF))
        {
            channelName[charIndex]=serialCom->getReceivedData()[charIndex];
        }
        else
        {
            channelName[charIndex]='\0';
            break;
        }
    }
    powModule->channelName[currentChannel-1]=QString::fromStdString(channelName);
    channelNameEdit->setText(powModule->channelName[currentChannel-1]);
    return true;
}

bool PowerModule::updatePhaseAndInversion()
{
    //Get active phase and inversion of phase, and display them

    //Phase
    if(!serialCom->readBytesAtWithRetry(powModule->serialNb,0x110+(currentChannel-1),1,3))
        return false;
    updating_phase=true;
    bool notconfig=not_configured->isChecked();
    if(serialCom->getReceivedData()[0]&0x04){//auto
        phase_auto->setChecked(true);
        phase1->disconnect();
        phase2->disconnect();
        phase3->disconnect();
        phase1->setEnabled(false);
        phase2->setEnabled(false);
        phase3->setEnabled(false);
        //invertBox->setEnabled(false);
        //invertBox->disconnect();
        if((serialCom->getReceivedData()[0]&0x08)==8) //marge suffisante
        {
            notconfig=false;
        }
        else
        {
            not_configured->setChecked(true);
            notconfig=true;
        }
    }
    else
    {
        phase_auto->setChecked(false);
        phase1->setEnabled(true);
        phase2->setEnabled(true);
        phase3->setEnabled(true);
       // invertBox->setEnabled(true);
        connect(phase1,SIGNAL(clicked()),this,SLOT(phaseChanged()));
        connect(phase2,SIGNAL(clicked()),this,SLOT(phaseChanged()));
        connect(phase3,SIGNAL(clicked()),this,SLOT(phaseChanged()));
      //  connect(not_configured,SIGNAL(clicked()),this,SLOT(phaseChanged()));
       // connect(invertBox,SIGNAL(clicked(bool)),this,SLOT(invertSign(bool)));
    }
    if(!notconfig)
    {
        phase=serialCom->getReceivedData()[0] & 0x03;
        switch(phase)
        {
        case 0:
            phase1->setChecked(true);
            break;
        case 1:
            phase2->setChecked(true);
            break;
        case 2:
            phase3->setChecked(true);
            break;
        default:
            phase1->setChecked(false);
            phase2->setChecked(false);
            phase3->setChecked(false);
        }
        //inversion of the current

    }
    //TODO : isConfig (for start config in manage module)
    if(serialCom->getReceivedData()[0]>>7 ==1)
        invertBox->setChecked(true);
    else
        invertBox->setChecked(false);

    return true;
}

//*********************
// New values Senders

void PowerModule::channelChanged(int _newChannel)
{
    //To switch active channel
    currentChannel=_newChannel;//+1;
    updateChannelInfo();
}

void PowerModule::channelNameChanged()
{
    //Send new name when changed

    if(serialCom->setModuleName(powModule->serialNb,0x010000A0+16*(currentChannel-1),channelNameEdit->text()))
        powModule->channelName[currentChannel-1]=channelNameEdit->text();
    updateModuleName();
}


void PowerModule::invertSign(bool _invert)
{   //if inversion checked or unchecked, send phase and inversion
    sendPhaseAndInversion();
}

void PowerModule::phaseChanged()//int _newphase)
{   //if phase changed, send phase and inversion
        if(phase_auto->isChecked()){
            phase1->setEnabled(false);
            phase2->setEnabled(false);
            phase3->setEnabled(false);
            invertBox->setEnabled(false);
        }
        else {
            phase1->setEnabled(true);
            phase2->setEnabled(true);
            phase3->setEnabled(true);
            invertBox->setEnabled(true);
        }
    sendPhaseAndInversion();
}

bool PowerModule::sendPhaseAndInversion()
{
    //Send phase and inversion of phase to module

    uchar valueToSend=0;
    if(phase_auto->isChecked()) valueToSend=4;
    else
    {
        if(phase1->isChecked()) valueToSend=0;
        else if(phase2->isChecked()) valueToSend=1;
        else if(phase3->isChecked()) valueToSend=2;
        if(invertBox->isChecked())
            valueToSend +=0x80;
    }
    serialCom->writeByteAtWithRetry(powModule->serialNb,0x110+(currentChannel-1),valueToSend,3);
    return updatePhaseAndInversion();
}

void PowerModule::mulCoeffChanged(int _newCoeff)
{
    // Send new Mul coefficient
    if(_newCoeff>255) _newCoeff=255;    //limit
    if(!serialCom->writeByteAtWithRetry(powModule->serialNb,0x01000080+2*(currentChannel-1),_newCoeff,3))
        updateChannelInfo();
    else powModule->mulCoeff[currentChannel-1]=_newCoeff;
}

void PowerModule::divCoeffChanged(int _newCoeff)
{
    // Send new Mul coefficient
    if(_newCoeff>255) _newCoeff=255;    //limit
    if(!serialCom->writeByteAtWithRetry(powModule->serialNb,0x01000080+2*(currentChannel-1)+1,_newCoeff,3))
        updateChannelInfo();
    else powModule->divCoeff[currentChannel-1]=_newCoeff;
}

void PowerModule::clearKwhClicked()
{
    // clear accumulated energy (set to 0 the 5bytes)
    bool ok=true;
    for(int index=0; index<5; index++)  //5 bytes //active
        ok=ok & serialCom->writeByteAtWithRetry(powModule->serialNb,(0x130+(currentChannel-1)*5+index),0x00,3);
    if(powModule->typeOfModule==powerModule2){
        for(int index=0; index<5; index++)  //5 bytes //reactive
            ok=ok & serialCom->writeByteAtWithRetry(powModule->serialNb,(0x550+(currentChannel-1)*6+index),0x00,3);
    }
}

void PowerModule::waveformClicked()
{
    // Display waveform window
    if(waveformwindow!=NULL)
        delete waveformwindow;
    commonRefreshTimer->disconnectTimeout();
    float coeff_mod=1./powModule->divCoeff[currentChannel-1] *powModule->mulCoeff[currentChannel-1];
    waveformwindow = new waveform(serialCom,commonRefreshTimer,powModule,currentChannel,coeff_mod);
    connect(waveformwindow,SIGNAL(waveformClosed()),this,SLOT(returnfromwaveform()));
    groupBoxPowerModule->setDisabled(true);
    waveformwindow->show();

}

void PowerModule::returnfromwaveform(){
  /*  if(waveformwindow!=NULL)
    delete waveformwindow;
    waveformwindow=NULL;*/
    commonRefreshTimer->connectTimeout(this);
    if(refreshIsPaused)
        commonRefreshTimer->stop();
    else commonRefreshTimer->startSmallInterval();
    groupBoxPowerModule->setEnabled(true);
}
