#ifndef SIMLE_HTTP_H
#define SIMLE_HTTP_H
#include <QtNetwork>
#define DELAI_UP 300
#define MAXINTERFACES 5
#ifdef _RPI_
#include "PCD8544.h"
#include <wiringPi.h>
#include "c_function.h"
#endif
// HttpDaemon is the the class that implements the simple HTTP server.


class  HttpSimple : public QTcpServer
{
    Q_OBJECT
public:
    HttpSimple(quint16 port, QObject* parent = 0)
        : QTcpServer(parent), disabled(false)
    {
        timeout_config=59;
        sec10=0;
        interfaces=0;
        current_ip_id=0;
        refresh_inter();
        settingFile = new QSettings(QString(DEFAULT_DIR)+"config.ini",QSettings::IniFormat);
        if(settingFile->contains("HTTP_SERV/HH_APPEL"))
            hh_appel=settingFile->value("HTTP_SERV/HH_APPEL").toString().toUShort();
        else hh_appel=4;
        if(settingFile->contains("HTTP_SERV/MM_APPEL"))
            mm_appel=settingFile->value("HTTP_SERV/MM_APPEL").toString().toUShort();
        else mm_appel=0;
        delete settingFile;
        each_sec=new QTimer;
        #ifdef _RPI_
        update_lcd();
       #endif
        connect(each_sec,SIGNAL(timeout()),this,SLOT(each_sec_t()));

        each_sec->start(1000);
        listen(QHostAddress::Any, port);
    }

    void incomingConnection(int socket)
    {
        if (disabled)
            return;

        // When a new client connects, the server constructs a QTcpSocket and all
        // communication with the client is done over this QTcpSocket. QTcpSocket
        // works asynchronously, this means that all the communication is done
        // in the two slots readClient() and discardClient().
        QTcpSocket* s = new QTcpSocket(this);
        connect(s, SIGNAL(readyRead()), this, SLOT(readClient()));
        connect(s, SIGNAL(disconnected()), this, SLOT(discardClient()));
        s->setSocketDescriptor(socket);
    }

    void pause()
    {
        disabled = true;
    }

    void resume()
    {
        disabled = false;
    }
signals:
    void req_ftp();
    void timeout_cfg();

public slots:
    void reinit_timeout()
    {
        timeout_config=59;
    }


private slots:
    void each_sec_t()
    {
        if(!--timeout_config) 
        {
        each_sec->stop();
        emit timeout_cfg();
        return;
        }
        if(++sec10>=10)
        {
            sec10=0;
#ifdef _RPI_

        refresh_inter();
#endif

        }
        if(sec10&0x01)
        {
#ifdef _RPI_
        if(current_ip_id<(interfaces-1)) current_ip_id++;
        else current_ip_id=0;
#endif
        }
 #ifdef _RPI_
        update_lcd();
 #endif
        //check time
        if(hh_appel>24)
        {
            if(((QTime::currentTime().addSecs(-10).hour()==(hh_appel-25))||
                (QTime::currentTime().addSecs(-10).hour()==(12+hh_appel-25))))
            {
                if(QTime::currentTime().addSecs(-10).minute()==mm_appel)
                {
                    qDebug()<<"h-10s";
                    emit req_ftp();
                }
            }

        }
        else if(hh_appel==24)
        {
            if(QTime::currentTime().addSecs(-10).minute()==mm_appel)
            {
                qDebug()<<"h-10s";
                 emit req_ftp();
            }
        }
        else
        {
            if((QTime::currentTime().addSecs(-10).hour()==hh_appel))
            {
                if(QTime::currentTime().addSecs(-10).minute()==mm_appel)
                {
                    qDebug()<<"h-10s";
                     emit req_ftp();
                }
            }
        }

    }

    void readClient()
    {
        if (disabled)
            return;

        // This slot is called when the client sent data to the server. The
        // server looks if it was a get request and sends a very simple HTML
        // document back.
        QTcpSocket* socket = (QTcpSocket*)sender();
        if (socket->canReadLine()) {
            QStringList tokens = QString(socket->readLine()).split(QRegExp("[ \r\n][ \r\n]*"));
            if (tokens[0] == "GET") {
                QTextStream os(socket);
                os.setAutoDetectUnicode(true);
                os << "HTTP/1.0 200 Ok\r\n"
                      "Content-Type: text/html; charset=\"utf-8\"\r\n"
                      "\r\n"
                      "<head>"
                      "<meta http-equiv=\"refresh\" content=\"20; url=\">"
                      "</head>"
                      "<h1>SERVER IN CONFIG MODE, PLEASE WAIT ...</h1>\n";
                socket->close();


                if (socket->state() == QTcpSocket::UnconnectedState) {
                    delete socket;
                }
            }
        }
    }
    void discardClient()
    {
        QTcpSocket* socket = (QTcpSocket*)sender();
        socket->deleteLater();
    }

private:
    uint8_t timeout_config;
    uint8_t current_ip_id;
    #ifdef _RPI_
    char line0[16],line1[16],line2[16],line3[16],line4[16],line5[16];
    void update_lcd()
    {
        QString line1_str="Network "+i_face[current_ip_id]+":";
        QString line4_str="Config Mode "+QString::number(timeout_config);
        LCDclear();
        QString Line0="v"+QString::number(VERSION_SOFT)+" "+QTime::currentTime().toString("hh:mm:ss");
        for(int i=0;i<14;i++)
        {
            if(Line0.length()>i)
               line0[i]= Line0.at(i).toAscii();
            else line0[i]='\0';
            if(line4_str.length()>i)
               line4[i]= line4_str.at(i).toAscii();
            else line4[i]='\0';
            if(interfaces>0)
            {
                if(line1_str.length()>i)
                   line1[i]= line1_str.at(i).toAscii();
                else line1[i]='\0';
            }
        }
        line0[15]='\0';
        line1[15]='\0';
        line2[15]='\0';
        line3[15]='\0';
        LCDdrawstring(2,0, line0);
        if(interfaces<1)
        {
            LCDdrawstring(2,10, "No Interfaces");
        }
        else
        {
            LCDdrawstring(0,10, line1);
            LCDdrawstring(0,19, ip[current_ip_id]);
        }

        LCDdrawstring(0,30, "Config. Mode");
        LCDdrawstring_c(0,40, line4);
        LCDdisplay();
    }
    #endif
    QSettings *settingFile;
    uchar hh_appel,mm_appel;
    QTimer *each_sec;
    bool disabled;
    char ip[MAXINTERFACES][16];
    uint8_t sec10;
    int interfaces;
    QString i_face[MAXINTERFACES];
    void refresh_inter()
    {
#ifdef _RPI_
        int sock;
        char *p;
        struct ifconf ifconf;
        struct ifreq ifreq[MAXINTERFACES];
        int i;

        // Create a socket or return an error.
        sock = socket(AF_INET, SOCK_STREAM, 0);
        if (sock < 0)
            qDebug()<<"socket";

        // Point ifconf's ifc_buf to our array of interface ifreqs.
        ifconf.ifc_buf = (char *) ifreq;

        // Set ifconf's ifc_len to the length of our array of interface ifreqs.
        ifconf.ifc_len = sizeof ifreq;

        //  Populate ifconf.ifc_buf (ifreq) with a list of interface names and addresses.
        if (ioctl(sock, SIOCGIFCONF, &ifconf) == -1)
            qDebug()<<"ioctl";

        // Divide the length of the interface list by the size of each entry.
        // This gives us the number of interfaces on the system.
        if(sizeof(ifreq[0])) interfaces = ifconf.ifc_len / sizeof(ifreq[0]);
        else interfaces=0;

        // Loop through the array of interfaces, printing each one's name and IP.
        for (i = 0; i < interfaces; i++)
        {
            if (-1==ioctl(sock, SIOCGIFADDR, &ifreq[i])) {
                 qDebug()<<"ioctl(SIOCGIFADDR) ";
                return ;
            }
            p=inet_ntoa(((struct sockaddr_in *)(&ifreq[i].ifr_addr))->sin_addr);
            strncpy(ip[i],p,sizeof(ip[i])-1);
            ip[i][sizeof(ip[i])-1]='\0';
            i_face[i]=ifreq[i].ifr_name;

            // Convert the binary IP address into a readable string.
        }
    closesocket(sock);
#endif
    }
};
#endif // SIMLE_HTTP_H
