#include "wirelessmodule.h"

//****************************
//  Constructor & Destructor
//****************************

WirelessModule::WirelessModule(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, Module *_wireless_Module, QWidget *parent) :
    QWidget(parent)
{
    //Constructor

    //init values
    serialCom=_serialPort;
    commonRefreshTimer=_commonRefreshTimer;
    wireless_Module=_wireless_Module;
    comboBoxSignalMapper = NULL;
    firstRefresh=false;
    fillingSensorTable=false;

    // Init Channel type Lists
    listTempHygroPulse <<"Temperature"<<"Humidity"<<"Pulse Count"<<"ON Time"<<"LQI to"<<"RSSI From"<<"Bat";
    listTempPulse <<"Temp 1"<<"Temp 2"<<"Pulse Count"<<"ON Time"<<"LQI to"<<"RSSI From"<<"Bat";
    listLamp <<"Light"<<"Switch Count"<<"ON Time"<<"LQI to"<<"RSSI From"<<"Bat";
    listLux <<"Light Sensor"<<"Switch Count"<<"ON Time"<<"LQI to"<<"RSSI From"<<"Bat";
    listCo2 <<"CO2"<<"Temp"<<"LQI to"<<"RSSI From"<<"Bat";
    listDelayAna <<"A1"<<"A2"<<"Switch Count"<<"ON Time"<<"LQI to"<<"RSSI From"<<"Bat";
    listPressure <<"Pressure"<<"LQI to"<<"RSSI From"<<"Bat";
    listPresence <<"Switch Count"<<"ON Time"<<"LQI to"<<"RSSI From"<<"Bat";
    listWattmeter <<"Volts"<<"Amps"<<"Active Power"<<"Apparent Power"
                  <<"Active Energy"<<"Appar. Energy"<<"LQI to"<<"RSSI From"<<"Bat";
    batt_volt<<"~"<<"2.9V"<<"3.0V"<<"3.4V"<<"2.2V"<<"2.3V"<<"2.4V"<<
        "2.5V"<<"2.6V"<<"2.7V"<<"2.8V"<<"3.1V"<<"3.2V"<<"3.3V"<<"3.5V"<<"3.7V";



    // To map signals from channel type comboBoxes
    comboBoxSignalMapper = new QSignalMapper(this);

    //Add GUI elements
    fillWindow();

    //put columns
    setTableWidget();

    //start timer
    commonRefreshTimer->connectTimeout(this);
    firstRefresh=true;
    commonRefreshTimer->startSmallInterval(); // insert a small delay before fetching values
}

WirelessModule::~WirelessModule()
{
    //TODO : delete ?
}

//****************************
//  Graphics Elements
//****************************

void WirelessModule::fillWindow()
{
    //Add GUI elements
    globalLayout = new QVBoxLayout;
    setLayout(globalLayout);
        groupBoxWireless=new QGroupBox("Wireless sensors");
            verticalLayout = new QVBoxLayout;
            groupBoxWireless->setLayout(verticalLayout);
                sensorTableWidget = new QTableWidget(0,8);
                verticalLayout->addWidget(sensorTableWidget);
                buttonLayout = new QHBoxLayout;
                    returnButton = new QPushButton("<= Back");
                    buttonLayout->addWidget(returnButton);
                    connect(returnButton,SIGNAL(clicked()),this,SLOT(backToManageModule()));
                    returnButton->setEnabled(false);
                    sensorNbButton = new QPushButton("Change sensor number");
                    buttonLayout->addWidget(sensorNbButton);
                    connect(sensorNbButton,SIGNAL(clicked()),this,SLOT(sensorNbButtonClicked()));
                    sensorNbButton->setEnabled(false);
                verticalLayout->addLayout(buttonLayout);
    globalLayout->addWidget(groupBoxWireless);
}

void WirelessModule::setTableWidget()
{
    // Place columns into sensorTableWidget
    sensorTableWidget->setVisible(true);
    sensorSNTableItem = new QTableWidgetItem("SN");
    sensorTableWidget->setHorizontalHeaderItem(0,sensorSNTableItem);

    sensorChannelTableItem = new QTableWidgetItem("Channel");
    sensorTableWidget->setHorizontalHeaderItem(1,sensorChannelTableItem);

    sensorNameTableItem = new QTableWidgetItem("Name");
    sensorTableWidget->setHorizontalHeaderItem(2,sensorNameTableItem);

    sensorValueTableItem = new QTableWidgetItem("Value");
    sensorTableWidget->setHorizontalHeaderItem(3,sensorValueTableItem);
    sensorDelayTableItem = new QTableWidgetItem("Last Rec");
    sensorTableWidget->setHorizontalHeaderItem(4,sensorDelayTableItem);
    sensorSignalFromTableItem = new QTableWidgetItem("RSSI From");
    sensorTableWidget->setHorizontalHeaderItem(5,sensorSignalFromTableItem);
    sensorSignalToTableItem = new QTableWidgetItem("LQi to");
    sensorTableWidget->setHorizontalHeaderItem(6,sensorSignalToTableItem);

    sensorBatteryTableItem = new QTableWidgetItem("Battery");
    sensorTableWidget->setHorizontalHeaderItem(7,sensorBatteryTableItem);

    sensorTableWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
    sensorTableWidget->setAutoScroll(false);
    sensorTableWidget->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);

    sensorItemTable.clear();    //to be sure
    sensorList.clear();
    sensorTableWidget->clearContents();
    sensorTableWidget->setRowCount(0);

    //connect to apply changes in a cell
    connect(sensorTableWidget,SIGNAL(cellChanged(int,int)),this,SLOT(valueChanged(int,int)));
}

//****************************
//  Sensor display
//****************************
bool WirelessModule::fillSensorTable()
{
    //Fill table with sensor, getting list and information

    //Clear all tables and widget
    sensorItemTable.clear();
    sensorList.clear();
    sensorTableWidget->clearContents();
    sensorTableWidget->setRowCount(wireless_Module->nbOfLines*6);
    if(channelComboBoxList.size()>0)
    {
        for(uint i=0; i<channelComboBoxList.size(); i++)
        {
            delete channelComboBoxList[i];
            channelComboBoxList[i]=NULL;
        }
    }
    channelComboBoxList.clear();

    //Nb of reading of 64 requested to fetch sensor names
    int nbReadingNames=ceil(wireless_Module->nbOfLines*3.0/2.0);
        // 6 sensors * 16 bytes / 64 max reading

    //char tab to store values from @500, 160 & 300
    uchar downloadedValuesSerialNb[96];
    uchar downloadedValuesChannelType[64];
    uchar downloadedValuesSensorNames[nbReadingNames*64];

    bool ok=true;
    //reading sensor SN - SensorSN : 0x010000A0   -  MSB..LSB
    ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x010000A0,8,3);
    if(!ok) return false;
    for(int charIndex=0; charIndex<64; charIndex++)
    {
        downloadedValuesSerialNb[charIndex]=serialCom->getReceivedData()[charIndex];
    }
    if(wireless_Module->nbOfLines>5)    // 2nd reading of 32 bytes requested
    {
        ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x010000A0+64,4,3);
        if(!ok) return false;
        for(int charIndex=0; charIndex<32; charIndex++)
            downloadedValuesSerialNb[charIndex+64]=serialCom->getReceivedData()[charIndex];
    }

    //reading sensor Channel - SensorCh : 0x01000040.
    int nbReadingChannel=8;
    if(wireless_Module->nbOfLines<=5) nbReadingChannel=4;
    ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x01000040,nbReadingChannel,3);
    if(!ok) return false;
    for(int charIndex=0; charIndex<nbReadingChannel*8; charIndex++)//Place in the table :
        downloadedValuesChannelType[charIndex]=serialCom->getReceivedData()[charIndex];

    //reading Name - 0x031FFD00 + 16 * sensorIndex
    for(int readingIndex=0; readingIndex<nbReadingNames; readingIndex++)
    {
        ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x031FFD00+readingIndex*64,8,3);
        if(!ok) return false;
        for(int charIndex=0; charIndex<64; charIndex++)//Place in the table :
            downloadedValuesSensorNames[charIndex+readingIndex*64]=serialCom->getReceivedData()[charIndex];
    }


    //Fetching information (sn, type, name) about each sensor, and add to list
    for(uchar sensorIndex=0; sensorIndex<wireless_Module->nbOfLines*6; sensorIndex++)
    {
        Sensor currentSensor;
        //read Serial Nb
        currentSensor.serialNb=peekUint16FromUcharTabLittleEndian(downloadedValuesSerialNb,sensorIndex*2);

        //read channelType
        currentSensor.channelType=downloadedValuesChannelType[sensorIndex];

        char sensorName[17];
        for(uint32 i =0; i<16;i++)
        {
            if(downloadedValuesSensorNames[i+sensorIndex*16]!=(0x00)&&downloadedValuesSensorNames[i+sensorIndex*16]!=(0xFF))
                sensorName[i]=downloadedValuesSensorNames[i+sensorIndex*16];
            else {
                sensorName[i]='\0'; //clear next char
                break;
            }
        }
        sensorName[16]='\0';
        currentSensor.sensorName=QString(sensorName);

        //Add to sensor list
        sensorList.push_back(currentSensor);

       /* // with channelType : fill the rest of the struct
        updateStructureFromChannelType(sensorList.size()-1);*/

        //add a new entry int the table of QtableWidget
        vector<QTableWidgetItem *> newLine;
        fillingSensorTable=true;

        // Add empty items, from 0 to 7
        for(uint16 columnIndex=0; columnIndex<8; columnIndex++)
        {
            //create item
            QTableWidgetItem *newItem = new QTableWidgetItem;
            newItem->setTextAlignment(Qt::AlignRight|Qt::AlignVCenter);
            if(columnIndex!=0 && columnIndex!=2) newItem->setFlags(newItem->flags()
                                    & ~Qt::ItemIsEditable & ~Qt::ItemIsSelectable);
            else newItem->setFlags(newItem->flags() & ~Qt::ItemIsSelectable);
                                            // to let serialNb & sensorName editable
            newLine.push_back(newItem);
            //link table entry with widget :
            sensorTableWidget->setItem(sensorIndex,columnIndex,newItem);
        }
        sensorItemTable.push_back(newLine);

        //add comboBox for channel selection
        QComboBox * selectChannelComboBox = new QComboBox;
        connect(selectChannelComboBox, SIGNAL(currentIndexChanged(int)),comboBoxSignalMapper,SLOT(map()));
        comboBoxSignalMapper->setMapping(selectChannelComboBox,int(sensorIndex));
        channelComboBoxList.push_back(selectChannelComboBox);
        sensorTableWidget->setCellWidget(sensorIndex,1,selectChannelComboBox);

        //add in sensorTableWidget
        updateSensorTableEntryFromList(sensorIndex);
        updateStructureFromChannelType(sensorIndex);
        fillingSensorTable=false;
    }
    connect(comboBoxSignalMapper,SIGNAL(mapped(int)),this,SLOT(comboBoxChanged(int)));

    sensorTableWidget->resizeRowsToContents();

    return true;
}


void WirelessModule::updateSensorTableEntryFromList(int _sensorIndex)
{
    // Update the sensor entry in the widget (after SN/name/channelType changed in the list)

    //Update SN from list
    sensorTableWidget->item(_sensorIndex,0)->setText(QString::number(
                sensorList[_sensorIndex].serialNb,16).rightJustified(4,'0').toUpper());
    if(sensorList[_sensorIndex].serialNb!=0)
    {
        //Update name from list
        sensorTableWidget->item(_sensorIndex,2)->setText(sensorList[_sensorIndex].sensorName);
        updateComboBoxList(_sensorIndex);
        updateComboBoxSelection(_sensorIndex);
    }
    else    //sensorSN = 0
    {
        updateComboBoxList(_sensorIndex);
        sensorItemTable[_sensorIndex][2]->setText("");
    }

    //Debug : Erase other cells
    for(int i=3; i<8;i++)
    {
        sensorItemTable[_sensorIndex][i]->setText("xxx");
    }
}

bool WirelessModule::updateVisibleSensorsValues()
{
    //refresh values of sensors shown in the tablewidget

    //In order to save time, all datas are fetched directly (not line by line)
    bool ok=true;
    returnButton->setEnabled(true); //enable buttons
    sensorNbButton->setEnabled(true);
    // compute the number of reading needed (first and last)
    uint firstVisibleSensor=sensorTableWidget->verticalScrollBar()->value();

    uint numberOfVisibleSensors;
    if(!sensorTableWidget->verticalScrollBar()->isVisible())
    {
        numberOfVisibleSensors=sensorList.size();   //all row are visible
    }
    else
    {
        numberOfVisibleSensors=((sensorTableWidget->verticalScrollBar()->height()
        -sensorTableWidget->horizontalHeader()->height())
        /(sensorTableWidget->rowHeight(0))+0.8);   //TODO : check
        if(numberOfVisibleSensors>(sensorList.size()-firstVisibleSensor))   //limit
            numberOfVisibleSensors=sensorList.size()-firstVisibleSensor;
    }

    int nbReading160=ceil(numberOfVisibleSensors/32.0);
    int nbReading300=ceil(numberOfVisibleSensors/12.0);
    int nbReading500=ceil(numberOfVisibleSensors/16.0);

    // array to store download values
    uchar downloadedValuesFrom160[nbReading160*64];
    uchar downloadedValuesFrom300[nbReading300*64];
    uchar downloadedValuesFrom500[nbReading500*64];

    //download values and fill arrays
    for(int readingIndex=0; readingIndex<nbReading160; readingIndex++)
    {
        ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x160+2*firstVisibleSensor+readingIndex*64,8,3);
        if(!ok) return false;
        for(int charIndex=0; charIndex<64; charIndex++)
            downloadedValuesFrom160[charIndex+readingIndex*64]=serialCom->getReceivedData()[charIndex];
    }
    for(int readingIndex=0; readingIndex<nbReading300; readingIndex++)
    {
        ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x300+5*firstVisibleSensor+readingIndex*64,8,3);
        if(!ok) return false;
        for(int charIndex=0; charIndex<64; charIndex++)
            downloadedValuesFrom300[charIndex+readingIndex*64]=serialCom->getReceivedData()[charIndex];
    }
    for(int readingIndex=0; readingIndex<nbReading500; readingIndex++)
    {
        ok=serialCom->readBytesAtWithRetry(wireless_Module->serialNb,0x500+4*firstVisibleSensor+readingIndex*64,8,3);
        if(!ok) return false;
        for(int charIndex=0; charIndex<64; charIndex++)
            downloadedValuesFrom500[charIndex+readingIndex*64]=serialCom->getReceivedData()[charIndex];
    }

    //fetch values and place into items
    for(uint sensorIndex=firstVisibleSensor; sensorIndex<firstVisibleSensor+numberOfVisibleSensors; sensorIndex++)
    {
        if(sensorList[sensorIndex].serialNb!=0) // if sensor defined
        {
            //place of value in array (without offset)
            int sensorIndexInDLValues=sensorIndex-firstVisibleSensor;

            int channelValue=peekUint16FromUcharTabBigEndian(downloadedValuesFrom160,sensorIndexInDLValues*2);
            if (channelValue==0x8000 || channelValue==0x7FFE)   //invalid values
                    sensorItemTable[sensorIndex][3]->setText("~");
            else if(sensorList[sensorIndex].channelType<0x80)
            {
                float channelFloatValue;
                if(channelValue>32767)channelFloatValue=(channelValue-65536)*sensorList[sensorIndex].coefficient;//pour grer les valeurs ngatives
                else channelFloatValue=channelValue*sensorList[sensorIndex].coefficient;
                if((sensorList[sensorIndex].channelType==0xA)&&
                   (channelFloatValue==-32767))
                    sensorItemTable[sensorIndex][3]->setText("N/C");
                else
                sensorItemTable[sensorIndex][3]->setText(QString::number(
                        channelFloatValue,'f',sensorList[sensorIndex].displayPrecision)
                        +sensorList[sensorIndex].unitString);

            }
            else if(sensorList[sensorIndex].channelType==0x82)

            {
                sensorItemTable[sensorIndex][3]->setText(QString::number((channelValue+1)/2.55)+"%");
            }
            else if(sensorList[sensorIndex].channelType==0x83)
            {
                sensorItemTable[sensorIndex][3]->setText("-"+QString::number(channelValue)+"dbm");
            }
            else if(sensorList[sensorIndex].channelType==0x84)
            {
                if((channelValue>15)||(channelValue<0))
                    channelValue=0;
                sensorItemTable[sensorIndex][3]->setText(batt_volt.at(channelValue));
            }
            else if((sensorList[sensorIndex].channelType==0x90)||(sensorList[sensorIndex].channelType==0x91))
            {
                channelValue=peekInt32FromUcharTabBigEndian(downloadedValuesFrom500,sensorIndexInDLValues*4);
                float channelFloatValue=channelValue*sensorList[sensorIndex].coefficient;
                sensorItemTable[sensorIndex][3]->setText(QString::number(
                        channelFloatValue,'f',sensorList[sensorIndex].displayPrecision)
                        +sensorList[sensorIndex].unitString);
             }
            else    //type 0x80,81,90,91,92
            {

                channelValue=peekUint32FromUcharTabBigEndian(downloadedValuesFrom500,sensorIndexInDLValues*4);
                float channelFloatValue=channelValue*sensorList[sensorIndex].coefficient;
                if(sensorList[sensorIndex].channelType==0x81) // format in hour for ON Time
                {
                    if(channelFloatValue>3600) {    //hour
                        channelFloatValue=channelFloatValue/3600.0;
                        sensorList[sensorIndex].unitString="h";
                        sensorList[sensorIndex].displayPrecision=1;
                    }
                    else {
                        sensorList[sensorIndex].unitString="s";
                        sensorList[sensorIndex].displayPrecision=0;
                    }
                }


                sensorItemTable[sensorIndex][3]->setText(QString::number(
                        channelFloatValue,'f',sensorList[sensorIndex].displayPrecision)
                        +sensorList[sensorIndex].unitString);
            }

            //read last record delay (seconds)
            uint16 lastRecord=peekUint16FromUcharTabBigEndian(downloadedValuesFrom300,sensorIndexInDLValues*5);
            if(lastRecord>3600) sensorItemTable[sensorIndex][4]->setText("Bad Com");
            else if(lastRecord>1000)
            {
                lastRecord=lastRecord/60;
                sensorItemTable[sensorIndex][4]->setText(QString::number(lastRecord)+"m");
            }
            else sensorItemTable[sensorIndex][4]->setText(QString::number(lastRecord)+"s");

            //read RSSI
            QString valueToPrint;
            if((downloadedValuesFrom300[(sensorIndexInDLValues*5)+4] & 0xf0) >0) valueToPrint="F ";
            valueToPrint+="-"+QString::number(downloadedValuesFrom300[(sensorIndexInDLValues*5)+2])+"dBm";
            sensorItemTable[sensorIndex][5]->setText(valueToPrint);

            //read LQI
            valueToPrint.clear();
            if((downloadedValuesFrom300[(sensorIndexInDLValues*5)+3] & 0x01 )>0) valueToPrint="F ";
            valueToPrint+=QString::number((downloadedValuesFrom300[(sensorIndexInDLValues*5)+3]*100/255)&0xFE)+"%";
            sensorItemTable[sensorIndex][6]->setText(valueToPrint);

            //read battery state
            valueToPrint.clear();
            uchar value=downloadedValuesFrom300[(sensorIndexInDLValues*5)+4]&0x0F;
            sensorItemTable[sensorIndex][7]->setText(batt_volt.at(value));
           /* if(value==14) sensorItemTable[sensorIndex][7]->setText("3.5V");
            else if(value==13) sensorItemTable[sensorIndex][7]->setText("3.3V");
            else if(value==12) sensorItemTable[sensorIndex][7]->setText("3.2V");
            else if(value==11) sensorItemTable[sensorIndex][7]->setText("3.1V");
            else if(value==10) sensorItemTable[sensorIndex][7]->setText("2.8V");
            else if(value==9) sensorItemTable[sensorIndex][7]->setText("2.7V");
            else if(value==8) sensorItemTable[sensorIndex][7]->setText("2.6V");
            else if(value==7) sensorItemTable[sensorIndex][7]->setText("2.5V");
            else if(value==3) sensorItemTable[sensorIndex][7]->setText("3.4V");
            else if(value==2) sensorItemTable[sensorIndex][7]->setText("3.0V");
            else if(value==1) sensorItemTable[sensorIndex][7]->setText("2.9V");
            else if(value==4) sensorItemTable[sensorIndex][7]->setText("2.2V");
            else if(value==5) sensorItemTable[sensorIndex][7]->setText("2.3V");
            else if(value==6) sensorItemTable[sensorIndex][7]->setText("2.4V");
            else if(value==15) sensorItemTable[sensorIndex][7]->setText("3.7V");
            else sensorItemTable[sensorIndex][7]->setText("~");*/

        }
        else    //sensor not defined, SN==0
        {
            for(int i=3; i<8;i++)
                sensorItemTable[sensorIndex][i]->setText("~");
        }
    }//end for(sensorIndex)

    return true;
}

void WirelessModule::timerSlot()
{   //Slot connected to timer, refresh the table

    commonRefreshTimer->stop();

    //serialCom->deleteErrorCom();    //retry

    if(firstRefresh)
    {
        fillSensorTable();
        firstRefresh=false;
        update();
        QCoreApplication::processEvents();
    }
    if(!updateVisibleSensorsValues())
        qDebug()<<"Error reading sensors values";

    commonRefreshTimer->startInterval();
}

void WirelessModule::updateComboBoxList(int _row)
{   //Refresh elements of the comboBox (type of channel), according to sensor SN
    sensorList[_row].channelType2=0;
    channelComboBoxList[_row]->clear(); //clear current elements
    if(sensorList[_row].serialNb!=0)
    {
        if(sensorList[_row].serialNb<0x0400)
        {//Sensor Type TempHygroPulse
            channelComboBoxList[_row]->addItems(listTempHygroPulse);

        }
        else if((sensorList[_row].serialNb>0x0399 && sensorList[_row].serialNb<0x0600)
                    || (sensorList[_row].serialNb>0x3999 && sensorList[_row].serialNb<0x6000))
        {//Sensor Type TempPulse
            channelComboBoxList[_row]->addItems(listTempPulse);
        }
        else if(sensorList[_row].serialNb>0x0599 && sensorList[_row].serialNb<0x1200)
        {//Sensor Type Lamp
            channelComboBoxList[_row]->addItems(listLamp);
        }
        else if(sensorList[_row].serialNb>0x1199 && sensorList[_row].serialNb<0x1500)
        {//Sensor Type presence

            channelComboBoxList[_row]->addItems(listPresence);
        }
        else if(sensorList[_row].serialNb>0x2499 && sensorList[_row].serialNb<0x3000)
        {//Sensor Type light
            sensorList[_row].channelType2=2;
            channelComboBoxList[_row]->addItems(listLux);
        }
        else if(sensorList[_row].serialNb>0x1499 && sensorList[_row].serialNb<0x2000)
        {//Sensor Type delayana
            sensorList[_row].channelType2=4;
            channelComboBoxList[_row]->addItems(listDelayAna);
        }
        else if(sensorList[_row].serialNb>0x1999 && sensorList[_row].serialNb<0x3000)
        {//Sensor Type pressure
            sensorList[_row].channelType2=1;
            channelComboBoxList[_row]->addItems(listPressure);
        }
        else if(sensorList[_row].serialNb>0x2999 && sensorList[_row].serialNb<0x4000)
        {//Sensor Type co2
            sensorList[_row].channelType2=3;
            channelComboBoxList[_row]->addItems(listCo2);
        }
        else if(sensorList[_row].serialNb>0x8200 )//&& sensorList[_row].serialNb<0x9999) modif lora
        {//Sensor Type Wattmeter
            channelComboBoxList[_row]->addItems(listWattmeter);
        }
        else qDebug()<<"error switching SN for channel List";
    }
}

void WirelessModule::updateComboBoxSelection(int _row)
{   //Auto-select an element of the comboBox (type of channel), according to channelType read

    if(sensorList[_row].serialNb<0x0400)
    {//Sensor Type TempHygroPulse
        if(sensorList[_row].channelType==0x81) channelComboBoxList[_row]->setCurrentIndex(3); //ON Time
        else if(sensorList[_row].channelType==0x80) channelComboBoxList[_row]->setCurrentIndex(2); //Pulses
        else if(sensorList[_row].channelType==1) channelComboBoxList[_row]->setCurrentIndex(1); //Humidity
        else if(sensorList[_row].channelType==0) channelComboBoxList[_row]->setCurrentIndex(0); //Temperature
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(4);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(5);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(6);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }
    else if((sensorList[_row].serialNb>0x0399 && sensorList[_row].serialNb<0x0600)
                    || (sensorList[_row].serialNb>0x3999 && sensorList[_row].serialNb<0x6000))
    {//Sensor Type TempPulse
        if(sensorList[_row].channelType==0x81) channelComboBoxList[_row]->setCurrentIndex(3); //ON Time
        else if(sensorList[_row].channelType==0x80) channelComboBoxList[_row]->setCurrentIndex(2); //Pulses
        else if(sensorList[_row].channelType==7) channelComboBoxList[_row]->setCurrentIndex(1); //Average Temp.
        else if(sensorList[_row].channelType==0) channelComboBoxList[_row]->setCurrentIndex(0); //Temperature
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(4);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(5);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(6);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }
    else if(sensorList[_row].serialNb>0x0599 && sensorList[_row].serialNb<0x1200)
    {//Sensor Type Lamp
        if(sensorList[_row].channelType==0x81) channelComboBoxList[_row]->setCurrentIndex(2); //ON Time
        else if(sensorList[_row].channelType==0x92) channelComboBoxList[_row]->setCurrentIndex(1); //Switch ON
        else if(sensorList[_row].channelType==6) channelComboBoxList[_row]->setCurrentIndex(0); //Light
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(3);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(4);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(5);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }

    else if(sensorList[_row].serialNb>0x1199 && sensorList[_row].serialNb<0x1500)
    {//Sensor Type presence
        if(sensorList[_row].channelType==0x81) channelComboBoxList[_row]->setCurrentIndex(1); //ON Time
        else if(sensorList[_row].channelType==0x92) channelComboBoxList[_row]->setCurrentIndex(0); //Switch ON
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(2);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(3);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(4);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }

    else if(sensorList[_row].serialNb>0x2499 && sensorList[_row].serialNb<0x3000)
    {//Sensor Type lux
        if(sensorList[_row].channelType==0x81) channelComboBoxList[_row]->setCurrentIndex(2); //ON Time
        else if(sensorList[_row].channelType==0x92) channelComboBoxList[_row]->setCurrentIndex(1); //Switch ON
        else if(sensorList[_row].channelType==0x8) channelComboBoxList[_row]->setCurrentIndex(0); //Light
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(3);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(4);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(5);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }
    else if(sensorList[_row].serialNb>0x1499 && sensorList[_row].serialNb<0x2000)
    {//Sensor Type delay ana
        if(sensorList[_row].channelType==0xB) channelComboBoxList[_row]->setCurrentIndex(0); //Ana1
        else if(sensorList[_row].channelType==7) channelComboBoxList[_row]->setCurrentIndex(1); //Ana2
        else if(sensorList[_row].channelType==0x81) channelComboBoxList[_row]->setCurrentIndex(2); //ON Time
        else if(sensorList[_row].channelType==0x92) channelComboBoxList[_row]->setCurrentIndex(3); //Switch ON
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(4);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(5);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(6);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }
    else if(sensorList[_row].serialNb>0x1999 && sensorList[_row].serialNb<0x3000)
    {//Sensor Type pressure
        if(sensorList[_row].channelType==0x9) channelComboBoxList[_row]->setCurrentIndex(0); //Pressure
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(1);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(2);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(3);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }
    else if(sensorList[_row].serialNb>0x2999 && sensorList[_row].serialNb<0x4000)
    {//Sensor Type Co2

        if(sensorList[_row].channelType==0xA) channelComboBoxList[_row]->setCurrentIndex(0); //Co2
        else if(sensorList[_row].channelType==7) channelComboBoxList[_row]->setCurrentIndex(1); //Average Temp.
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(2);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(3);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(4);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }
    else if(sensorList[_row].serialNb>0x8200 )//&& sensorList[_row].serialNb<0x9999) modif pour lora
    {//Sensor Type Wattmeter
        if(sensorList[_row].channelType==0x91) channelComboBoxList[_row]->setCurrentIndex(5); //App. Energy
        else if(sensorList[_row].channelType==0x90) channelComboBoxList[_row]->setCurrentIndex(4); //Active Energy
        else if(sensorList[_row].channelType==5) channelComboBoxList[_row]->setCurrentIndex(3); //Apparent Power
        else if(sensorList[_row].channelType==4) channelComboBoxList[_row]->setCurrentIndex(2); //Active Power
        else if(sensorList[_row].channelType==3) channelComboBoxList[_row]->setCurrentIndex(1); //Amps
        else if(sensorList[_row].channelType==2) channelComboBoxList[_row]->setCurrentIndex(0); //Volts
        else if(sensorList[_row].channelType==0x82)
           channelComboBoxList[_row]->setCurrentIndex(6);
        else if(sensorList[_row].channelType==0x83)
            channelComboBoxList[_row]->setCurrentIndex(7);
        else if(sensorList[_row].channelType==0x84)
            channelComboBoxList[_row]->setCurrentIndex(8);
        else channelComboBoxList[_row]->setCurrentIndex(-1);
    }
    else
    {
        qDebug()<<"error switching SN for channel List : "<<QString::number(sensorList[_row].serialNb);
        channelComboBoxList[_row]->setCurrentIndex(-1);
    }
}



void WirelessModule::updateStructureFromChannelType(int _sensorIndex)
{   //Update structure of sensor (coefficient, unit & precision), according to channelType
    switch(sensorList[_sensorIndex].channelType)
    {
        case 0://Temperature
            switch(sensorList[_sensorIndex].channelType2)
            {

            default:
                sensorList[_sensorIndex].coefficient=0.01;
                sensorList[_sensorIndex].displayPrecision=1;
                sensorList[_sensorIndex].unitString="C";
                break;
            }
            break;
        case 1://Humidity
            sensorList[_sensorIndex].coefficient=.01;
            sensorList[_sensorIndex].displayPrecision=1;
            sensorList[_sensorIndex].unitString="%RH";
            break;
        case 2://Volts
            sensorList[_sensorIndex].coefficient=.01;
            sensorList[_sensorIndex].displayPrecision=1;
            sensorList[_sensorIndex].unitString="V";
            break;
        case 3://Amps
            sensorList[_sensorIndex].coefficient=.01;
            sensorList[_sensorIndex].displayPrecision=2;
            sensorList[_sensorIndex].unitString="A";
            break;
        case 4://Active Power
            sensorList[_sensorIndex].coefficient=.1;
            sensorList[_sensorIndex].displayPrecision=1;
            sensorList[_sensorIndex].unitString="W";
            break;
        case 5://Apparent Power
            sensorList[_sensorIndex].coefficient=.1;
            sensorList[_sensorIndex].displayPrecision=1;
            sensorList[_sensorIndex].unitString="VA";
            break;
        case 6://Light%
            sensorList[_sensorIndex].coefficient=100.0/255.0;
            sensorList[_sensorIndex].displayPrecision=0;
            sensorList[_sensorIndex].unitString="%";
            break;
        case 7://Average Temp
            switch(sensorList[_sensorIndex].channelType2)
            {
            case 4:
                sensorList[_sensorIndex].coefficient=1;
                sensorList[_sensorIndex].displayPrecision=0;
                sensorList[_sensorIndex].unitString="";
                break;
            default:
                sensorList[_sensorIndex].coefficient=.01;
                sensorList[_sensorIndex].displayPrecision=1;
                sensorList[_sensorIndex].unitString="C";
                break;
            }
            break;
         case 9:
                sensorList[_sensorIndex].coefficient=.016;
                sensorList[_sensorIndex].displayPrecision=2;
                sensorList[_sensorIndex].unitString="Pa";
                break;
         case 8:
                sensorList[_sensorIndex].coefficient=1;
                sensorList[_sensorIndex].displayPrecision=0;
                sensorList[_sensorIndex].unitString="lux";
                break;
         case 0xA:
                sensorList[_sensorIndex].coefficient=1;
                sensorList[_sensorIndex].displayPrecision=0;
                sensorList[_sensorIndex].unitString="ppm";
                break;
         case 0xB:
                sensorList[_sensorIndex].coefficient=1;
                sensorList[_sensorIndex].displayPrecision=0;
                sensorList[_sensorIndex].unitString="";
                break;

        case 0x80://Pulses
            sensorList[_sensorIndex].coefficient=1;
            sensorList[_sensorIndex].displayPrecision=0;
            sensorList[_sensorIndex].unitString="";
            break;
        case 0x81://ON Time
            sensorList[_sensorIndex].coefficient=1;
            sensorList[_sensorIndex].displayPrecision=0;
            sensorList[_sensorIndex].unitString="s";
            break;
        case 0x90://Active Energy
            sensorList[_sensorIndex].coefficient=.1;
            sensorList[_sensorIndex].displayPrecision=1;
            sensorList[_sensorIndex].unitString="Wh";
            break;
        case 0x91://App. Energy
            sensorList[_sensorIndex].coefficient=.1;
            sensorList[_sensorIndex].displayPrecision=1;
            sensorList[_sensorIndex].unitString="VAh";
            break;
        case 0x92://Switch ON (Lamp)
            sensorList[_sensorIndex].coefficient=1;
            sensorList[_sensorIndex].displayPrecision=0;
            sensorList[_sensorIndex].unitString="";
            break;
        default:        // Error if can't be determined (mainly for debug)
            sensorList[_sensorIndex].coefficient=1;
            sensorList[_sensorIndex].displayPrecision=0;
            sensorList[_sensorIndex].unitString="Err";
    }
}

//****************************
//  Button/Interaction Slots
//****************************

void WirelessModule::backToManageModule()
{   //Button back clicked, stop refresh and return to module grid
    commonRefreshTimer->disconnectTimeout();
    emit backToMainWindow();    //Signal to restart the main window (refresh Values)
}

void WirelessModule::startTimerAgain()
{
    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval(); // insert a small delay before fetching values
}

void WirelessModule::sensorNbButtonClicked()
{
    //Dialog window to change the number of sensors (=numberOfLines*6)

    bool okWindow = false;
    int newValue=QInputDialog::getInt(this,"Number of wireless sensors",
            "Enter the new number of sensors",wireless_Module->nbOfLines*6,0,48,6,&okWindow);    //0 to 48
    if(okWindow)
    {
        newValue=ceil(newValue/6.0);
        //Write the new number of lines :
        serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x0100008D,newValue,3);
        wireless_Module->nbOfLines=newValue;    // this also update the module list
        fillSensorTable();
    }
}

void WirelessModule::valueChanged(int row, int column)
{
    //A cell text changed (serial Nb or name)
    if(column<3 && !fillingSensorTable) //To avoid signal from GUI
    {
        switch(column)
        {
        case 0:
            setSerialNb(row);
            break;
        case 2:
            setSensorName(row);
            break;
        default:
            qDebug()<<"Value changed in unauthorized cell !";
        }
    }
}

void WirelessModule::setSerialNb(int _row)
{
    //To change serial number
    bool ok =true;
    if(sensorItemTable[_row][0]->text()=="") sensorItemTable[_row][0]->setText("0000");
    uint16 newSerialNb=sensorItemTable[_row][0]->text().toInt(&ok,16); //!ok if not integer

    if(ok)
    {
        ok = serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000A0+_row*2+1,newSerialNb & 0x00FF, 3);
        if(ok) ok = serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x010000A0+_row*2,(newSerialNb & 0xFF00)>>8, 3);
        if(!ok)
        {
            QMessageBox::critical(this,"Error SN","Error writing Serial Number");
        }
        else
        {
            fillingSensorTable=true;
            //Update list
            sensorList[_row].serialNb=newSerialNb;
            sensorItemTable[_row][0]->setText(QString::number(newSerialNb,16).rightJustified(4,'0').toUpper());
            //Update combobox list from serialNb
            updateComboBoxList(_row);
            //Erase sensor name
            sensorItemTable[_row][2]->setText("");
            if(newSerialNb!=0)  //TODO : check
            {
                updateComboBoxSelection(_row);
                channelComboBoxList[_row]->showPopup();//Unfold the comboBox (next cell)
            }
            fillingSensorTable=false;
        }
    }
    else QMessageBox::critical(this,"Error SN","Invalid format for Serial Number");

    if(!ok) //updating failed, reset sensor
    {
        fillingSensorTable=true;
        sensorList[_row].serialNb=0;
        sensorItemTable[_row][0]->setText("0000");
        updateComboBoxList(_row);
        updateComboBoxSelection(_row);
        sensorItemTable[_row][2]->setText("");
        fillingSensorTable=false;
    }
}

void WirelessModule::setSensorName(int _row)
{
    //To change a sensor name
    QString newName = sensorItemTable[_row][2]->text();
    if(newName.size()>16)
    {
        newName.truncate(16);
        fillingSensorTable=true;
        sensorItemTable[_row][2]->setText(newName);
        fillingSensorTable=false;
    }
    bool ok=serialCom->setModuleName(wireless_Module->serialNb,0x031FFD00+16*_row,newName);
    if(!ok)
    {
        QMessageBox::critical(this,"Error sensor Name","Error changing the sensor name");
        fillingSensorTable=true;
        sensorItemTable[_row][2]->setText("");
        fillingSensorTable=false;
    }
}

void WirelessModule::comboBoxChanged(int _row)
{
    //Selection of ComboBox of row [_row], have changed, send channel type
    sensorList[_row].channelType2=0;
    if(!fillingSensorTable)
    {
        //Send channel type to module
        if(channelComboBoxList[_row]->currentText()=="Temperature")
        {
           sensorList[_row].channelType=0;
        }
        else if(channelComboBoxList[_row]->currentText()=="Humidity")
        {
            sensorList[_row].channelType=1;
        }
        else if(channelComboBoxList[_row]->currentText()=="Volts")
        {
            sensorList[_row].channelType=2;

        }
        else if(channelComboBoxList[_row]->currentText()=="Amps")
        {
            sensorList[_row].channelType=3;

        }
        else if(channelComboBoxList[_row]->currentText()=="Active Power")
        {
            sensorList[_row].channelType=4;

        }
        else if(channelComboBoxList[_row]->currentText()=="Apparent Power")
        {
            sensorList[_row].channelType=5;
        }
        else if(channelComboBoxList[_row]->currentText()=="Light")
        {
            sensorList[_row].channelType=6;
        }
        else if(channelComboBoxList[_row]->currentText()=="Temp 2")
        {
            sensorList[_row].channelType=7;
        }
        else if(channelComboBoxList[_row]->currentText()=="Temp 1")
        {
            sensorList[_row].channelType=0;
        }
        else if(channelComboBoxList[_row]->currentText()=="Temp")
        {
            sensorList[_row].channelType=7;
           // sensorList[_row].channelType2=3;
        }
        else if(channelComboBoxList[_row]->currentText()=="Pressure")
        {
            sensorList[_row].channelType=9;
        }
        else if(channelComboBoxList[_row]->currentText()=="Light Sensor")
        {
            sensorList[_row].channelType=8;
        }
        else if(channelComboBoxList[_row]->currentText()=="CO2")
        {
            sensorList[_row].channelType=0xA;
        }
        else if(channelComboBoxList[_row]->currentText()=="A1")
        {
            sensorList[_row].channelType=0xB;
        }
        else if(channelComboBoxList[_row]->currentText()=="A2")
        {
            sensorList[_row].channelType=7;
            sensorList[_row].channelType2=4;
        }
        else if(channelComboBoxList[_row]->currentText()=="Pulse Count")
        {
            sensorList[_row].channelType=0x80;
        }
        else if(channelComboBoxList[_row]->currentText()=="ON Time")
        {
            sensorList[_row].channelType=0x81;
        }
        else if(channelComboBoxList[_row]->currentText()=="LQI to")
        {
            sensorList[_row].channelType=0x82;
        }
        else if(channelComboBoxList[_row]->currentText()=="RSSI From")
        {
            sensorList[_row].channelType=0x83;
        }
        else if(channelComboBoxList[_row]->currentText()=="Bat")
        {
            sensorList[_row].channelType=0x84;
        }
        else if(channelComboBoxList[_row]->currentText()=="Active Energy")
        {
            sensorList[_row].channelType=0x90;
        }
        else if(channelComboBoxList[_row]->currentText()=="Appar. Energy")
        {
            sensorList[_row].channelType=0x91;
        }
        else if(channelComboBoxList[_row]->currentText()=="Switch Count")
        {
            sensorList[_row].channelType=0x92;
        }
        else QMessageBox::critical(this,"Undetermined Type","Error : type of channel can't be determined");

        bool ok = serialCom->writeByteAtWithRetry(wireless_Module->serialNb,0x01000040+_row,sensorList[_row].channelType, 3);
        if(!ok) QMessageBox::critical(this,"Writing type error","Error sending type of channel");

        updateStructureFromChannelType(_row);
        //Give focus to name
        sensorTableWidget->editItem(sensorItemTable[_row][2]);
    }
}

//****************************
//  Debug
//****************************
void WirelessModule::displaySensorList()
{
    //Display list of sensor SN and names
    for(uint listIndex=0; listIndex<sensorList.size(); listIndex++)
    {
        qDebug()<<"sensor"<<listIndex<<" : ("<<QString::number(sensorList[listIndex].serialNb,16)
                <<") "<<sensorList[listIndex].sensorName;
    }
}
